/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.compositionengine;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.MeasurementTool;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicediscovery.SDResult;
import fr.inria.icocoa.servicediscovery.ServiceDiscovery;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.automaton.Automaton;
import fr.inria.icocoa.servicemodel.automaton.DoubleHashMap;
import fr.inria.icocoa.servicemodel.automaton.State;
import fr.inria.icocoa.servicemodel.automaton.Symbol;
import fr.inria.icocoa.servicemodel.automaton.qos.QoSAttribute;

/**
 * COCOA_CI.java
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class COCOA_CI {

	private Automaton EnvAutomaton, TaskAutomaton;

	/** The service discovery used for composition */
	private ServiceDiscovery sd;

	public COCOA_CI(ServiceDiscovery sd) {
		this.sd = sd;
	}

	public HashMap<String, List<String>> integrateConversations(
			Automaton EnvAutomaton, Automaton TaskAutomaton)
			throws NoSolutionException {
		this.EnvAutomaton = EnvAutomaton;
		this.TaskAutomaton = TaskAutomaton;

		// DebugPrinter.print("EnvAutomaton = "+EnvAutomaton,DebugPrinter.INFO);
		// DebugPrinter.print("TaskAutomaton =
		// "+TaskAutomaton,DebugPrinter.INFO);

		if (verifier(EnvAutomaton.getInitialState(),
			TaskAutomaton.getInitialState()))
			DebugPrinter.print("There exist at least a composition",
				DebugPrinter.INFO);
		else {
			DebugPrinter.print("There is not any composition",
				DebugPrinter.INFO);
			throw new NoSolutionException("There is not any composition");
		}

		HashMap<String, List<String>> result = extractPaths(
			EnvAutomaton.getEpsilonClosure(), TaskAutomaton.getEpsilonClosure());

		return result;
	}

	Vector<String> cheminT = new Vector<String>();

	DoubleHashMap dhm = new DoubleHashMap();

	List<State> pileEtatsVisites = new Vector<State>();

	// --------------------------------------------------------------------------
	int nbrCall = 0;

	// List PathesList = new ArrayList();

	HashMap alreadyCalculated = new HashMap();

	@SuppressWarnings("unchecked")
	private boolean verifier(State EnvState, State TaskState) {
		nbrCall++;
		// cette methode vrifie si l'etat StatEnv est equivalent l'etat
		// StateTask en utilisant les tables des suivants EnvSuiv et TSuiv et
		// les tats finaux EnvFinaux et TFinaux
		List<Symbol> TaskStateSuiv = TaskAutomaton.getFollowingSymbols().get(
			TaskState); // rcupere les symboles suivants de StateTask
		List<Symbol> EnvStateSuiv = EnvAutomaton.getFollowingSymbols().get(
			EnvState); // rcupere les symboles suivants de StateEnv
		Boolean result = (Boolean) dhm.getValue(alreadyCalculated, EnvState,
			TaskState);
		if (result != null) {
			// DebugPrinter.print("Result != null =>
			// result="+result,DebugPrinter.INFO);
			return (result);
		} else {
			if (TaskState.isFinalState() && EnvState.isFinalState()
					&& TaskStateSuiv == null) {
				// DebugPrinter.print("Verifier=true pour :" +
				// EnvState.getName()+"
				// "+TaskState.getName(),DebugPrinter.INFO);
				return (true);
			}
			// DebugPrinter.print("Verifier=false pour :" + EnvState.getName()+"
			// "+TaskState.getName(),DebugPrinter.INFO);
			if ((TaskState.isFinalState() && !EnvState.isFinalState())
					|| !contains(EnvStateSuiv, TaskStateSuiv)) {
				DebugPrinter.print("TaskState.isFinalState() = "
						+ TaskState.isFinalState(), DebugPrinter.INFO);
				DebugPrinter.print("EnvState.isFinalState() = "
						+ EnvState.isFinalState(), DebugPrinter.INFO);
				DebugPrinter.print("contains(EnvStateSuiv, TaskStateSuiv) = "
						+ contains(EnvStateSuiv, TaskStateSuiv),
					DebugPrinter.INFO);
				DebugPrinter.print("final state=false pour :"
						+ EnvState.getName() + " " + TaskState.getName(),
					DebugPrinter.INFO);
				return (false);
			} else {
				List<Boolean> globalTabresu = new ArrayList<Boolean>();
				// DebugPrinter.print(TaskStateSuiv, TaskState.getName()+"
				// "+"TaskStateSuiv",1);

				// TODO if TaskStateSuiv == null
				for (Iterator<Symbol> e = TaskStateSuiv.iterator(); e.hasNext();) {
					List<Boolean> localTabresu = new ArrayList<Boolean>();
					List<State> enfants = new Vector<State>();
					Symbol CurrentTerminal = e.next();
					State TaskStateToCheck = (State) dhm.getValueAtList(
						TaskAutomaton.getEpsilonClosure(), TaskState,
						CurrentTerminal, 0);
					Symbol EquivalentTerminalInEnv = EnvAutomaton.getSymbolNamed(CurrentTerminal.getName());
					List<State> case_StateEnv_Terminal = (List) dhm.getValue(
						EnvAutomaton.getEpsilonClosure(), EnvState,
						EquivalentTerminalInEnv);
					if (case_StateEnv_Terminal != null) {
						for (Iterator<State> e2 = case_StateEnv_Terminal.iterator(); e2.hasNext();) {
							State EnvStateToCheck = e2.next();
							enfants.add(EnvStateToCheck);
							boolean boo = verifier(EnvStateToCheck,
								TaskStateToCheck);
							dhm.putValue(alreadyCalculated,
								Boolean.valueOf(boo), EnvStateToCheck,
								TaskStateToCheck);
							localTabresu.add(boo);
							if (boo) {
								String stringTransition = TaskState.getName()
										+ ";"
										+ CurrentTerminal.getName()
										+ ";"
										+ TaskStateToCheck.getName()
										+ "__"
										+ EnvState.getName()
										+ ";"
										+ EquivalentTerminalInEnv.getServiceName()
										+ "."
										+ CurrentTerminal.getNameSplited()
										+ ";" + EnvStateToCheck.getName()
										+ "__" + boo;
								// DebugPrinter.print("stringTransition =
								// "+stringTransition, 1);
								cheminT.add(stringTransition);
							}

						}
						globalTabresu.add(localTabresu.contains(true));

					}
				} // finfor
				return !(globalTabresu.contains(false));
			}
		}
	}

	@SuppressWarnings( { "unchecked", "unused" })
	private boolean Oldverifier(State EnvState, State TaskState) {
		// System.out.println("Verifier avec "+EnvState.getName()+ "
		// "+TaskState.getName());
		nbrCall++;
		pileEtatsVisites.add(TaskState);
		// cette methode vrifie si l'etat StatEnv est equivalent l'etat
		// StateTask en utilisant les tables des suivants EnvSuiv et TSuiv et
		// les tats finaux EnvFinaux et TFinaux
		List<Symbol> TaskStateSuiv = TaskAutomaton.getFollowingSymbols().get(
			TaskState); // rcupere les symboles suivants de StateTask
		List<Symbol> EnvStateSuiv = EnvAutomaton.getFollowingSymbols().get(
			EnvState); // rcupere les symboles suivants de StateEnv
		boolean resu = true;
		// Il faut prendre en compte les boucles...marquer les tats deja visits
		// !!
		// if(!contains(EnvStateSuiv,TaskStateSuiv))System.out.println("la merde
		// vient de contains...");
		if (TaskState.isFinalState() && EnvState.isFinalState()
				&& TaskStateSuiv == null) {
			return (true);
		}
		// if(EnvState.isFinalState())System.out.println("TOTO final");
		if ((TaskState.isFinalState() && !EnvState.isFinalState())
				|| !contains(EnvStateSuiv, TaskStateSuiv)) {
			// QoS Disabling: TaskAutomaton.getFormulaTree().reinit();
			return (false);
		} else {
			for (Iterator<Symbol> e = TaskStateSuiv.iterator(); e.hasNext();) {
				List<Boolean> tabresu = new Vector<Boolean>();
				List<State> enfants = new Vector<State>();
				Symbol CurrentTerminal = e.next();
				State TaskStateToCheck = (State) dhm.getValueAtList(
					TaskAutomaton.getEpsilonClosure(), TaskState,
					CurrentTerminal, 0);
				Symbol EquivalentTerminalInEnv = EnvAutomaton.getSymbolNamed(CurrentTerminal.getName());
				List<State> case_StateEnv_Terminal = (List) dhm.getValue(
					EnvAutomaton.getEpsilonClosure(), EnvState,
					EquivalentTerminalInEnv);
				if (case_StateEnv_Terminal != null
						&& !pileEtatsVisites.contains(TaskStateToCheck)) {
					for (Iterator<State> e2 = case_StateEnv_Terminal.iterator(); e2.hasNext();) {
						if (TaskState.equals(TaskAutomaton.getInitialState())) {
							// QoS Disabling:
							// TaskAutomaton.getFormulaTree().reinit();
						}
						State EnvStateToCheck = e2.next();
						/*
						 * on suppose que l'automate de la tache est dterministe !!
						 * en fait, si on veut rsoudre ce probleme il faut faire
						 * l'union des suivants... a aurait march avec un
						 * algorithme iteratif...
						 */
						// ----------------------------------------------------
						/*
						 * QoS Disabling: String transitionKey =
						 * automaton.getTransition(TaskState.getName(),CurrentTerminal.getName(),TaskStateToCheck.getName());
						 * String equivalentTransitionKeyEnv =
						 * automaton.getTransition(EnvState.getName(),CurrentTerminal.getName(),EnvStateToCheck.getName());
						 * 
						 * int index =
						 * ((Transition)TaskAutomaton.getTransitionProperties().get(transitionKey)).getIndexInTheFormulaTree();
						 * HashMap mapOfAttributes =
						 * ((Transition)EnvAutomaton.getTransitionPropertiesClosure().get(equivalentTransitionKeyEnv)).getQoSAttributes();
						 * TaskAutomaton.getFormulaTree().setQoSAttributes(index,mapOfAttributes);
						 * HashMap QoSresult =
						 * TaskAutomaton.getFormulaTree().evaluateAll(TaskAutomaton.getFormulaTree().getRootNode());
						 */
						// ----------------------------------------------------
						// QoS Disabling:
						// if(isRespected(TaskAutomaton.getQoSRequirements(),QoSresult)){
						// QoS Disabling: System.out.println(transitionKey + " "
						// +equivalentTransitionKeyEnv + " " + QoSresult);
						enfants.add(EnvStateToCheck);
						Boolean boo;
						if (Oldverifier(EnvStateToCheck, TaskStateToCheck)) {
							boo = Boolean.TRUE;
						} else {
							boo = Boolean.FALSE;
						}
						String s = TaskState.getName() + ";"
								+ CurrentTerminal.getName() + ";"
								+ TaskStateToCheck.getName() + "__"
								+ EnvState.getName() + ";"
								+ CurrentTerminal.getServiceName() + "."
								+ CurrentTerminal.getName() + ";"
								+ EnvStateToCheck.getName() + "__"
								+ boo.toString();
						cheminT.add(s);
						tabresu.add(boo);
						// QoS Disabling: }else{
						// QoS Disabling:
						// TaskAutomaton.getFormulaTree().reinit();
						// peut etre faut il remettre, ici, les valeurs par
						// defaut dans l'arbre??
						// QoS Disabling: }
					}
					if (TaskState.equals(TaskAutomaton.getInitialState())) {
						if (tabresu.contains(Boolean.TRUE)) {
							resu = true;
						} else {
							resu = false;
						}

					} else {
						if (tabresu.contains(Boolean.FALSE)) {
							resu = false;
						} else {
							resu = true;
						}
					}
				} else {
				}
			} // finfor
		}
		return (resu);
	}

	// ******************************************************************************************************************
	// --------------------------------------------------------------------------
	public boolean contains(List<Symbol> grandV, List<Symbol> petitV) {

		// cette methode vrifie si le vecteur petitV est inclut dans le vecteur
		// grandV
		if (grandV == null || petitV == null)
			return true;
		for (Iterator e = petitV.iterator(); e.hasNext();) {
			Symbol elt = (Symbol) e.next();
			if (!containsASymbolNamed(grandV, elt.getName()))
				return (false);
		}
		return (true);
	}

	private boolean containsASymbolNamed(List<Symbol> grandV, String name) {
		for (Iterator e = grandV.iterator(); e.hasNext();) {
			Symbol symbol = (Symbol) e.next();
			if (symbol.getName().equals(name)) {
				return (true);
			}
		}
		return false;
	}

	// --------------------------------------------------------------------------
	@SuppressWarnings("unchecked")
	private HashMap<String, List<String>> extractPaths(
			HashMap<State, HashMap<Symbol, List<State>>> EnvF,
			HashMap<State, HashMap<Symbol, List<State>>> TF) { // cette methode
		// retire les
		// chemins
		// invalides de
		// la liste des
		// chemins
		// resultats

		HashMap<String, List<String>> resu = new HashMap<String, List<String>>();
		Vector<String> copie = (Vector<String>) cheminT.clone();
		for (Iterator<String> e = cheminT.iterator(); e.hasNext();) {
			String transition = e.next();
			String[] elt = transition.split("__");
			if (elt[2].equals("false")) {
				String[] envTransition = elt[1].split(";");
				String stateToDelete = envTransition[2];
				for (Enumeration e2 = copie.elements(); e2.hasMoreElements();) {
					String transition2 = (String) e2.nextElement();
					String[] elt2 = transition2.split("__");
					if (elt2[1].startsWith(stateToDelete)) {
						copie.remove(transition2);
					}
					copie.remove(transition);
				}
			}
		}
		// pm.printList(copie);
		List<String> taskTransitions = new Vector<String>();
		for (Iterator<String> e3 = copie.iterator(); e3.hasNext();) {
			String ligne = e3.next();
			String[] elt = ligne.split("__");
			taskTransitions.add(elt[0]);
			List<String> v = resu.get(elt[0]);
			if (v == null)
				v = new Vector<String>();
			v.add(elt[1]);
			resu.put(elt[0], v);
		}
		// System.err.println("resu: " + resu);
		// System.err.println("taskT: " + taskTransitions);

		// DebugPrinter.print("",1);
		// PrintMethods.printPaths(resu, taskTransitions);

		return resu;
	}

	@SuppressWarnings("unused")
	private boolean isRespected(HashMap QoSRequirements, HashMap QoSresult) {
		boolean isRespected = true;
		for (Iterator it = QoSRequirements.keySet().iterator(); it.hasNext();) {
			Integer actualAttribute = (Integer) it.next();
			Double reqAttribute = ((Double) QoSRequirements.get(actualAttribute));
			Double provAttribute = (Double) QoSresult.get(actualAttribute);
			if (actualAttribute.equals(QoSAttribute.LATENCY_OBJ)) {
				if (provAttribute.doubleValue() > reqAttribute.doubleValue())
					isRespected = false;
			} else if (actualAttribute.equals(QoSAttribute.AVAILABILITY_OBJ)) {
				if (provAttribute.doubleValue() < reqAttribute.doubleValue())
					isRespected = false;
			}
		}
		return (isRespected);
	}

	/**
	 * This method compose all services to generate a list of path to compose
	 * services to obtain the task
	 * 
	 * @param userTask :
	 *            the parsed UserTask
	 * @return the list of path to compose all selected services and obtain the
	 *         required task
	 * @throws ParsingException
	 * @throws NoSolutionException
	 */
	public Map<String, List<String>> compose(UserTask userTask)
			throws ParsingException, NoSolutionException {

		// DebugPrinter.print("Start");

		// creation of the User task

		// Generation de l'automate de la tache partir du workflow
		long timeTaskAutomaton = System.currentTimeMillis();
		userTask.setAutomaton(new Automaton(userTask));		
		timeTaskAutomaton = System.currentTimeMillis() - timeTaskAutomaton;
		MeasurementTool.displayTime("Task Automation creation : ", timeTaskAutomaton);
		DebugPrinter.print("Task automaton(envAutomaton)\n"+userTask.getAutomaton(),DebugPrinter.INFO);
		/*
		 * matching step: find out all services that provide capabilities that
		 * match the task required capabilities
		 */

		/*
		 * Service discovery -> Reinit stat + calculate time
		 */
		sd.getMatcher().reInitStat();
		long time = System.currentTimeMillis();
		SDResult sd_result = sd.discoverServices(userTask);
		time = System.currentTimeMillis() - time;
		MeasurementTool.displayTime("Service Discovery : ", time);
		DebugPrinter.print("Service Discovery has been done in " + time
				+ " ms\nThe matcher that has been used is "
				+ sd.getMatcher().getClass().getName()
				+ ".\nThe numbers of matching are SM:"
				+ sd.getMatcher().getStat()[0] + " CDM:"
				+ sd.getMatcher().getStat()[1] + " IOM:"
				+ sd.getMatcher().getStat()[2], DebugPrinter.INFO);
		DebugPrinter.print("Service Discovery Result =\n"+sd_result, DebugPrinter.INFO);

		if(sd_result==null)
			throw new NoSolutionException("No Service correspond to your functionalities");
		
		List<Service> resultingServices = sd_result.getListOfService();

		// test the result of matching
		if (resultingServices.isEmpty())
			throw new NoSolutionException(
					"No Service correspond to your functionalities");
		else {
			/* Union of the resulting selected services */
			long timeServiceAutomaton = System.currentTimeMillis();
			Automaton envAutomaton = new Automaton(resultingServices);
			timeServiceAutomaton = System.currentTimeMillis() - timeServiceAutomaton;
			MeasurementTool.displayTime("Services Automation creation : ", timeServiceAutomaton);
			DebugPrinter.print("Service automaton(envAutomaton)\n"+envAutomaton,DebugPrinter.INFO);

			/* Conversation Integration */
			long compositionTime = System.currentTimeMillis();
			Map<String, List<String>> result = integrateConversations(
				envAutomaton, userTask.getAutomaton());
			compositionTime = System.currentTimeMillis() - compositionTime;
			MeasurementTool.displayTime("Service composition : ", compositionTime);
			DebugPrinter.print(result,"result",DebugPrinter.INFO);
			return result;
		}
	}
}
