package fr.inria.icocoa.semrep;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.thoughtworks.xstream.XStream;

import fr.inria.icocoa.semrep.excp.InvocationException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.GroundingProperty;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;

public class CapabilityInvoker {

	private String wsdlUri;
	private String namespace;
	private Map<String, Capability> capabilityMap;

	private XStream xstream = new XStream();

	public CapabilityInvoker(String wsdl) {
		super();
		WsdlResult wr = (WsdlResult) xstream.fromXML(wsdl);
		this.wsdlUri = wr.getWsdlUri();
		this.namespace = wr.getNamespace();
		this.capabilityMap = wr.getCapabilityMap();
	}

	private String translateCapabilityName(String capability)
			throws IllegalArgumentException {
		String result = capability;
		if (capabilityMap != null) {
			String capabilityName = namespace + "#" + capability;
			if (capabilityMap.containsKey(capabilityName)) {
				GroundingProperty gp = capabilityMap.get(capabilityName).getGrounding();
				if (gp != null) {
					result = gp.getOperationName().split("#")[1];
				} else {
					throw new IllegalArgumentException(
							"There is no grounding associated with capability "
									+ capability);
				}
			} else {
				throw new IllegalArgumentException("Unrecognised capability: "
						+ capability);
			}
		}
		return result;
	}

	private String[] translateInputNames(String capability, String[] inputs) {
		if (capabilityMap != null) {
			String capabilityName = namespace + "#" + capability;
			String[] result = new String[inputs.length];
			for (int i = 0; i < inputs.length; ++i) {
				Map<String, Input> inputMap = capabilityMap.get(capabilityName).getInputs();
				if (inputMap != null && inputMap.containsKey(inputs[i])) {
					Input input = inputMap.get(inputs[i]);
					result[i] = input.getGroundingValue();
				} else {
					throw new IllegalArgumentException("Unrecognised input "
							+ inputs[i] + " for capability " + capability);
				}
			}
			return result;
		} else {
			return inputs;
		}
	}

	private Map<String, Object> translateOutputNames(String capability,
			Map<String, Object> outputs) {
		if (capabilityMap != null) {
			String capabilityName = namespace + "#" + capability;
			Map<String, Object> result = new HashMap<String, Object>();
			for (String wsdlOutputName : outputs.keySet()) {
				Collection<Output> capabilityOutputs = capabilityMap.get(
					capabilityName).getOutputs().values();
				boolean found = false;
				for (Output co : capabilityOutputs) {
					if (co.getGroundingValue().split("#")[1].equals(wsdlOutputName)) {
						String capabilityOutputName = co.getLocalName().split(
							"#")[1];
						result.put(capabilityOutputName,
							outputs.get(wsdlOutputName));
						found = true;
						break;
					}
				}
				if (!found) {
					throw new IllegalArgumentException("Unrecognised output "
							+ wsdlOutputName + " for capability " + capability);
				}
			}
			return result;
		} else {
			return outputs;
		}
	}

	@SuppressWarnings("unchecked")
	public Map<String, Object> invoke(String capability, String... args)
			throws InvocationException {
		try {
			String translatedCapabilityName = translateCapabilityName(capability);
			// String[] translatedInputNames = translateInputNames(capability,
			// args);
			DynamicInvoker invoker = new DynamicInvoker(wsdlUri);
			String[] invokeArgs = new String[args.length + 2];
			invokeArgs[0] = wsdlUri;
			invokeArgs[1] = translatedCapabilityName;
			System.arraycopy(args, 0, invokeArgs, 2, args.length);

			Map<String, Object> outputs = invoker.invokeMethod(
				translatedCapabilityName, null, invokeArgs);
			return translateOutputNames(capability, outputs);
		} catch (Exception e) {
			throw new InvocationException(e);
		}
	}

}
