/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep;

import java.util.List;

import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;

/**
 * This interface extends the functionality of SemanticRepository so as to allow
 * more complex algorithm. Thoses extended functionality will be used by the
 * compositionengine and the Service Discovery process.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 27, 2007 10:56:31 AM
 */
public interface SemanticRepositoryForCompositionEngine extends
		SemanticRepository {

	/**
	 * Go through the services contained into the repository to select the one
	 * that contains
	 * 
	 * @param requiredCapa
	 *            the required Capability
	 * @return the match between the required Capability and the found one,
	 *         <i>empty list</i> if no match is found
	 */
	List<CapabilityMatch> getServicesContainingCapability(
			Capability requiredCapa);

	/**
	 * Get a Service object thanks its name
	 * 
	 * @param serviceName
	 *            the name of the required Service
	 * @return the Service object, <i>null</i> if no object with such names
	 *         corresponds.
	 */
	Service getServiceObjectWithLocalName(String serviceName);

	/**
	 * Display all services contained into the repository bu using the method
	 * fr.inria.icocoa.servicemodel.Service#print(java.io.PrintStream) with
	 * PrintStream = System.out
	 */
	void displayAllServices();

	/**
	 * Get the Service object which has the given URI of amigo-s description
	 * file
	 * 
	 * @param owlFileURI
	 *            the URI of the amigo-s description file
	 * @return the required Service, <i>null</i> if no Service with such URI
	 *         exist
	 */
	Service getServiceObjectFromOwlFileURI(String owlFileURI);

	/**
	 * Get the Service object which has the given Base of amigo-s description
	 * file
	 * 
	 * @param baseURL
	 *            the base URL of the amigo-s description file
	 * @return the required Service, <i>null</i> if no Service with such URL
	 *         exist
	 */
	Service getServiceObjectFromBaseURL(String baseURL);

	/**
	 * @return the matcher
	 */
	Matcher getMatcher();

	/**
	 * @param matcher
	 *            the matcher to set
	 */
	void setMatcher(Matcher matcher);
}
