package fr.inria.icocoa.semrep;

import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.semrep.excp.InvocationException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.GroundingProperty;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.IOMatch;
import fr.inria.icocoa.servicemodel.matching.ServiceMatch;

public class ServiceMatchCapabilityInvoker {

	private ServiceMatch match;

	public ServiceMatchCapabilityInvoker(ServiceMatch match)
			throws MalformedURLException {
		super();
		this.match = match;
	}

	public Map<String, Object> invoke(String taskCapabilityName, String... args)
			throws InvocationException {

		List<CapabilityMatch> capabilityMatches = match.getCapabilityMatches();

		String serviceCapabilityName = null;
		List<IOMatch> taskToServiceOutputMatches = null;
		for (CapabilityMatch capaMatch : capabilityMatches) {
			if (capaMatch.getTaskCapabilityLocalName().equals(
				taskCapabilityName)) {
				serviceCapabilityName = capaMatch.getServiceCapabilityLocalName();
				taskToServiceOutputMatches = capaMatch.getOutputsMapping();
				break;
			}
		}
		if (serviceCapabilityName == null) {
			throw new InvocationException("Task capability name \""
					+ taskCapabilityName + "\" not found.");
		}

		try {
			Service service = match.getService();
			Capability capability = service.getProvidedCapability(service.getBase()
					+ "#" + serviceCapabilityName);

			System.out.printf("calling:\n%s\n%s\n%s\n",
				service.getWsdlDescriptionFile(), capability,
				Arrays.toString(args));
			Map<String, Object> map = invokeCorrespondingWsdlOperation(
				service.getWsdlDescriptionFile(), capability, args);

			return convertToTaskOutputNames(taskToServiceOutputMatches,
				capability, map);
		} catch (Exception e) {
			throw new InvocationException(e);
		}
	}

	private Map<String, Object> convertToTaskOutputNames(
			List<IOMatch> taskToServiceOutputMatches, Capability capability,
			Map<String, Object> map) {
		Map<String, Object> result = new HashMap<String, Object>();

		for (Output output : capability.getOutputs().values()) {
			String outputGroundingName = Tools.getSecondPart(output.getGroundingValue());

			if (map.containsKey(outputGroundingName)) {
				String outputServiceName = Tools.getSecondPart(output.getLocalName());
				String outputTaskName = getTaskOutputName(outputServiceName,
					taskToServiceOutputMatches);
				result.put(outputTaskName, map.get(outputGroundingName));
			}
		}

		return result;
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> invokeCorrespondingWsdlOperation(String wsdl,
			Capability capability, String... args) throws Exception {
		GroundingProperty grounding = capability.getGrounding();
		String wsdlOperationName = Tools.getSecondPart(grounding.getOperationName());

		DynamicInvoker invoker = new DynamicInvoker(wsdl);
		String[] invokeArgs = new String[args.length + 2];
		invokeArgs[0] = wsdl;
		invokeArgs[1] = wsdlOperationName;
		System.arraycopy(args, 0, invokeArgs, 2, args.length);

		return invoker.invokeMethod(wsdlOperationName, null, invokeArgs);
	}

	private String getTaskOutputName(String serviceOutputName,
			List<IOMatch> taskToServiceOutputMatches) {
		String result = null;
		for (IOMatch ioMatch : taskToServiceOutputMatches) {
			String matchName = Tools.getSecondPart(ioMatch.getServiceLocalName());
			if (matchName.equals(serviceOutputName)) {
				result = Tools.getSecondPart(ioMatch.getTaskLocalName());
				break;
			}
		}
		return result;
	}

	// private String getWsdlInputName(List<IOMatch> taskToServiceIOMatches,
	// Capability capa, String
	// taskInputName) {
	// String serviceInputName = getServiceInputName(taskInputName,
	// taskToServiceIOMatches);
	// Input serviceInput = capa.getInputs().get(serviceInputName);
	// return serviceInput.getGroundingValue();
	// }
	//
	// private String getServiceInputName(String taskInputName, List<IOMatch>
	// ioMatches) {
	// String result = null;
	// for (IOMatch ioMatch : ioMatches) {
	// if
	// (Tools.getSecondPart(ioMatch.getTaskLocalName()).equals(taskInputName)) {
	// result = ioMatch.getServiceLocalName();
	// break;
	// }
	// }
	// return result;
	// }

}
