/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.test.generator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;

import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.MeasurementTool;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;

/**
 * JavaServiceGenerator.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 14, 2008 10:08:55 AM
 */
public class JavaServiceGenerator {

	private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");

	private SemanticRepositoryForCompositionEngine repository = null;

	/**
	 * generate a number of java services, with them AmigoS Description. Get the
	 * WSDL Description from the server where they are deployed
	 * 
	 * @param numberOfServices
	 *            the number of Service to generate
	 * @param numberOfMethodPerService
	 *            the number of method to generate for each generated services
	 * @param numberOfInputPerMethod
	 *            the number of paramater to generate for each method of each
	 *            generated services
	 * @param wsAddress
	 *            the URL of the Web service that permit to deploy java WS
	 *            remotely
	 * @return a list of all filepath of files that have been generated
	 */
	public List<String> generateServices(int numberOfServices,
			int numberOfMethodPerService, int numberOfInputPerMethod,
			String wsAddress) {

		// create dirs
		File wsdldir = new File(JavaServiceGenerator.userDir
				+ JavaServiceGenerator.slash + "GeneratedServices"
				+ JavaServiceGenerator.slash + "wsdlFiles");
		wsdldir.mkdirs();
		File amigosdir = new File(JavaServiceGenerator.userDir
				+ JavaServiceGenerator.slash + "GeneratedServices"
				+ JavaServiceGenerator.slash + "amigoSFiles");
		amigosdir.mkdirs();
		File javadir = new File(JavaServiceGenerator.userDir
				+ JavaServiceGenerator.slash + "GeneratedServices"
				+ JavaServiceGenerator.slash + "javaFiles");
		javadir.mkdirs();

		// collect all path of created File (filepath)
		LinkedList<String> generatedFilePath = new LinkedList<String>();
		
		DebugPrinter.print("start generation of Ontology...",DebugPrinter.INFO);
		
		
		// get time
		long start = System.currentTimeMillis();

		
				// write in a File
		try {
			//this.writeInAFile(JavaServiceGenerator.slash
			//		+ "GeneratedOntology.owl", ontology);// generated ontology
			File f = this.generateGenericOntology(numberOfServices,
					numberOfMethodPerService, numberOfInputPerMethod,JavaServiceGenerator.userDir
					+ JavaServiceGenerator.slash + "GeneratedServices"+JavaServiceGenerator.slash
					+ "GeneratedOntology.owl");

			generatedFilePath.add(f.getAbsolutePath());
		} catch (IOException e) {
			e.printStackTrace();
		}

		DebugPrinter.print("Ontology generated...",DebugPrinter.INFO);
		
		// generate all files
		for (int i = 0; i < numberOfServices; i++) {
			String javaCode = this.generateJavaCode(i,
					numberOfMethodPerService, numberOfInputPerMethod, false);
			// DebugPrinter.print("------------------------------------------------\n"+
			// javaCode, DebugPrinter.INFO);

			// write in a File
			try {
				File f = this.writeInAFile(JavaServiceGenerator.slash
						+ "javaFiles" + JavaServiceGenerator.slash
						+ "GeneratedService" + i + ".java", javaCode);
				generatedFilePath.add(f.getAbsolutePath());
				DebugPrinter.print("Service "+i+" generated...",DebugPrinter.INFO);
				
			} catch (IOException e) {
				e.printStackTrace();
			}

			// deploy on a server and get WSDL File
			try {
				String wsdl = this.sendToWS(javaCode, wsAddress,
						"GeneratedService" + i);
				File f = this.writeInAFile(JavaServiceGenerator.slash
						+ "wsdlFiles" + JavaServiceGenerator.slash
						+ "GeneratedService" + i + ".wsdl", wsdl);
				generatedFilePath.add(f.getAbsolutePath());
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (ServiceException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}

			// generate AmigoS file
			String[] otherargs = new String[] {
					"http://amigo.gforge.inria.fr/owl/GeneratedService" + i
							+ ".wsdl",
					"GeneratedService" + i,
					"http://amigo.gforge.inria.fr/owl/GeneratedService" + i
							+ ".owl",
					"http://www.inria.fr/myGeneratedOntology.owl#GeneratedService"
							+ i + "Type", "Generated Service nb " + i,
					"GeneratedService" + i };

			String args[] = new String[] {
					generatedFilePath.getLast(),
					JavaServiceGenerator.userDir + JavaServiceGenerator.slash
							+ "GeneratedServices" + JavaServiceGenerator.slash
							+ "amigoSFiles" + JavaServiceGenerator.slash
							+ "GeneratedService" + i + ".owl" };
			try {
				String amigoSfilePath = new AmigoSGenerator().generateAmigoS(
						args, otherargs, i);
				generatedFilePath.add(amigoSfilePath);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (WSDLException e) {
				e.printStackTrace();
			}

		}

		DebugPrinter.print(generatedFilePath, "List of generated files",
				DebugPrinter.INFO);

		// get current time
		long middle = System.currentTimeMillis();

		// register all services
		this.registerGeneratedService(generatedFilePath);

		// get current time
		long end = System.currentTimeMillis();

		MeasurementTool.displayTime("creation time :", middle - start);
		MeasurementTool.displayTime("registration time :", end - middle);

		return generatedFilePath;
	}

	/**
	 * generate a number of java services, with them AmigoS Description. Get the
	 * WSDL Description from the server where they are deployed
	 * 
	 * @param numberOfServices
	 *            the number of Service to generate
	 * @param numberOfMethodPerService
	 *            the number of method to generate for each generated services
	 * @param numberOfInputPerMethod
	 *            the number of paramater to generate for each method of each
	 *            generated services
	 * @param wsAddress
	 *            the URL of the Web service that permit to deploy java WS
	 *            remotely
	 * @return a list of all filepath of files that have been generated
	 */
	public List<String> generateTask(int numberOfServices, String wsAddress) {

		// create dirs
		File taskdir = new File(JavaServiceGenerator.userDir
				+ JavaServiceGenerator.slash + "GeneratedServices"
				+ JavaServiceGenerator.slash + "taskFiles");
		taskdir.mkdirs();

		// collect all path of created File (filepath)
		LinkedList<String> generatedFilePath = new LinkedList<String>();

		String javaCode = this.generateJavaCode(numberOfServices,
				numberOfServices, 0, true);
		// DebugPrinter.print("------------------------------------------------\n"+
		// javaCode, DebugPrinter.INFO);

		// write in a File
		try {
			File f = this.writeInAFile(JavaServiceGenerator.slash + "taskFiles"
					+ JavaServiceGenerator.slash + "GeneratedTask"
					+ numberOfServices + ".java", javaCode);
			generatedFilePath.add(f.getAbsolutePath());
		} catch (IOException e) {
			e.printStackTrace();
		}

		// deploy on a server and get WSDL File
		try {
			String wsdl = this.sendToWS(javaCode, wsAddress, "GeneratedTask"
					+ numberOfServices);
			File f = this.writeInAFile(JavaServiceGenerator.slash + "taskFiles"
					+ JavaServiceGenerator.slash + "GeneratedTask"
					+ numberOfServices + ".wsdl", wsdl);
			generatedFilePath.add(f.getAbsolutePath());
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// get first generated service and generate task
		String[] otherargs = new String[] {
				"http://amigo.gforge.inria.fr/owl/GeneratedService0.wsdl",
				"GeneratedTask",
				"http://amigo.gforge.inria.fr/owl/GeneratedTask.owl",
				"http://www.inria.fr/myGeneratedOntology.owl#GeneratedTask",
				"Generated Task", "GeneratedTask" };

		String args[] = new String[] {
				JavaServiceGenerator.userDir + JavaServiceGenerator.slash
						+ "GeneratedServices" + JavaServiceGenerator.slash
						+ "taskFiles" + JavaServiceGenerator.slash
						+ "GeneratedTask" + numberOfServices + ".wsdl",
				JavaServiceGenerator.userDir + JavaServiceGenerator.slash
						+ "GeneratedServices" + JavaServiceGenerator.slash
						+ "taskFiles" + JavaServiceGenerator.slash
						+ "GeneratedTask" + numberOfServices + ".owl" };

		try {
			String amigoSfilePath = new AmigoSTaskGeneratorWithSequence()
					.generateAmigoS(args, otherargs, numberOfServices);
			generatedFilePath.add(amigoSfilePath);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (WSDLException e) {
			e.printStackTrace();
		}

		DebugPrinter.print(generatedFilePath, "generated file for task",
				DebugPrinter.INFO);

		return generatedFilePath;
	}

	/**
	 * Execute the class
	 * 
	 * @param args
	 *            the starting arguments
	 *            <ul>
	 *            <li><i>args[0]</i> : the number of service to generate</li>
	 *            <li><i>args[1]</i> : the number of method to generate for
	 *            each generated services</li>
	 *            <li><i>args[2]</i> : tthe number of paramater to generate
	 *            for each method of each generated services</li>
	 *            <li><i>args[3]</i> : the WS that permits to deploy services
	 *            remotly</li>
	 *            </ul>
	 */
	public static void main(String[] args) {
		List<String> result = new JavaServiceGenerator().generateServices(
				Integer.parseInt(args[0]), Integer.parseInt(args[1]), Integer
						.parseInt(args[2]), args[3]);
		DebugPrinter.print(result, "List of generatedFiles", DebugPrinter.INFO);
	}

	/**
	 * Generate Java code of a service number i with j methods
	 * 
	 * @param i
	 *            the ID of the class
	 * @param j
	 *            the number of methods that have to be generated
	 * @param l
	 *            the number of input par methods
	 * @param isATask
	 *            <i>true</i> if the generated class correspond to a task,
	 *            <i>false</i> otherwise
	 * @return the java code of this class
	 */
	private String generateJavaCode(int i, int j, int l, boolean isATask) {

		String result = "";
		result += "/**\n ##############################################################################\n # Copyright (C) 2006-2010 INRIA\n #\n # This library is free software; you can redistribute it and/or\n # modify it under the terms of the GNU Lesser General Public\n # License as published by the Free Software Foundation; either\n # version 2 of the License, or (at your option) any later version.\n #\n # This library is distributed in the hope that it will be useful,\n # but WITHOUT ANY WARRANTY; without even the implied warranty of\n # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n # Lesser General Public License for more details.\n #\n # You should have received a copy of the GNU Lesser General Public\n # License along with this library; if not, write to the Free Software\n # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n ##############################################################################\n *\n */\n";

		// declare Class

		result += (isATask) ? "\n\npublic class GeneratedTask" + i + " {\n\n"
				: "\n\npublic class GeneratedService" + i + " {\n\n";

		// class ID static variables
		result += "\tprivate static int classID = " + i + ";\n\n";

		// create method
		for (int k = 0; k < j; k++) {
			result += "\tpublic String method_" + k + "(";

			// inputs
			for (int m = 0; m < l; m++) {
				result += "String args" + m;
				if (m < l - 1)
					result += ",";
			}
			result += "){";

			// create String of params
			result += "\n\t\tString params = \"[\"+";
			if (l > 0) {
				for (int m = 0; m < l; m++) {
					result += "args" + m;
					if (m < l - 1)
						result += "+\",\"+";
				}
			} else
				result += "\"void\"";
			result += "+\"]\";";

			result += "\n\t\tSystem.out.println(\"This is result of method "
					+ k
					+ " for the service with ID = \"+"
					+ ((isATask) ? "GeneratedTask" : "GeneratedService")
					+ i
					+ ".classID+\" my params are = \"+params);\n\t\treturn \"This is result of method "
					+ k + " for the service with ID = \"+"
					+ ((isATask) ? "GeneratedTask" : "GeneratedService") + i
					+ ".classID+\" my params are = \"+params;\n\t}\n\n";
		}

		// end the class
		result += "}";

		return result;
	}

	/**
	 * Write the java code of the Service in a File
	 * 
	 * @param filePath
	 *            the relative filePath of the file inside directory
	 *            "GeneratedServices"
	 * @param javaCode
	 *            the source code of the classe
	 * @return the generated File object.
	 * @throws IOException
	 */
	private File writeInAFile(String filePath, String javaCode)
			throws IOException {

		// create file
		File file = new File(JavaServiceGenerator.userDir
				+ JavaServiceGenerator.slash + "GeneratedServices"
				+ JavaServiceGenerator.slash + filePath);
		file.createNewFile();

		// write inside the file
		BufferedWriter out = new BufferedWriter(new FileWriter(file));
		out.write(javaCode);

		// close streams
		out.flush();
		out.close();

		return file;

	}

	/**
	 * Send the Generated WS and deploy it
	 * 
	 * @param sourceCode
	 *            the source code of the Service
	 * @param serviceName
	 *            the name of the service once deployed
	 * @param wsAdress
	 *            the adress of the Web service that permit to deploy the
	 *            generated WS
	 * @return WSDL File of the deployedService
	 * @throws ServiceException
	 * @throws IOException
	 */
	private String sendToWS(String sourceCode, String wsAddress,
			String serviceName) throws ServiceException, IOException {

		// invoke WS
		org.apache.axis.client.Service service = new org.apache.axis.client.Service();
		org.apache.axis.client.Call call = (org.apache.axis.client.Call) service
				.createCall();
		call.setTargetEndpointAddress(new URL(wsAddress));
		call.setOperationName(new QName(wsAddress, "createService"));
		String serviceAddressComplex = (String) call.invoke(new Object[] {
				sourceCode, serviceName });// is like
		// "http://chico1:9080/axis/GeneratedService9.jws
		// : Service Deployed"

		// get the address of deployed result
		String serviceAddress = serviceAddressComplex.split(" : ")[0].trim();

		// get the URL of the WSDLFile
		URL wsdlAddressURL = new URL(serviceAddress + "?wsdl");

		//DebugPrinter.print(">>>> URL= " + wsdlAddressURL, DebugPrinter.INFO);

		// open the Stream
		BufferedReader in = new BufferedReader(new InputStreamReader(
				wsdlAddressURL.openStream()));

		// read the file content
		String inputLine;
		String wsdlFile = "";
		while ((inputLine = in.readLine()) != null)
			wsdlFile += inputLine;

		// close the Stream
		in.close();

		return wsdlFile;

	}

	/**
	 * Generate the ontology that will be used for the generated services
	 * 
	 * @param numberOfServices
	 *            the number of Service to generate
	 * @param numberOfMethodPerService
	 *            the number of method to generate for each generated services
	 * @param numberOfInputPerMethod
	 *            the number of paramater to generate for each method of each
	 *            generated services
	 * @param filePath the name of the file where the ontology will be written
	 * @return the file created that contains the ontology
	 * @throws IOException 
	 */
	private File generateGenericOntology(int numberOfServices,
			int numberOfMethodPerService, int numberOfInputPerMethod, String filePath) throws IOException {
		
//		 create file
		File file = new File(filePath);
		file.createNewFile();
		
		// open the stream
		BufferedWriter out = new BufferedWriter(new FileWriter(file));
		
		
		out.write("<?xml version=\"1.0\"?>\n<rdf:RDF\n\txmlns:protege=\"http://protege.stanford.edu/plugins/owl/protege#\"\n\txmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n\txmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\"");
		out.write("\n\txmlns:owl=\"http://www.w3.org/2002/07/owl#\"");
		out.write("\n\txmlns=\"http://www.inria.fr/myGeneratedOntology.owl#\"");
		out.write("\n\txml:base=\"http://www.inria.fr/myGeneratedOntology.owl\">\n");
		out.write("\n\t<owl:Ontology rdf:about=\"\">");
		out.write("\n\t\t<owl:imports rdf:resource=\"http://amigo.gforge.inria.fr/owl/Amigo-S.owl\"/>");
		out.write("\n\t</owl:Ontology>");
		
		
		
//		 for each services
		// isolate Service Capability type so as to find them more quickly
		for (int i = 0; i < numberOfServices; i++) {
		/* Generate the type */
		out.write("\n\n\t<owl:Class rdf:ID=\"GeneratedService"	+ i + "Type\"");
		out.write("\n\t\t<rdfs:subClassOf rdf:resource=\"http://amigo.gforge.inria.fr/owl/Amigo-S.owl#Capability\"/>");
		out.write("\n\t</owl:Class>");		
		
		}

		// for each services
		for (int i = 0; i < numberOfServices; i++) {	
		
			
			//DebugPrinter.print("\n********\nfor i\n********\n"+result,DebugPrinter.INFO);
			
			// for each capability
			for (int j = 0; j < numberOfMethodPerService; j++) {
				out.write("\n\n\t<owl:Class rdf:ID=\"GeneratedCapabilitySem_"
						+ i + "_" + j + "\">");
				out.write("\n\t\t<rdfs:subClassOf rdf:resource=\"http://amigo.gforge.inria.fr/owl/Amigo-S.owl#Capability\"/>");
				
				/* Generate some equivalent */
				/*out.write("\n\t\t<owl:equivalentClass>");
				out.write("\n\t\t\t<owl:Class rdf:ID=\"GeneratedCapabilitySemEquivalent1_"
						+ i + "_" + j +"\"/>");
			    out.write("\n\t\t</owl:equivalentClass>");
			    out.write("\n\t\t<owl:equivalentClass>");
				out.write("\n\t\t\t<owl:Class rdf:ID=\"GeneratedCapabilitySemEquivalent2_"
						+ i + "_" + j +"\"/>");
			    out.write("\n\t\t</owl:equivalentClass>");
			    out.write("\n\t\t<owl:equivalentClass>");
				out.write("\n\t\t\t<owl:Class rdf:ID=\"GeneratedCapabilitySemEquivalent3_"
						+ i + "_" + j +"\"/>");
			    out.write("\n\t\t</owl:equivalentClass>");*/
			    
			    
				
				out.write("\n\t</owl:Class>");

				/* Generate some child for each Concept */
				out.write("\n\n\t<owl:Class rdf:ID=\"GeneratedCapabilitySemChild_"
						+ i + "_" + j + "\">");
				out.write("\n\t\t<rdfs:subClassOf rdf:resource=\"#GeneratedCapabilitySem_"
						+ i + "_" + j + "\"/>");
				out.write("\n\t</owl:Class>");

				out.write("\n\n\t<owl:Class rdf:ID=\"GeneratedCapabilitySemChildChild_"
						+ i + "_" + j + "\">");
				out.write("\n\t\t<rdfs:subClassOf rdf:resource=\"#GeneratedCapabilitySemChild_"
						+ i + "_" + j + "\"/>");
				out.write("\n\t</owl:Class>");

				

				out.write("\n\t<owl:Class rdf:ID=\"GeneratedOutputSem_" + i
						+ "_" + j + "\"/>");
				
				//DebugPrinter.print("\n********\nfor j\n********\n"+result,DebugPrinter.INFO);

				// for each input
				for (int k = 0; k < numberOfInputPerMethod; k++) {
					out.write("\n\t<owl:Class rdf:ID=\"GeneratedInputSem_" + i
							+ "_" + j + "_" + k + "\"/>");
					
					/* Generate some equivalent */
					out.write("\n\t\t<owl:equivalentClass>");
					out.write("\n\t\t\t<owl:Class rdf:ID=\"GeneratedInputSemEquivalent1_" + i
							+ "_" + j + "_" + k +"\"/>");
				    out.write("\n\t\t</owl:equivalentClass>");
				    out.write("\n\t\t<owl:equivalentClass>");
					out.write("\n\t\t\t<owl:Class rdf:ID=\"GeneratedInputSemEquivalent2_" + i
							+ "_" + j + "_" + k +"\"/>");
				    out.write("\n\t\t</owl:equivalentClass>");
				    out.write("\n\t\t<owl:equivalentClass>");
					out.write("\n\t\t\t<owl:Class rdf:ID=\"GeneratedInputSemEquivalent3_" + i
							+ "_" + j + "_" + k +"\"/>");
				    out.write("\n\t\t</owl:equivalentClass>");			    
				    
					
					out.write("\n\t</owl:Class>");

					/* Generate some child for each Concept */
					out.write("\n\n\t<owl:Class rdf:ID=\"GeneratedInputSemChild_" + i
							+ "_" + j + "_" + k +"\">");
					out.write("\n\t\t<rdfs:subClassOf rdf:resource=\"#GeneratedInputSem_" + i
							+ "_" + j + "_" + k + "\"/>");
					out.write("\n\t</owl:Class>");

					out.write("\n\n\t<owl:Class rdf:ID=\"GeneratedInputSemChildChild_" + i
							+ "_" + j + "_" + k +"\">");
					out.write("\n\t\t<rdfs:subClassOf rdf:resource=\"#GeneratedInputSemChild_" + i
							+ "_" + j + "_" + k +"\"/>");
					out.write("\n\t</owl:Class>");
					
					//DebugPrinter.print("\n********\nfor k\n********\n"+result,DebugPrinter.INFO);

				}
			}
		}
		out.write("\n</rdf:RDF>");
//close streams
		out.flush();
		out.close();

		return file;
	}

	/**
	 * Add the generated services in the repository
	 * 
	 * @param generatedFilePath
	 *            a list of all filepath of files that have been generated
	 */
	public void registerGeneratedService(List<String> generatedFilePath) {

		this.repository = (this.repository == null) ? SemanticRepositoryFactory
				.getInstanceCE() : this.repository;

		String[] uuids = new String[generatedFilePath.size()];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 1; i < generatedFilePath.size(); i = i + 3) {
			DebugPrinter.print("> Registering " + generatedFilePath.get(i),
					DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(generatedFilePath.get(i + 2),
						generatedFilePath.get(i + 1));
			} catch (Exception e) {
				DebugPrinter.print(e, DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);
	}

}
