package fr.inria.icocoa.test.semreptest;

import java.util.Map;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.semrep.CapabilityInvoker;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.excp.InvocationException;
import fr.inria.icocoa.semrep.excp.SemanticRepositoryException;

public class CapabilityInvokerTest {

	public static void main(String[] args) throws SemanticRepositoryException {

		String userDir = System.getProperty("user.dir");
		String slash = System.getProperty("file.separator");

		String[] semDescs = { "amigo-s" + slash + "ServiceA.owl",
				"amigo-s" + slash + "ServiceB.owl",
				"amigo-s" + slash + "ServiceC.owl",
				"amigo-s" + slash + "ServiceD.owl",
		// "amigo-s" + slash + "PiLowService.owl",
		// "amigo-s" + slash + "MultimediaBrowser.owl",
		// "amigo-s" + slash + "VerySimpleService.owl"
		};
		String[] wsdls = { "wsdl" + slash + "ServiceA.wsdl",
				"wsdl" + slash + "ServiceB.wsdl",
				"wsdl" + slash + "ServiceC.wsdl",
				"wsdl" + slash + "ServiceD.wsdl",
		// "wsdl" + slash + "PiLowService.wsdl",
		// "wsdl" + slash + "MultimediaBrowser.wsdl",
		// "wsdl" + slash + "VerySimpleService.wsdl"
		};
		String[] uuids = new String[semDescs.length];

		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();

		semrep.clearRepository();

		System.out.println("Registering all services...");
		for (int i = 0; i < semDescs.length; ++i) {
			System.out.println("> Registered " + semDescs[i]);
			try {
				uuids[i] = semrep.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				System.err.println("Couldn't add service: " + e);
				e.printStackTrace();
			}
		}
		System.out.println("End of registration.");

		String SDLFilenameA = userDir + slash + "amigo-s" + slash + "TaskA.owl";
		String SDLFilenameB = userDir + slash + "amigo-s" + slash + "TaskB.owl";
		String SDLFilenameC = userDir + slash + "amigo-s" + slash + "TaskC.owl";
		String SDLFilenameD = userDir + slash + "amigo-s" + slash + "TaskD.owl";

		String serviceA = semrep.getServiceWithName("http://amigo.gforge.inria.fr/owl/ServiceA.owl#ServiceA");
		System.out.printf("Using %s for Service A:\n", serviceA);
		String wsdl = semrep.getWsdlForService(serviceA);
		CapabilityInvoker serviceAInvoker = new CapabilityInvoker(wsdl);
		try {
			Double serviceAOp1Result = (Double) serviceAInvoker.invoke(
				"ServiceAOp1Capability").get("ServiceAOp1Output");
			Integer serviceAOp2Result = (Integer) serviceAInvoker.invoke(
				"ServiceAOp2Capability", "3.14", "Aop2").get(
				"ServiceAOp2Output");
			String serviceAOp3Result = (String) serviceAInvoker.invoke(
				"ServiceAOp3Capability", "314").get("ServiceAOp3Output");
			Map<String, Object> serviceAOp4Result = serviceAInvoker.invoke(
				"ServiceAOp4Capability", "Aop4");

			System.out.printf(
				"Service A results (using match %s): op1=%s op2=%s op3=%s op4=%s\n",
				serviceA, serviceAOp1Result, serviceAOp2Result,
				serviceAOp3Result, serviceAOp4Result);
		} catch (InvocationException e) {
			System.err.println(e);
			e.printStackTrace();
		}

		System.exit(0);

		String[] taskAMatches = semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameA));
		// System.out.println("\n\n*** Matches for TaskA: " + taskAMatches);
		// System.out.println("*** Done TaskA matches ***\n\n");
		for (String serviceMatch : taskAMatches) {
			System.out.printf("Using %s for Task A:\n", serviceMatch);
			// String wsdl = semrep.getWsdlForService(serviceMatch);
			// CapabilityInvoker taskAInvoker = new CapabilityInvoker(wsdl);
			// // Task A
			// Double taskAOp1Result;
			// try {
			// taskAOp1Result = (Double) taskAInvoker.invoke(
			// "TaskAOp1Capability").get("TaskAOp1Output");
			// Integer taskAOp2Result = (Integer) taskAInvoker.invoke(
			// "TaskAOp2Capability", "3.14", "Aop2").get(
			// "TaskAOp2Output");
			// String taskAOp3Result = (String) taskAInvoker.invoke(
			// "TaskAOp3Capability", "314").get("TaskAOp3Output");
			// Map<String, Object> taskAOp4Result = taskAInvoker.invoke(
			// "TaskAOp4Capability", "Aop4");
			//
			// System.out
			// .printf(
			// "Task A results (using match %s): op1=%s op2=%s op3=%s op4=%s\n",
			// serviceMatch, taskAOp1Result, taskAOp2Result,
			// taskAOp3Result, taskAOp4Result);
			// } catch (InvocationException e) {
			// System.err.println(e);
			// e.printStackTrace();
			// }
		}

		String[] taskBMatches = semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameB));
		// System.out.println("\n\n*** Matches for TaskB: " + taskBMatches);
		// System.out.println("*** Done TaskB matches ***\n\n");
		for (String serviceMatch : taskBMatches) {
			System.out.printf("Using %s for Task B:\n", serviceMatch);
			// CapabilityInvoker taskBInvoker = new
			// CapabilityInvoker(serviceMatch);
			// // Task B
			// Double taskBOp1Result = (Double)
			// taskBInvoker.invoke("TaskBOp1Capability").get("TaskBOp1Output");
			// Integer taskBOp2Result = (Integer)
			// taskBInvoker.invoke("TaskBOp2Capability",
			// "Aop2").get("TaskBOp2Output");
			//
			// System.out.printf("Task B results (using match %s): op1=%s
			// op2=%s\n", serviceMatch
			// .getService().getLocalName(), taskBOp1Result, taskBOp2Result);
			System.out.printf("WSDL for service %s: %s\n", serviceMatch,
				semrep.getWsdlForService(serviceMatch));
		}

		String[] taskCMatches = semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameC));
		// System.out.println("\n\n*** Matches for TaskC: " + taskCMatches);
		// System.out.println("*** Done TaskC matches ***\n\n");
		for (String serviceMatch : taskCMatches) {
			System.out.printf("Using %s for Task C:\n", serviceMatch);
			// CapabilityInvoker taskCInvoker = new
			// CapabilityInvoker(serviceMatch);
			// // Task C
			// Double taskCOp1Result = (Double)
			// taskCInvoker.invoke("TaskCOp1Capability").get("TaskCOp1Output");
			// Integer taskCOp2Result = (Integer)
			// taskCInvoker.invoke("TaskCOp2Capability", "3.14",
			// "Aop2").get(
			// "TaskCOp2Output");
			// String taskCOp3Result = (String)
			// taskCInvoker.invoke("TaskCOp3Capability",
			// "314").get("TaskCOp3Output");
			//
			// System.out.printf("Task C results (using match %s): op1=%s op2=%s
			// op3=%s\n",
			// serviceMatch
			// .getService().getLocalName(), taskCOp1Result, taskCOp2Result,
			// taskCOp3Result);
		}

		String[] taskDMatches = semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameD));
		// System.out.println("\n\n*** Matches for TaskD: " + taskDMatches);
		// System.out.println("*** Done TaskD matches ***\n\n");
		for (String serviceMatch : taskDMatches) {
			System.out.printf("Using %s for Task D:\n", serviceMatch);
			// CapabilityInvoker taskDInvoker = new
			// CapabilityInvoker(serviceMatch);
			// // Task D
			// Double taskDOp1Result = (Double)
			// taskDInvoker.invoke("TaskDOp1Capability").get("TaskDOp1Output");
			// Integer taskDOp2Result = (Integer)
			// taskDInvoker.invoke("TaskDOp2Capability", "3.14",
			// "Aop2").get(
			// "TaskDOp2Output");
			// String taskDOp3Result = (String)
			// taskDInvoker.invoke("TaskDOp3Capability",
			// "314").get("TaskDOp3Output");
			// Map<String, Object> taskDOp4Result =
			// taskDInvoker.invoke("TaskDOp4Capability",
			// "Aop4");
			//
			// System.out.printf("Task D results (using match %s): op1=%s op2=%s
			// op3=%s op4=%s\n",
			// serviceMatch
			// .getService().getLocalName(), taskDOp1Result, taskDOp2Result,
			// taskDOp3Result,
			// taskDOp4Result);
		}
	}
}
