package fr.inria.icocoa.test.semreptest;

import java.util.Arrays;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.excp.SemanticRepositoryException;

public class SemanticRepositoryTest {

	public static void main(String[] args) throws SemanticRepositoryException {
		performTest();
	}

	private static void performTest() throws SemanticRepositoryException {
		String userDir = System.getProperty("user.dir");
		String slash = System.getProperty("file.separator");

		String[] semDescs = { "amigo-s" + slash + "ServiceA.owl",
				"amigo-s" + slash + "ServiceB.owl",
				"amigo-s" + slash + "ServiceC.owl",
				"amigo-s" + slash + "ServiceD.owl",
				"amigo-s" + slash + "PiLowService.owl",
				"amigo-s" + slash + "MultimediaBrowser.owl",
				"amigo-s" + slash + "VerySimpleService.owl" };
		String[] wsdls = { "wsdl" + slash + "ServiceA.wsdl",
				"wsdl" + slash + "ServiceB.wsdl",
				"wsdl" + slash + "ServiceC.wsdl",
				"wsdl" + slash + "ServiceD.wsdl",
				"wsdl" + slash + "PiLowService.wsdl",
				"wsdl" + slash + "MultimediaBrowser.wsdl",
				"wsdl" + slash + "VerySimpleService.wsdl" };
		String[] uuids = new String[semDescs.length];

		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();

		semrep.clearRepository();

		System.out.println("Registration of all services");
		for (int i = 0; i < semDescs.length; ++i) {
			System.out.println("> Registering " + semDescs[i]);
			try {
				uuids[i] = semrep.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				System.err.println("Couldn't add service: " + e);
				e.printStackTrace();
			}
		}
		System.out.println("End of registration.");

		System.out.println("All services: "
				+ Arrays.toString(semrep.getAllServices()));

		DebugPrinter.print("SemRep = "+semrep,DebugPrinter.INFO);
		
		System.out.println("ServiceA: "
				+ semrep.getServiceWithName("http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA"));
		System.out.println("ServiceB: "
				+ semrep.getServiceWithName("http://www.hitech-projects.com/euprojects/amigo/ServiceB.owl#ServiceB"));
		System.out.println("ServiceC: "
				+ semrep.getServiceWithName("http://www.hitech-projects.com/euprojects/amigo/ServiceC.owl#ServiceC"));
		System.out.println("ServiceD: "
				+ semrep.getServiceWithName("http://www.hitech-projects.com/euprojects/amigo/ServiceD.owl#ServiceD"));
		System.out.println("ServiceX: "
				+ semrep.getServiceWithName("http://www.hitech-projects.com/euprojects/amigo/ServiceD.owl#ServiceX"));
		System.out.println();

		System.out.println("Type1: "
				+ Arrays.toString(semrep.getServicesWithType("http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#Type1")));
		System.out.println("Type2: "
				+ Arrays.toString(semrep.getServicesWithType("http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#Type2")));
		System.out.println("Type3: "
				+ Arrays.toString(semrep.getServicesWithType("http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#Type3")));
		System.out.println("TypeX: "
				+ Arrays.toString(semrep.getServicesWithType("http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#TypeX")));
		System.out.println();

		String SDLFilenameA = userDir + slash + "amigo-s" + slash + "TaskA.owl";
		String SDLFilenameB = userDir + slash + "amigo-s" + slash + "TaskB.owl";
		String SDLFilenameC = userDir + slash + "amigo-s" + slash + "TaskC.owl";
		String SDLFilenameD = userDir + slash + "amigo-s" + slash + "TaskD.owl";

		System.out.println("URI taskA = "
				+ Tools.getURIStringFromFilePath(SDLFilenameA));

		System.out.println("TaskA: "
				+ Arrays.toString(semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameA))));
		System.out.println("TaskB: "
				+ Arrays.toString(semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameB))));
		System.out.println("TaskC: "
				+ Arrays.toString(semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameC))));
		System.out.println("TaskD: "
				+ Arrays.toString(semrep.getServicesForTask(Tools.getURIStringFromFilePath(SDLFilenameD))));

		// ////////QoS

		String[] list = semrep.getServicesWithType("http://www.thalesgroup.com/setha-dyonisos-demo.owl#PiService");
		for (String s : list) {
			System.out.println("QoSPart of " + s + " = "
					+ semrep.getQoSProperties(s));
		}

		// ///////
		for (int i = 0; i < uuids.length; i++) {
			semrep.removeService(uuids[i]);
		}

		System.out.println("All services removed:\n"
				+ Arrays.toString(semrep.getAllServices()));
	}
}
