/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.test.semreptest;

import java.io.IOException;
import java.util.List;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.MeasurementTool;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.compositionengine.CompositionEngine;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicediscovery.SDResult;
import fr.inria.icocoa.servicediscovery.ServiceDiscovery;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser;
import fr.inria.icocoa.test.generator.JavaServiceGenerator;

/**
 * ServiceDiscoveryTest.java
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 18, 2008 4:27:16 PM
 */
public class ServiceCompositionTest {
	
	private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");


	/**
	 * Execute the class
	 * 
	 * @param args
	 *            the starting arguments
	 *            <ul>
	 *            <li><i>args[0]</i> : the number of service to generate</li>
	 *            <li><i>args[1]</i> : the number of method to generate for
	 *            each generated services</li>
	 *            <li><i>args[2]</i> : the number of paramater to generate
	 *            for each method of each generated services</li>
	 *            <li><i>args[3]</i> : the WS that permits to deploy services
	 *            remotly</li>
	 *            <li><i>args[4]</i> : the number of service to sequence for the task</li>
	 *            </ul>
	 * @throws IOException 
	 * @throws ParsingException 
	 */
	public static void main(String[] args) throws ParsingException, IOException {
				
		SemanticRepositoryForCompositionEngine repository = SemanticRepositoryFactory.getInstanceCE();		
		repository.clearRepository();
		
		//generate services
		new JavaServiceGenerator().generateServices(
				Integer.parseInt(args[0]), Integer.parseInt(args[1]), Integer
						.parseInt(args[2]), args[3]);
		//generate task
		List<String> filepath = new JavaServiceGenerator().generateTask(
				Integer.parseInt(args[4]), args[3]);
			
		
		String SDLFilenameA = filepath.get(2);
		
		/* get service from parsing */
		Long time = System.currentTimeMillis();
		UserTask ut = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcessFromString(
				Tools.readFromFile(SDLFilenameA), false));
		time = System.currentTimeMillis() - time;		
		MeasurementTool.displayTime("Task Parsing :", time);
		//ut.print(System.out);

		/* do service composition */
		//ServiceDiscovery sdv = new ServiceDiscovery();
		CompositionEngine ce = new CompositionEngine();
		
		time = System.currentTimeMillis();		
		List<String> deployedServices;
		try {
			deployedServices = ce.deployProcessAsService(ut);
			DebugPrinter.print(deployedServices, "deployedServices",
					DebugPrinter.INFO);
		} catch (Exception e) {
			e.printStackTrace();
		}		
		time = System.currentTimeMillis() - time;
		MeasurementTool.displayTime("Service composition :", time);
		/*DebugPrinter.print("Service composition :"+time+" ms", DebugPrinter.INFO);
		DebugPrinter.print("SDResult = \n"+sdr,DebugPrinter.INFO);
		DebugPrinter.print("Service Discovery has been done in " + time
				+ " ms\nThe matcher that has been used is "
				+ repository.getMatcher().getClass().getName()
				+ ".\nThe numbers of matching are SM:"
				+ repository.getMatcher().getStat()[0] + " CM:"
				+ repository.getMatcher().getStat()[1] + " IOM:"
				+ repository.getMatcher().getStat()[2], DebugPrinter.INFO);*/
	}

}
