package fr.inria.icocoa.test.semreptest.junittests.compositionengine;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.compositionengine.CompositionEngine;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser;

public class CompositionEngineTest {

	private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");

	private static SemanticRepositoryForCompositionEngine repository = null;

	/**
	 * Registration of required services for tests.
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {

		repository = SemanticRepositoryFactory.getInstanceCE();

		// String userDir = System.getProperty("user.dir");
		// String slash = System.getProperty("file.separator");

		String[] semDescs = { "amigo-s" + slash + "ServiceA.owl",
				"amigo-s" + slash + "ServiceB.owl",
				"amigo-s" + slash + "ServiceC.owl",
				"amigo-s" + slash + "ServiceD.owl",
				"amigo-s" + slash + "SimpleService.owl",
				"amigo-s" + slash + "CompositeService.owl",
				"amigo-s" + slash + "GetPiPart1Service.owl",
				"amigo-s" + slash + "GetPiPart2Service.owl",
				"amigo-s" + slash + "PiLowService.owl",
				"amigo-s" + slash + "SebEchoService.owl",
				"amigo-s" + slash + "MultimediaBrowser.owl",
				// "amigo-s" + slash + "MultimediaSelector.owl",
				"amigo-s" + slash + "VerySimpleService.owl" };
		String[] wsdls = { "wsdl" + slash + "ServiceA.wsdl",
				"wsdl" + slash + "ServiceB.wsdl",
				"wsdl" + slash + "ServiceC.wsdl",
				"wsdl" + slash + "ServiceD.wsdl",
				"wsdl" + slash + "SimpleService.wsdl",
				"wsdl" + slash + "CompositeService.wsdl",
				"wsdl" + slash + "GetPiPart1Service.wsdl",
				"wsdl" + slash + "GetPiPart2Service.wsdl",
				"wsdl" + slash + "PiLowService.wsdl",
				"wsdl" + slash + "SebEchoService.wsdl",
				"wsdl" + slash + "MultimediaBrowser.wsdl",
				// "wsdl" + slash + "MultimediaSelector.wsdl",
				"wsdl" + slash + "VerySimpleService.wsdl" };
		String[] uuids = new String[semDescs.length];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
					DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e, DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);
		// repository.displayAllServices();

		// getListOfServices();

	}

	/**
	 * Clean repository after test
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();
		semrep.clearRepository();
	}

	@Test
	public final void testDeployProcessAsService_SimpleServicesWithOnlyAtomic()
			throws Exception {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash + "TaskA.owl";

		CompositionEngine ce = new CompositionEngine();

		UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
		List<String> deployedServices = ce.deployProcessAsService(task);
		DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);

	}

	/**
	 * Test to compose an atomic task with only one atomic service. Input/Output
	 * are only string
	 * 
	 * @throws Exception
	 * 
	 */
	@Test
	public final void testDeployProcessAsService_SebEchoService()
			throws Exception {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "SebEchoTask.owl";

		CompositionEngine ce = new CompositionEngine();

		UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
		List<String> deployedServices = ce.deployProcessAsService(task);
		DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);

	}

	// @Test
	public final void testDeployProcessAsService_RequiredAtomicButProvidedComposite()
			throws Exception {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "Task_MultimediaBrowser_OnlyRequired.owl";

		CompositionEngine ce = new CompositionEngine();

		UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
		List<String> deployedServices = ce.deployProcessAsService(task);
		DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);

	}

	// @Test
	public final void testDeployProcessAsService_RequiredCompositeAndProvidedComposite()
			throws Exception {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "T_MultimediaPlayer_DB.owl";

		CompositionEngine ce = new CompositionEngine();

		UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
		List<String> deployedServices = ce.deployProcessAsService(task);
		DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);

	}

	private static List<String> getListOfServices() {
		org.apache.axis.client.Service service = new org.apache.axis.client.Service();

		try {
			org.apache.axis.client.Call call = (org.apache.axis.client.Call) service
					.createCall();

			String BPEL_SERVER = "chico1:9080";

			String BPEL_DEPLOYEMENT_SERVICE = "/active-bpel/services/BpelEngineAdmin";

			String BPEL_OPERATIONAME = "getProcessList";

			String url = "http://" + BPEL_SERVER + BPEL_DEPLOYEMENT_SERVICE;

			call.setTargetEndpointAddress(new URL(url));

			String namespace = url;

			call.setOperationName(new QName(namespace, BPEL_OPERATIONAME));

			/*
			 * call.invoke(new Object[] {});
			 * 
			 * DebugPrinter.print("reponse message =
			 * \n"+call.getResponseMessage(),DebugPrinter.INFO); List l =
			 * call.getOutputValues();
			 * 
			 * DebugPrinter.print(l,"results", DebugPrinter.INFO);
			 */

			URL httpurl = new URL(
					"http://chico1:9080/BpelAdmin/deployed_processes.jsp");

			BufferedReader in = new BufferedReader(new InputStreamReader(
					httpurl.openStream()));

			String htmlPage = "";

			String line = "";
			String toFind = "deployed_process_detail.jsp";
			Map<String, Integer> collected = new HashMap<String, Integer>();
			while ((line = in.readLine()) != null) {
				htmlPage += line + "\n";
				if (line.contains(toFind)) {
					try {
						// get the end of the string after the string toFind:
						// <td align="left">&nbsp;<a
						// href='deployed_process_detail.jsp?pdid=0&tab=0'>MultimediaPlayer_MediaServer</a></td>
						// ->
						// ?pdid=0&tab=0'>MultimediaPlayer_MediaServer</a></td>
						String endOfLine = line.substring(line
								.lastIndexOf(toFind)
								+ toFind.length());

						// get the name of services
						// ?pdid=0&tab=0'>MultimediaPlayer_MediaServer</a></td>
						// ->MultimediaPlayer_MediaServer
						int start = endOfLine.indexOf(">") + 1;
						int stop = endOfLine.indexOf("<");
						String serviceName = endOfLine.substring(start, stop);

						// get pidnumber
						// ?pdid=0&tab=0'>MultimediaPlayer_MediaServer</a></td>
						// -> 0
						String pdid = "?pdid=";
						int startSubstring = endOfLine.indexOf(pdid)
								+ pdid.length();
						int stopSubstring = endOfLine.indexOf("&");
						String pdidNumber = endOfLine.substring(startSubstring,
								stopSubstring);

						collected
								.put(serviceName, Integer.parseInt(pdidNumber));
					} catch (IndexOutOfBoundsException e) {
						DebugPrinter.print(e, DebugPrinter.INFO);
					} catch (NumberFormatException e) {
						DebugPrinter.print(e, DebugPrinter.INFO);
					}

				}
			}

			// DebugPrinter.print("htmlPage = \n"+htmlPage,DebugPrinter.INFO);
			DebugPrinter.print(collected, "collected", DebugPrinter.INFO);

		} catch (Exception e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
		}
		return null;
	}

}
