package fr.inria.icocoa.test.semreptest.junittests.compositionengine;

import java.util.List;

import org.junit.AfterClass;
import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.compositionengine.CompositionEngine;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser;

public class CompositionEngineTestAlarmClock {

	private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");

	private static SemanticRepositoryForCompositionEngine repository = null;

	/**
	 * Clean repository after test
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();
		semrep.clearRepository();
	}

	@Test
	public final void testDeployProcessAlarmoClockService() throws Exception {

		repository = SemanticRepositoryFactory.getInstanceCE();
		repository.clearRepository();

		String[] semDescs = { "amigo-s" + slash + "ServiceA.owl",
				"amigo-s" + slash + "ServiceB.owl",
				"amigo-s" + slash + "ServiceC.owl",
				"amigo-s" + slash + "ServiceD.owl",
				"amigo-s" + slash + "SimpleService.owl",
				"amigo-s" + slash + "CompositeService.owl",
				"amigo-s" + slash + "CoffeeMachine.owl",
				"amigo-s" + slash + "FmRadio.owl",
				"amigo-s" + slash + "GetPiPart1Service.owl",
				"amigo-s" + slash + "GetPiPart2Service.owl",
				"amigo-s" + slash + "PiLowService.owl",
				"amigo-s" + slash + "SebEchoService.owl",
				"amigo-s" + slash + "MultimediaBrowser.owl",
				"amigo-s" + slash + "VerySimpleService.owl" };
		String[] wsdls = { "wsdl" + slash + "ServiceA.wsdl",
				"wsdl" + slash + "ServiceB.wsdl",
				"wsdl" + slash + "ServiceC.wsdl",
				"wsdl" + slash + "ServiceD.wsdl",
				"wsdl" + slash + "SimpleService.wsdl",
				"wsdl" + slash + "CompositeService.wsdl",
				"wsdl" + slash + "CoffeeMachine.wsdl",
				"wsdl" + slash + "FmRadio.wsdl",
				"wsdl" + slash + "GetPiPart1Service.wsdl",
				"wsdl" + slash + "GetPiPart2Service.wsdl",
				"wsdl" + slash + "PiLowService.wsdl",
				"wsdl" + slash + "SebEchoService.wsdl",
				"wsdl" + slash + "MultimediaBrowser.wsdl",
				"wsdl" + slash + "VerySimpleService.wsdl" };
		String[] uuids = new String[semDescs.length];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
				DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e, DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);

		// repository.getServiceObjectWithLocalName("http://chico1:9080/axis/services/CoffeeMachine.owl#CoffeeMachine").print(System.err);
		// repository.getServiceObjectWithLocalName("http://chico1:9080/axis/services/FmRadio.owl#FmRadio").print(System.err);

		String SDLFilenameA = Tools.getURIStringFromFilePath(userDir + slash
				+ "amigo-s" + slash + "AlarmClockTask.owl");

		CompositionEngine ce = new CompositionEngine();

		
			UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
			List<String> deployedServices = ce.deployProcessAsService(task);
			DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);

		
	}

}
