package fr.inria.icocoa.test.semreptest.junittests.compositionengine;

import java.util.List;

import org.junit.AfterClass;
import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.MeasurementTool;
import fr.inria.icocoa.compositionengine.CompositionEngine;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser;

public class CompositionEngineTestGetPiCapa {

	private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");

	private static SemanticRepositoryForCompositionEngine repository = null;

	/**
	 * Clean repository after test
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();
		semrep.clearRepository();
		
		MeasurementTool.displayAllStoredMeasurement();
	}

	@Test
	public final void testDeployProcessAsService_AtomicRequiredAtomicProvided() throws Exception {

		repository = SemanticRepositoryFactory.getInstanceCE();
		repository.clearRepository();

		String[] semDescs = { "amigo-s" + slash + "SimpleService.owl" };
		String[] wsdls = { "wsdl" + slash + "SimpleService.wsdl" };
		String[] uuids = new String[semDescs.length];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
				DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e, DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);

		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "SimpleTask.owl";

		CompositionEngine ce = new CompositionEngine();

		
			UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
			List<String> deployedServices = ce.deployProcessAsService(task);
			DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);
		
	}

	// @Test
	public final void testDeployProcessAsService_AtomicRequiredCompositeProvided() throws Exception {

		repository = SemanticRepositoryFactory.getInstanceCE();
		repository.clearRepository();

		String[] semDescs = { "amigo-s" + slash
				+ "CompositeServiceWithOnlyProvided.owl" };
		String[] wsdls = { "wsdl" + slash + "CompositeService.wsdl" };
		String[] uuids = new String[semDescs.length];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
				DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e, DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);

		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "SimpleTaskBis.owl";

		CompositionEngine ce = new CompositionEngine();

		
			UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
			List<String> deployedServices = ce.deployProcessAsService(task);
			DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);
		
	}

	@Test
	public final void testDeployProcessAsService_CompositecRequiredCompositeProvided() {

		repository = SemanticRepositoryFactory.getInstanceCE();
		repository.clearRepository();

		String[] semDescs = { "amigo-s" + slash
				+ "CompositeServiceWithOnlyProvided.owl" };
		String[] wsdls = { "wsdl" + slash + "CompositeService.wsdl" };
		String[] uuids = new String[semDescs.length];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
				DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e, DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);

		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "CompositeTask.owl";

		CompositionEngine ce = new CompositionEngine();

		try {
			UserTask task = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcess(
				SDLFilenameA, false));
			List<String> deployedServices = ce.deployProcessAsService(task);
			DebugPrinter.print(deployedServices, "deployedServices",
				DebugPrinter.INFO);
		} catch (Exception e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		}
	}

}
