/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.test.semreptest.junittests.dag;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG;
import fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAGVertex;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.WeakSemanticMatcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;

/**
 * CapabilitiesDAGTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 27, 2008 9:37:49 AM
 */
public class CapabilitiesDAGTest {

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#insertCapability(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testInsertCapability() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		assertEquals("size = 1",1, dag.getSize());
		dag.insertCapability(audioServer);
		assertEquals("size = 2",2, dag.getSize());
		dag.insertCapability(videoServer);
		assertEquals("size = 3",3, dag.getSize());
		dag.insertCapability(mp3Server);
		assertEquals("size = 4",4, dag.getSize());
		dag.insertCapability(oggServer);
		assertEquals("size = 5",5, dag.getSize());
		dag.insertCapability(audioServer2);
		assertEquals("size = 5 (bis)",5, dag.getSize());

		DebugPrinter.print("The DAG\n" + dag.toString(), DebugPrinter.INFO);

		CapabilitiesDAG dag2 = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag2.insertCapability(digitalServer);
		dag2.insertCapability(mp3Server);
		dag2.insertCapability(oggServer);
		dag2.insertCapability(videoServer);
		dag2.insertCapability(audioServer2);
		dag2.insertCapability(audioServer);

		DebugPrinter.print("The DAG2\n" + dag2.toString(), DebugPrinter.INFO);

		CapabilitiesDAG dag3 = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag3.insertCapability(mp3Server);
		dag3.insertCapability(oggServer);
		dag3.insertCapability(videoServer);
		dag3.insertCapability(audioServer2);
		dag3.insertCapability(audioServer);
		dag3.insertCapability(digitalServer);

		DebugPrinter.print("The DAG3\n" + dag3.toString(), DebugPrinter.INFO);

		CapabilitiesDAG dag4 = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag4.insertCapability(audioServer2);
		assertEquals("dag4 - size = 1",1, dag4.getSize());
		dag4.insertCapability(oggServer);
		assertEquals("dag4 - size = 2",2, dag4.getSize());		
		dag4.insertCapability(videoServer);
		assertEquals("dag4 - size = 3",3, dag4.getSize());
		dag4.insertCapability(audioServer);
		assertEquals("dag4 - size = 3 (bis)",3, dag4.getSize());		
		dag4.insertCapability(digitalServer);
		assertEquals("dag4 - size = 4",4, dag4.getSize());		
		dag4.insertCapability(mp3Server);
		assertEquals("dag4 - size = 5",5, dag4.getSize());		
		dag4.insertCapability(fmRadio);
		assertEquals("dag4 - size = 6",6, dag4.getSize());		
		dag4.insertCapability(coffeeMachine);
		assertEquals("dag4 - size = 7",7, dag4.getSize());		
		dag4.insertCapability(alarmClock);
		assertEquals("dag4 - size = 8",8, dag4.getSize());
		

		DebugPrinter.print("The DAG4\n" + dag4.toString(), DebugPrinter.INFO);

		Matcher matcher = new WeakSemanticMatcher();
		CapabilitiesDAG dag5 = new CapabilitiesDAG(matcher);
		CapabilitiesDAGVertex digitalServerNode = new CapabilitiesDAGVertex(
				digitalServer, digitalServer.getSemanticType(), matcher);
		CapabilitiesDAGVertex audioServerNode = new CapabilitiesDAGVertex(
				audioServer, audioServer.getSemanticType(), matcher);
		CapabilitiesDAGVertex videoServerNode = new CapabilitiesDAGVertex(
				videoServer, videoServer.getSemanticType(), matcher);
		CapabilitiesDAGVertex mp3ServerNode = new CapabilitiesDAGVertex(
				mp3Server, mp3Server.getSemanticType(), matcher);
		CapabilitiesDAGVertex oggServerNode = new CapabilitiesDAGVertex(
				oggServer, oggServer.getSemanticType(), matcher);

		audioServerNode.addNewChildNode(mp3ServerNode);
		audioServerNode.addNewChildNode(oggServerNode);
		mp3ServerNode.addNewParentNode(audioServerNode);
		oggServerNode.addNewParentNode(audioServerNode);
		digitalServerNode.addNewChildNode(audioServerNode);
		digitalServerNode.addNewChildNode(videoServerNode);
		audioServerNode.addNewParentNode(digitalServerNode);
		videoServerNode.addNewParentNode(digitalServerNode);
		audioServerNode.addNewEquivalentCapability(audioServer2);
		dag5.addARoot(digitalServerNode);

		assertEquals("same object", dag, dag);
		assertEquals("same 1", dag, dag2);
		assertEquals("same 2", dag, dag3);
		assertEquals("same 3", dag, dag5);
		assertNotSame("different", dag, dag4);

	}
	
	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#getVertex(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testGetVertex() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");
		
		Capability station = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#Station",
				"station");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);
		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);

		

		assertNotNull("get Capa ogg",dag.getVertex(oggServer));
		assertNull("get Capa station",dag.getVertex(station));
		assertEquals("get Capa ogg",dag.getVertex(oggServer).getAssociatedObject() , oggServer);
		assertEquals("get Capa coffee machine",dag.getVertex(coffeeMachine).getAssociatedObject() , coffeeMachine);
		assertEquals("get Capa alarm clock",dag.getVertex(alarmClock).getAssociatedObject() , alarmClock);
		assertEquals("get Capa fm Radio",dag.getVertex(fmRadio).getAssociatedObject() , fmRadio);
		assertEquals("get Capa mp3 ",dag.getVertex(mp3Server).getAssociatedObject() , mp3Server);
		assertEquals("get Capa video",dag.getVertex(videoServer).getAssociatedObject() , videoServer);
		assertEquals("get Capa audio",dag.getVertex(audioServer2).getAssociatedObject() , audioServer);
		assertEquals("get Capa digital",dag.getVertex(digitalServer).getAssociatedObject() , digitalServer);
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#getAllLeaves()}.
	 */
	@Test
	public final void testGetAllLeaves() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);

		DebugPrinter.print("The DAG\n" + dag.toString(), DebugPrinter.INFO);

		List<CapabilitiesDAGVertex> leaves = dag.getAllLeaves();

		assertTrue("number of vertex", leaves.size() == 3);

		/* Create list of capability that are leaves in the current dag */
		List<Capability> toSearch = new ArrayList<Capability>();
		toSearch.add(mp3Server);
		toSearch.add(oggServer);
		toSearch.add(videoServer);

		/* Check the result */
		boolean allFound = checkTheResult(toSearch, leaves);
		assertTrue("all required capabilities found", allFound);

		DebugPrinter.print("The DAG completed\n" + dag.toString(),
				DebugPrinter.INFO);

		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);

		//update the list of leaves
		leaves = dag.getAllLeaves();

		//update of the list of element to find as leaves
		toSearch.add(alarmClock);
		toSearch.add(fmRadio);
		toSearch.add(coffeeMachine);

		//check result
		assertTrue("number of vertex 2", leaves.size() == 6);
		allFound = checkTheResult(toSearch, leaves);
		assertTrue("all required capabilities found 2", allFound);

	}
	
	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#getAllCapabilityMatch(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testGetAllCapabilityMatch() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");
		Capability station = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#Station",
				"station");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);
		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);
		
		//DebugPrinter.print("dag\n"+dag,DebugPrinter.INFO);
		//DebugPrinter.print(dag.getAllCapabilityMatch(digitalServer),"dag.getAllCapabilityMatch(digitalServer)",DebugPrinter.INFO);
		assertEquals("digital server match",5,dag.getAllCapabilityMatch(digitalServer).size());
		//DebugPrinter.print(dag.getAllCapabilityMatch(oggServer),"dag.getAllCapabilityMatch(oggServer)",DebugPrinter.INFO);
		assertEquals("oggServer server match",3,dag.getAllCapabilityMatch(oggServer).size());
		assertEquals("alarmClock server match",1,dag.getAllCapabilityMatch(alarmClock).size());
		assertEquals("station server match",0,dag.getAllCapabilityMatch(station).size());
	}
	
	
	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#getTypeToServiceMap()}.
	 */
	@Test
	public final void testGetTypeToServiceMap() {
		
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");
		
		
		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);
		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);
		
		Map<SemanticType,List<Service>> map = dag.getTypeToServiceMap();
		DebugPrinter.print(map,"dag.getTypeToServiceMap()",DebugPrinter.INFO);
		
		/* Required result */
		List<SemanticType> requiredKeys = new ArrayList<SemanticType>();
		requiredKeys.add(digitalServer.getSemanticType());
		requiredKeys.add(audioServer.getSemanticType());
		requiredKeys.add(videoServer.getSemanticType());
		requiredKeys.add(mp3Server.getSemanticType());
		requiredKeys.add(oggServer.getSemanticType());
		requiredKeys.add(alarmClock.getSemanticType());
		requiredKeys.add(fmRadio.getSemanticType());
		requiredKeys.add(coffeeMachine.getSemanticType());
		
		assertEquals("size of the map",8,map.size());
		assertTrue("chek keys of the map",Tools.compareList(requiredKeys,new ArrayList<SemanticType>(map.keySet())));	
	
	}

	
	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#getRoots()}.
	 */
	@Test
	public final void testGetRoots() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);

		//DebugPrinter.print("The DAG\n" + dag.toString(), DebugPrinter.INFO);

		List<CapabilitiesDAGVertex> roots = new ArrayList<CapabilitiesDAGVertex>(
				dag.getRoots().values());

		assertTrue("number of root", roots.size() == 1);

		/* Create list of capability that are leaves in the current dag */
		List<Capability> toSearch = new ArrayList<Capability>();
		toSearch.add(digitalServer);

		/* Check the result */
		boolean allFound = checkTheResult(toSearch, roots);
		assertTrue("all required capabilities found", allFound);

		/* Complete the DAG */
		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);

		//update the list of leaves
		roots = new ArrayList<CapabilitiesDAGVertex>(dag.getRoots().values());

		//update of the list of element to find as leaves
		toSearch.add(alarmClock);
		toSearch.add(fmRadio);
		toSearch.add(coffeeMachine);

		//check result
		assertTrue("number of vertex 2", roots.size() == 4);
		allFound = checkTheResult(toSearch, roots);
		assertTrue("all required capabilities found 2", allFound);

	}
	
	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#removeAVertex(fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAGVertex)}.
	 */
	@Test
	public final void testRemoveAVertex() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);		
		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);

		//remove simple root
		CapabilitiesDAGVertex fmVertex = dag.getVertex(fmRadio);
		dag.removeAVertex(fmVertex);
		
		DebugPrinter.print("dag after first remove (fmradio) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("1 - number of root = 3",3,dag.getRoots().size());
		assertEquals("1 - number of leaf = 5",5,dag.getAllLeaves().size());
		assertEquals("1 - number of element = 7",7,dag.getSize());
		
		//remove simple leaf
		CapabilitiesDAGVertex mp3Vertex = dag.getVertex(mp3Server);
		dag.removeAVertex(mp3Vertex);
		
		DebugPrinter.print("dag after second remove (mp3Server) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("2 - number of root = 3 ",3,dag.getRoots().size());
		assertEquals("2 - number of leaf = 4",4,dag.getAllLeaves().size());
		assertEquals("2 - number of element = 6",6,dag.getSize());
		
		//remove node
		CapabilitiesDAGVertex audioVertex = dag.getVertex(audioServer);
		dag.removeAVertex(audioVertex);
		
		DebugPrinter.print("dag after third remove (audioServer) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("3 - number of root = 3 ",3,dag.getRoots().size());
		assertEquals("3 - number of leaf = 4",4,dag.getAllLeaves().size());
		assertEquals("3 - number of element = 5",5,dag.getSize());

//		remove complex root
		CapabilitiesDAGVertex digitalVertex = dag.getVertex(digitalServer);
		dag.removeAVertex(digitalVertex);
		
		DebugPrinter.print("dag after third remove (digitalServer) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("4 - number of root = 4 ",4,dag.getRoots().size());
		assertEquals("4 - number of leaf = 4",4,dag.getAllLeaves().size());
		assertEquals("4 - number of element = 4",4,dag.getSize());
		
		
		
	}
	
	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG#removeACapability(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testRemoveACapabality() {
		Capability digitalServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability",
				"DigitalServer");
		Capability audioServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer");
		Capability videoServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability",
				"VideoServer");
		Capability mp3Server = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability",
				"Mp3AudioServer");
		Capability oggServer = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability",
				"OggAudioServer");
		Capability audioServer2 = new Capability(
				"http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability",
				"AudioServer2");

		Capability alarmClock = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#AlarmClock",
				"AlarmClock");
		Capability fmRadio = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#FmRadio",
				"FmRadio");
		Capability coffeeMachine = new Capability(
				"http://amigo.gforge.inria.fr/owl/AlarmClockOntology.owl#CoffeeMachine",
				"CoffeeMachine");

		CapabilitiesDAG dag = new CapabilitiesDAG(new WeakSemanticMatcher());
		dag.insertCapability(digitalServer);
		dag.insertCapability(audioServer);
		dag.insertCapability(videoServer);
		dag.insertCapability(mp3Server);
		dag.insertCapability(oggServer);
		dag.insertCapability(audioServer2);		
		dag.insertCapability(alarmClock);
		dag.insertCapability(fmRadio);
		dag.insertCapability(coffeeMachine);

//		remove equivalent Capability
		dag.removeACapability(audioServer2);
		
		DebugPrinter.print("dag after remove (audioServer2) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("0 - number of root = 4 ",4,dag.getRoots().size());
		assertEquals("0 - number of leaf = 6",6,dag.getAllLeaves().size());
		assertEquals("0 - number of element = 7",7,dag.getSize());
		
		//remove simple root
		dag.removeACapability(fmRadio);
		
		DebugPrinter.print("dag after first remove (fmradio) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("1 - number of root = 3",3,dag.getRoots().size());
		assertEquals("1 - number of leaf = 5",5,dag.getAllLeaves().size());
		assertEquals("1 - number of element = 6",6,dag.getSize());
		
		//remove simple leaf
		dag.removeACapability(mp3Server);
		
		DebugPrinter.print("dag after second remove (mp3Server) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("2 - number of root = 3 ",3,dag.getRoots().size());
		assertEquals("2 - number of leaf = 4",4,dag.getAllLeaves().size());
		assertEquals("2 - number of element = 5",5,dag.getSize());
		
		//remove node <- don't exist
		dag.removeACapability(audioServer);
		
		DebugPrinter.print("dag after third remove (audioServer) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("3 - number of root = 3 ",3,dag.getRoots().size());
		assertEquals("3 - number of leaf = 4",4,dag.getAllLeaves().size());
		assertEquals("3 - number of element = 5",5,dag.getSize());

//		remove complex root
		dag.removeACapability(digitalServer);
		
		DebugPrinter.print("dag after third remove (digitalServer) = \n"+dag, DebugPrinter.INFO);		
		assertEquals("4 - number of root = 4 ",4,dag.getRoots().size());
		assertEquals("4 - number of leaf = 4",4,dag.getAllLeaves().size());
		assertEquals("4 - number of element = 4",4,dag.getSize());
		
		
		
	}

	/**
	 * Check that all capabilities of the list are contained into the list of vertex
	 * @param toFind the list of required capabilities
	 * @param listOfVertex the list of vertex we have to find in
	 * @return <i>true</i> of all required capabilities have been found in the list of vertex, <i>false</i> otherwise
	 */
	private static boolean checkTheResult(List<Capability> toFind,
			List<CapabilitiesDAGVertex> listOfVertex) {
		boolean allFound = true;// indicate if all element have been found.
		for (Capability searchedElement : toFind) {

			boolean elementFound = false;// indicate if current required
			// element has been found
			for (CapabilitiesDAGVertex vertex : listOfVertex) {
				// if semantic type are equals
				if (searchedElement.getSemanticType().equals(
						vertex.getAssociatedObject().getSemanticType())) {
					// the required element has been found, so escape the loop
					elementFound = true;
					break;
				}
			}
			// if we are here and the last element has not been found, it means
			// that this element is missing
			// => allFound is false and we can leave the loop
			if (!elementFound) {
				allFound = false;
				break;
			}

		}
		return allFound;
	}
}
