/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.test.semreptest.junittests.dag;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAGVertex;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.WeakSemanticMatcher;

/**
 * CapabilitiesDAGVertexTest.java
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 29, 2008 10:59:12 AM
 */
public class CapabilitiesDAGVertexTest {

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	/**
	 * Test method for {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAGVertex#hashCode()}.
	 */
	@Test
	public final void testHashCode() {
		Capability digitalServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability", "DigitalServer");
		Capability audioServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability", "AudioServer");
		Capability videoServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability", "VideoServer");
		Capability mp3Server = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability", "Mp3AudioServer");
		Capability oggServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability", "OggAudioServer");
		Capability audioServerBis = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability", "AudioServerBis");
		
		Matcher matcher = new WeakSemanticMatcher();
		
		CapabilitiesDAGVertex digitalServerNode = new CapabilitiesDAGVertex(digitalServer,digitalServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex audioServerNode = new CapabilitiesDAGVertex(audioServer,audioServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex videoServerNode = new CapabilitiesDAGVertex(videoServer,videoServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex mp3ServerNode = new CapabilitiesDAGVertex(mp3Server,mp3Server.getSemanticType(),matcher);
		CapabilitiesDAGVertex oggServerNode = new CapabilitiesDAGVertex(oggServer,oggServer.getSemanticType(),matcher);
		
		
		CapabilitiesDAGVertex digitalServerNode2 = new CapabilitiesDAGVertex(digitalServer,digitalServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex audioServerNode2 = new CapabilitiesDAGVertex(audioServer,audioServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex videoServerNode2 = new CapabilitiesDAGVertex(videoServer,videoServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex mp3ServerNode2 = new CapabilitiesDAGVertex(mp3Server,mp3Server.getSemanticType(),matcher);
		CapabilitiesDAGVertex oggServerNode2 = new CapabilitiesDAGVertex(oggServer,oggServer.getSemanticType(),matcher);
		
		
		assertEquals("same object",digitalServerNode.hashCode(), digitalServerNode.hashCode());
		assertEquals("same content (easy)",digitalServerNode.hashCode(), digitalServerNode2.hashCode());

		audioServerNode.addNewChildNode(mp3ServerNode);
		audioServerNode.addNewChildNode(oggServerNode);
		mp3ServerNode.addNewParentNode(audioServerNode);
		oggServerNode.addNewParentNode(audioServerNode);		
		digitalServerNode.addNewChildNode(audioServerNode);
		digitalServerNode.addNewChildNode(videoServerNode);
		audioServerNode.addNewParentNode(digitalServerNode);
		videoServerNode.addNewParentNode(digitalServerNode);
		audioServerNode.addNewEquivalentCapability(audioServerBis);
		
		
		audioServerNode2.addNewChildNode(mp3ServerNode2);
		audioServerNode2.addNewChildNode(oggServerNode2);
		mp3ServerNode2.addNewParentNode(audioServerNode2);
		oggServerNode2.addNewParentNode(audioServerNode2);		
		digitalServerNode2.addNewChildNode(audioServerNode2);
		digitalServerNode2.addNewChildNode(videoServerNode2);
		audioServerNode2.addNewParentNode(digitalServerNode2);
		videoServerNode2.addNewParentNode(digitalServerNode2);
		audioServerNode2.addNewEquivalentCapability(audioServerBis);
		
		
		assertEquals("same content (less easy)",digitalServerNode.hashCode(), digitalServerNode2.hashCode());
		assertEquals("same content (less easy) 2",videoServerNode.hashCode(), videoServerNode2.hashCode());
		assertEquals("same content (less easy) 3",mp3ServerNode.hashCode(), mp3ServerNode2.hashCode());

		assertNotSame("different",mp3Server.hashCode(),oggServerNode.hashCode());
	}

	/**
	 * Test method for {@link fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAGVertex#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObject() {
		Capability digitalServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#DigitalServerCapability", "DigitalServer");
		Capability audioServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability", "AudioServer");
		Capability videoServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#VideoServerCapability", "VideoServer");
		Capability mp3Server = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#Mp3AudioServerCapability", "Mp3AudioServer");
		Capability oggServer = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#OggAudioServerCapability", "OggAudioServer");
		Capability audioServerBis = new Capability("http://www-rocq.inria.fr/~bianco/MyMultimediaOntology.owl#AudioServerCapability", "AudioServerBis");
		
		Matcher matcher = new WeakSemanticMatcher();
		
		CapabilitiesDAGVertex digitalServerNode = new CapabilitiesDAGVertex(digitalServer,digitalServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex audioServerNode = new CapabilitiesDAGVertex(audioServer,audioServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex videoServerNode = new CapabilitiesDAGVertex(videoServer,videoServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex mp3ServerNode = new CapabilitiesDAGVertex(mp3Server,mp3Server.getSemanticType(),matcher);
		CapabilitiesDAGVertex oggServerNode = new CapabilitiesDAGVertex(oggServer,oggServer.getSemanticType(),matcher);
		//CapabilitiesDAGVertex audioServerBisNode = new CapabilitiesDAGVertex(audioServerBis,audioServerBis.getSemanticType(),matcher);
		
		
		CapabilitiesDAGVertex digitalServerNode2 = new CapabilitiesDAGVertex(digitalServer,digitalServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex audioServerNode2 = new CapabilitiesDAGVertex(audioServer,audioServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex videoServerNode2 = new CapabilitiesDAGVertex(videoServer,videoServer.getSemanticType(),matcher);
		CapabilitiesDAGVertex mp3ServerNode2 = new CapabilitiesDAGVertex(mp3Server,mp3Server.getSemanticType(),matcher);
		CapabilitiesDAGVertex oggServerNode2 = new CapabilitiesDAGVertex(oggServer,oggServer.getSemanticType(),matcher);
		//CapabilitiesDAGVertex audioServerBisNode2 = new CapabilitiesDAGVertex(audioServerBis,audioServerBis.getSemanticType(),matcher);
		
		
		assertEquals("same object",digitalServerNode, digitalServerNode);
		assertEquals("same content (easy)",digitalServerNode, digitalServerNode2);

		audioServerNode.addNewChildNode(mp3ServerNode);
		audioServerNode.addNewChildNode(oggServerNode);
		mp3ServerNode.addNewParentNode(audioServerNode);
		oggServerNode.addNewParentNode(audioServerNode);		
		digitalServerNode.addNewChildNode(audioServerNode);
		digitalServerNode.addNewChildNode(videoServerNode);
		audioServerNode.addNewParentNode(digitalServerNode);
		videoServerNode.addNewParentNode(digitalServerNode);
		audioServerNode.addNewEquivalentCapability(audioServerBis);
		
		
		audioServerNode2.addNewChildNode(mp3ServerNode2);
		audioServerNode2.addNewChildNode(oggServerNode2);
		mp3ServerNode2.addNewParentNode(audioServerNode2);
		oggServerNode2.addNewParentNode(audioServerNode2);		
		digitalServerNode2.addNewChildNode(audioServerNode2);
		digitalServerNode2.addNewChildNode(videoServerNode2);
		audioServerNode2.addNewParentNode(digitalServerNode2);
		videoServerNode2.addNewParentNode(digitalServerNode2);
		audioServerNode2.addNewEquivalentCapability(audioServerBis);
		
		
		assertEquals("same content (less easy)",digitalServerNode, digitalServerNode2);
		assertEquals("same content (less easy) 2",videoServerNode, videoServerNode2);
		assertEquals("same content (less easy) 3",mp3ServerNode, mp3ServerNode2);

		assertNotSame("different",mp3Server,oggServerNode);
	}

}
