/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.test.semreptest.junittests.servicediscovery;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicediscovery.SDResult;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.MatchingType;

/**
 * SDResultTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Oct 3, 2007 10:49:50 AM
 */
public class SDResultTest {

	// private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");

	private static SemanticRepositoryForCompositionEngine repository = null;

	/**
	 * Registration of required services for tests.
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {

		repository = SemanticRepositoryFactory.getInstanceCE();

		String[] semDescs = { "amigo-s" + slash + "ServiceA.owl",
				"amigo-s" + slash + "ServiceB.owl",
				"amigo-s" + slash + "ServiceC.owl",
				"amigo-s" + slash + "ServiceD.owl",
				"amigo-s" + slash + "PiLowService.owl",
				"amigo-s" + slash + "MultimediaBrowser.owl",
				"amigo-s" + slash + "VerySimpleService.owl" };
		String[] wsdls = { "wsdl" + slash + "ServiceA.wsdl",
				"wsdl" + slash + "ServiceB.wsdl",
				"wsdl" + slash + "ServiceC.wsdl",
				"wsdl" + slash + "ServiceD.wsdl",
				"wsdl" + slash + "PiLowService.wsdl",
				"wsdl" + slash + "MultimediaBrowser.wsdl",
				"wsdl" + slash + "VerySimpleService.wsdl" };
		String[] uuids = new String[semDescs.length];

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
				DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e.getStackTrace(), e.getMessage(),
					DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);

	}

	/**
	 * Clean repository after test
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();
		semrep.clearRepository();
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicediscovery.SDResult#SDResult(java.util.List)}.
	 */
	@Test
	public final void testSDResult() {

		// get service object
		Service sa = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceA.owl#ServiceA");
		Service sc = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceC.owl#ServiceC");
		Service sb = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceB.owl#ServiceB");

		// create capabilitymatch
		CapabilityMatch cm1 = new CapabilityMatch(sb.getCapabilityFromTypeName(
			"Op1Semantic").get(0),
				sa.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT);
		CapabilityMatch cm2 = new CapabilityMatch(sb.getCapabilityFromTypeName(
			"Op2Semantic").get(0),
				sa.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT);
		CapabilityMatch cm3 = new CapabilityMatch(sb.getCapabilityFromTypeName(
			"Op2Semantic").get(0),
				sc.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT);

		// create List of CapabilityMatch
		List<CapabilityMatch> lcm = new ArrayList<CapabilityMatch>();
		lcm.add(cm1);
		lcm.add(cm2);
		lcm.add(cm3);
		// DebugPrinter.print(lcm,"lcm",DebugPrinter.INFO);

		// Create SDResult object
		SDResult sdr = new SDResult(lcm);

		/*
		 * check SDResultObject
		 */
		// DebugPrinter.print("sdr="+sdr, DebugPrinter.INFO);
		// check map of selected service
		Map<String, Service> expected = new HashMap<String, Service>();
		expected.put("ServiceA", sa);
		expected.put("ServiceC", sc);
		Map<String, Service> listSelectedService = sdr.getServices();

		assertEquals("size of map", 2, listSelectedService.size());
		assertTrue("same map", Tools.compareMap(listSelectedService, expected));

		// check map of CapabilityMatch
		List<CapabilityMatch> cml1 = new ArrayList<CapabilityMatch>();
		cml1.add(cm1);

		List<CapabilityMatch> cml2 = new ArrayList<CapabilityMatch>();
		cml2.add(cm2);
		cml2.add(cm3);

		assertEquals("check size of map", 2, sdr.getMatches().size());
		assertTrue("check list 1", Tools.compareList(cml1,
			sdr.getMatches().get("ServiceBOp1Capability")));
		assertTrue("check list 2", Tools.compareList(cml2,
			sdr.getMatches().get("ServiceBOp2Capability")));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicediscovery.SDResult#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObject() {

		// get service object
		Service sa = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceA.owl#ServiceA");
		Service sc = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceC.owl#ServiceC");
		Service sb = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceB.owl#ServiceB");
		Service sd = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceD.owl#ServiceD");

		// create capabilitymatch
		CapabilityMatch cm1 = new CapabilityMatch(sb.getCapabilityFromTypeName(
			"Op1Semantic").get(0),
				sa.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT);
		CapabilityMatch cm2 = new CapabilityMatch(sb.getCapabilityFromTypeName(
			"Op2Semantic").get(0),
				sa.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT);
		CapabilityMatch cm3 = new CapabilityMatch(sb.getCapabilityFromTypeName(
			"Op2Semantic").get(0),
				sc.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT);
		CapabilityMatch cm4 = new CapabilityMatch(sd.getCapabilityFromTypeName(
			"Op3Semantic").get(0),
				sa.getCapabilityFromTypeName("Op3Semantic").get(0),
				MatchingType.EXACT);
		CapabilityMatch cm5 = new CapabilityMatch(sc.getCapabilityFromTypeName(
			"Op3Semantic").get(0),
				sc.getCapabilityFromTypeName("Op3Semantic").get(0),
				MatchingType.EXACT);

		// create List 1 of CapabilityMatch
		List<CapabilityMatch> lcm = new ArrayList<CapabilityMatch>();
		lcm.add(cm1);
		lcm.add(cm2);
		lcm.add(cm3);

		// create List 2 of CapabilityMatch
		List<CapabilityMatch> lcm2 = new ArrayList<CapabilityMatch>();
		lcm2.add(cm2);
		lcm2.add(cm3);
		lcm2.add(cm1);

		// create List 3 of CapabilityMatch
		List<CapabilityMatch> lcm3 = new ArrayList<CapabilityMatch>();
		lcm3.add(cm3);
		lcm3.add(cm1);

		// create List 4 of CapabilityMatch
		List<CapabilityMatch> lcm4 = new ArrayList<CapabilityMatch>();
		lcm4.add(cm1);
		lcm4.add(cm2);
		lcm4.add(cm3);
		lcm4.add(cm4);
		lcm4.add(cm5);

		// create List 5 of CapabilityMatch
		List<CapabilityMatch> lcm5 = new ArrayList<CapabilityMatch>();
		lcm5.add(cm4);
		lcm5.add(cm5);

		// Create SDResult objects
		SDResult sdr = new SDResult(lcm);
		SDResult sdr2 = new SDResult(lcm);
		SDResult sdr3 = new SDResult(lcm2);
		SDResult sdr4 = new SDResult(lcm3);
		SDResult sdr5 = new SDResult(lcm4);
		SDResult sdr6 = new SDResult(lcm5);

		/*
		 * Expected true
		 */
		assertTrue("test same object", sdr.equals(sdr));

		assertTrue("test quite same object", sdr.equals(sdr2));
		assertTrue("test quite same object symetric", sdr2.equals(sdr));

		assertTrue("test equivalent object", sdr3.equals(sdr));
		assertTrue("test equivalent object symetric", sdr.equals(sdr3));

		/*
		 * Expected false
		 */
		assertFalse("test partial (less object) equivalent object",
			sdr4.equals(sdr));
		assertFalse("test  partial (less object) equivalent object symetric",
			sdr.equals(sdr4));

		assertFalse("test partial (more object) equivalent object",
			sdr5.equals(sdr));
		assertFalse("test  partial (more object) equivalent object symetric",
			sdr.equals(sdr5));

		assertFalse("test different object", sdr6.equals(sdr));
		assertFalse("test different symetric", sdr.equals(sdr6));

	}

}
