package fr.inria.icocoa.test.semreptest.osgi;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;
import org.ungoverned.gravity.servicebinder.Lifecycle;
import org.ungoverned.gravity.servicebinder.ServiceBinderContext;

import com.francetelecom.amigo.core.AmigoException;
import com.francetelecom.amigo.core.AmigoLdapLookup;
import com.francetelecom.amigo.core.AmigoService;

import fr.inria.icocoa.exception.SDSDCAEException;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.excp.SemanticRepositoryException;

public class SemanticRepositoryTestComponent implements Lifecycle {

	private Logger log;
	private AmigoLdapLookup lookup;
	private BundleContext context;

	public SemanticRepositoryTestComponent(ServiceBinderContext sbc) {
		log = Logger.getLogger(this.getClass().getName());
		context = sbc.getBundleContext();
	}

	public void bindLookupService(AmigoLdapLookup lookup) {
		this.lookup = lookup;
	}

	public void unbindLookupService(AmigoLdapLookup lookup) {
		this.lookup = lookup;
	}

	public void activate() {
		log.info("Activating Semrep Test, lookup = " + lookup);
		if (lookup != null) {
			AmigoService[] services;
			try {
				services = lookup.lookup("urn:amigo", "SemanticServiceRepository", 3);
				
				

				if (services.length == 0) {
					log.error("Unable to find services matching SemanticServiceRepository.");
				} else {
					log.info("Found " + services.length
							+ " services!!");

					String slash = System.getProperty("file.separator");
					String dir = System.getProperty("user.dir");
					String[] semDescs = { dir+ slash + "amigo-s" + slash + "ServiceA.owl",
							 dir + slash + "amigo-s" + slash + "ServiceB.owl",
							 dir+ slash + "amigo-s" + slash + "ServiceC.owl",
							 dir+ slash + "amigo-s" + slash + "ServiceD.owl",
							 dir+ slash + "amigo-s" + slash + "PiLowService.owl",
							 dir+ slash + "owl" + slash + "CoffeeMachine.owl",
							 dir+ slash + "owl" + slash + "FmRadio.owl",
							 //dir+ slash + "owl/CoffeeMachine.owl",
							 //dir+ slash + "owl/FmRadio.owl",
							 dir+ slash + "amigo-s" + slash + "MultimediaBrowser.owl",
							 dir+ slash + "amigo-s" + slash + "VerySimpleService.owl" };
					String[] wsdls = {  dir+ slash + "wsdl2" + slash + "ServiceA.wsdl",
							 dir+ slash + "wsdl2" + slash + "ServiceB.wsdl",
							 dir+ slash + "wsdl2" + slash + "ServiceC.wsdl",
							 dir+ slash + "wsdl2" + slash + "ServiceD.wsdl",
							 dir+ slash + "wsdl2" + slash + "PiLowService.wsdl",
							 dir+ slash + "wsdl" + slash + "CoffeeMachine.wsdl",
							 dir+ slash + "wsdl" + slash + "FmRadio.wsdl",
							// dir+ slash + "wsdl/CoffeeMachine.wsdl",
							 //dir+ slash + "wsdl/FmRadio.wsdl",
							 dir+ slash + "wsdl2" + slash + "MultimediaBrowser.wsdl",
							 dir+ slash + "wsdl2" + slash + "VerySimpleService.wsdl" };

					for (int i = 0; i < services.length; i++) {
						log.info(i + ": " + services[i].getReference());

						SemanticRepository semrep = (SemanticRepository) services[i].getSpecificStub(SemanticRepository.class);
						semrep.clearRepository();

						log.info("Registration of all services");
						String[] uuids = new String[semDescs.length];
						for (int k = 0; k < semDescs.length; ++k) {
							log.info("> Registering " + semDescs[k]);
							try {
								uuids[i] = semrep.addService(semDescs[k],
									wsdls[k]);
							} catch (SDSDCAEException e) {
								log.error("Couldn't add service: " + e);
								//e.printStackTrace();
							}
						}
						log.info("End of registration.");

						String[] allServices = semrep.getAllServices();
						log.info("All services: "
								+ Arrays.toString(allServices));
						
						for(int j=0; j<semrep.getAllServices().length;j++){
							log.info(semrep.getServiceTypeFromServiceName(allServices[j]));					
						}

						String[] list = semrep.getServicesWithType("http://www.thalesgroup.com/setha-dyonisos-demo.owl#PiService");
						for (String s : list) {
							log.info("QoSPart of " + s + " = "
									+ semrep.getQoSProperties(s));
						}
						
						String taskURI = dir+ slash + "tasks" + slash + "AlarmClockTask.owl";
						String[] composedServices;
						try {
							composedServices = semrep.getServicesForTask(taskURI);
							log.info("All composed services: "
									+ Arrays.toString(composedServices));
						} catch (SemanticRepositoryException e) {
							e.printStackTrace();
						}
						
						String taskURI2 = dir+ slash + "tasks/AlarmClockTask.owl";;
						String[] composedServices2;
						try {
							composedServices2 = semrep.getServicesForTask(taskURI2);
							log.info("All composed services: "
									+ Arrays.toString(composedServices2));
						} catch (SemanticRepositoryException e) {
							e.printStackTrace();
						}
						
					}
				}
			} catch (AmigoException e) {
				log.warn("Lookup failed for services of type 'SemanticServiceRepository'.");
			}
		} else {
			log.warn("No lookup service found for Semantic Repository Test bundle.");
		}

		log.info("New NEW NEW");
		new Thread() {
			@Override
			public void run() {
				try {
					context.getBundle().stop();
				} catch (BundleException e) {
					log.warn("Unable to stop bundle.", e);
				}
			}
		}.start();

	}

	public void deactivate() {
		log.info("Stopping the Semantic Repository Test bundle.");
	}

}
