package vtt.vp.bundle;

import java.awt.Color;
import java.util.Properties;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceReference;

import vtt.vp.adapter.interf.VPAmigoAdapter;
import vtt.vp.bundle.impl.VPCallbackServiceImpl;
import vtt.vp.gui.VantagePoint;

public class VantagePointActivator implements BundleActivator{
	
	private boolean vantagePointLoaded = false;	
	private BundleContext bundleContext;	
	private ServiceReference[] refs;

	/**
	 * When starting VantagePointActivator launches VantagePoint and registers VPCallbackInterface
	 * to OSGi service registry.
	 */
	public void start(BundleContext bc) throws Exception {	
				
		System.out.println(bc.getBundle().getHeaders().get(Constants.BUNDLE_NAME) + " starting...");
		bundleContext = bc;
		
		new Thread(new Runnable(){
			public void run(){				
				launchVantagePoint(bundleContext);
				try {
					registerVPCallbackInterface(bundleContext);
				}catch (Exception e) {
					e.printStackTrace();
				}
			}
		}).start();		
		
	}

	/**
	 * When stopping VPBundleActivator closes VantagePoint.
	 */
	public void stop(BundleContext bc) throws Exception {		
		System.out.println(bc.getBundle().getHeaders().get(Constants.BUNDLE_NAME) + " stopping...");
		closeVantagePoint();		
		bundleContext = null;		
	}	
	
	public String registerService(String descURI, String grouURI){
				
		//Search the VPAmigoAdapter OSGi service and call the method
		String serviceURI = null;
		ServiceReference adapterRef = bundleContext.getServiceReference(VPAmigoAdapter.class.getName());
		if(adapterRef != null){
			VPAmigoAdapter adapter = (VPAmigoAdapter)bundleContext.getService(adapterRef);
			serviceURI = adapter.registerService(descURI, grouURI);
		}else{
			VantagePoint.printMessage("Adapter service not found", Color.red);
		}
		
		return serviceURI;
		
	}
	
	public String[] getAllServices(){
		
		//Search the VPAmigoAdapter OSGi service and call the method
		ServiceReference adapterRef = bundleContext.getServiceReference(VPAmigoAdapter.class.getName());
		String services[] = null;
		if(adapterRef != null){
			VPAmigoAdapter adapter = (VPAmigoAdapter)bundleContext.getService(adapterRef);
			services = adapter.getAllServices();
		}else{
			VantagePoint.printMessage("Adapter service not found", Color.red);
		}
		
		return services;
		
	}
	
	public void removeService(String uri){
		
		//Search the VPAmigoAdapter OSGi service and call the method
		ServiceReference adapterRef = bundleContext.getServiceReference(VPAmigoAdapter.class.getName());		
		if(adapterRef != null){
			VPAmigoAdapter adapter = (VPAmigoAdapter)bundleContext.getService(adapterRef);
			adapter.removeService(uri);
		}else{
			VantagePoint.printMessage("Adapter service not found", Color.red);
		}
				
	}
	
	public void clearAllServices(){
		
		//Search the VPAmigoAdapter OSGi service and call the method
		ServiceReference adapterRef = bundleContext.getServiceReference(VPAmigoAdapter.class.getName());		
		if(adapterRef != null){
			VPAmigoAdapter adapter = (VPAmigoAdapter)bundleContext.getService(adapterRef);
			adapter.clearAllServices();
		}else{
			VantagePoint.printMessage("Adapter service not found", Color.red);
		}
				
	}
	
	public String[] getServicesForTask(String taskURI){
		
		//Search the VPAmigoAdapter OSGi service and call the method
		ServiceReference adapterRef = bundleContext.getServiceReference(VPAmigoAdapter.class.getName());	
		String services[] = null;
		if(adapterRef != null){
			VPAmigoAdapter adapter = (VPAmigoAdapter)bundleContext.getService(adapterRef);
			services = adapter.getServicesForTask(taskURI);
		}else{
			VantagePoint.printMessage("Adapter service not found", Color.red);
		}
		
		return services;
		
	}
	
	public String[] getServicesWithType(String serviceType){
		
		//Search the VPAmigoAdapter OSGi service and call the method
		ServiceReference adapterRef = bundleContext.getServiceReference(VPAmigoAdapter.class.getName());	
		String services[] = null;
		if(adapterRef != null){
			VPAmigoAdapter adapter = (VPAmigoAdapter)bundleContext.getService(adapterRef);
			services = adapter.getServicesWithType(serviceType);
		}else{
			VantagePoint.printMessage("Adapter service not found", Color.red);
		}
		
		return services;
		
	}
	
	/////////////////////////////////////////////////////////////////////////////////
	//
	// Private methods
	//
	//

	private void launchVantagePoint(BundleContext bc) {
		System.out.println("VantagePoint launching...");
		try{
			VantagePoint.launch();			
			VantagePoint.vpActivator = this;
			vantagePointLoaded = true;
		}catch(Exception e){
			System.err.println("Could not access vantagepoint.jar");
			e.printStackTrace();
		}
	}

	private void closeVantagePoint() {
		System.out.println("VantagePoint closing...");
		if(vantagePointLoaded)
			VantagePoint.exit();
		vantagePointLoaded = false;	
	}
	
	private void registerVPCallbackInterface(BundleContext bc)throws Exception {		
		VPCallbackService vpInterface = new VPCallbackServiceImpl();		
		Properties props = new Properties();
		props.put("VPCallbackService", "");		
		bc.registerService(VPCallbackService.class.getName(), vpInterface, props);
		System.out.println("VPCallbackService registered");
	}	
}