package vtt.vp.context;

import java.io.InputStream;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import vtt.vp.context.source.VPContextSource;

import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;


/**
 * Context source repository.
 * @author tptoni
 *
 */
public class ContextSourceRepository {

	private ConcurrentHashMap<String, VPContextSource> contextSources = new ConcurrentHashMap<String, VPContextSource>();
	private Vector<ContextSourceRepositoryListener> listeners;
	
	public ContextSourceRepository(){
		listeners = new Vector<ContextSourceRepositoryListener>(10);
	}
	
	/**
	 * Register a context source to repository
	 * @param vpcs
	 */
	public void registerContextSource(VPContextSource vpcs){
		contextSources.put(vpcs.getName(), vpcs);
		for(int i=0; i<listeners.size(); i++)
			listeners.elementAt(i).contextSourceRegistered(vpcs);
	}
	
	/**
	 * Remove a context source from the repository. Thn pit list, ett vain cs itse voi
	 * poistaa itsens.
	 * @param csURI
	 */
	public void unregisterContextSource(String csName){				
		VPContextSource vpcs = contextSources.remove(csName);
		for(int i=0; i<listeners.size(); i++)
			listeners.elementAt(i).contextSourceRemoved(vpcs);
	}
	
	/**
	 * Request a context source reference from the repository.
	 * @param csURI
	 * @return
	 */
	public VPContextSource requestContextSource(String csURI){
		VPContextSource cs = contextSources.get(csURI);
		if(cs != null)
			return new VPContextSourceRef(cs.getURI());
		else
			return null;
	}
	
	public int getContextSourceCount(){
		return contextSources.size();
	}
	
	public void registerListener(ContextSourceRepositoryListener csrl){
		listeners.add(csrl);
	}
	
	public void unregisterListener(ContextSourceRepositoryListener csrl){
		listeners.remove(csrl);
	}
	
	public void printCSR(){
		
		Set s = contextSources.entrySet();
		
		Iterator i = s.iterator();
		
		while(i.hasNext())
			System.out.println(i.next());
		
	}
	
	/**
	 * VPContextSourceRef works a stub that maps method calls to corresponding
	 * context sources. This is to isolate the users of context sources from
	 * the actual classes that might disappear at any given time.
	 * @author tptoni
	 *
	 */
	public class VPContextSourceRef implements VPContextSource{
		
		private String contextSourceURI;
		
		private VPContextSourceRef(String csURI){
			contextSourceURI = csURI;			
		}

		public boolean doAskQuery(String qStr) {
			// TODO Auto-generated method stub
			return false;
		}

		public QuerySolution[] doSelectQuery(String qStr) {
			// TODO Auto-generated method stub
			return null;
		}

		public String doSelectQueryXML(String qStr) {
			// TODO Auto-generated method stub
			return null;
		}

		public String getURI() {
			// TODO Auto-generated method stub
			return null;
		}

		public void modify(Statement[] remStmts, Statement[] addStmts) {
			// TODO Auto-generated method stub
			
		}

		public ContextEventSubscription subscribeConditionalContextEvent(String qStr, VPContextEventListener cel) {
			// TODO Auto-generated method stub
			return null;
		}

		public ContextEventSubscription subscribeContextEvent(VPContextEventListener cel) {
			// TODO Auto-generated method stub
			return null;
		}

		public ContextEventSubscription subscribeContextEvent(SimpleSelector s, VPContextEventListener cel) {
			// TODO Auto-generated method stub
			return null;
		}

		public ContextEventSubscription subscribeContextEvent(String qStr, VPContextEventListener cel) {
			// TODO Auto-generated method stub
			return null;
		}

		public void unsubscribe(ContextEventSubscription ces) {
			// TODO Auto-generated method stub
			
		}

		public String getName() {
			// TODO Auto-generated method stub
			return null;
		}

		public int getType() {
			// TODO Auto-generated method stub
			return 0;
		}

		public String getValueString() {
			// TODO Auto-generated method stub
			return null;
		}

		public InputStream getRDFDataStream() {
			// TODO Auto-generated method stub
			return null;
		}
		
		
		
	}
	
}
