package vtt.vp.gui;

import java.awt.Color;

import vtt.vp.bundle.VantagePointActivator;
import vtt.vp.gui.service.ServiceRepositoryTableModel;

/**
 *  This class is the main class of VantagePoint. It can be executed individually
 *  to test around or started in OSGi to simulate Amigo computing environment.
 *  
 * @author tptoni
 *
 */
public class VantagePoint {
		
	public static VantagePointActivator vpActivator;
	public static final String vpPath = "c:\\workspace\\VP_1.0_Amigo\\";
	
	/**
	 * The graphical user interface
	 */
	private static VantagePointGUI vpGui;
		
	/**
	 * Main starts VantagePoint.
	 * @param args
	 */
	public static void main(String args[]){
		launch();
	}
	
	private VantagePoint(){}
	
	/**
	 * Class constructor creates CSR, SR and VantagePoint GUI.
	 *
	 */
	public static void launch(){
				
		if(vpGui != null) return;
		
		java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
            	vpGui = new VantagePointGUI();	
            }
        });		
		
		ServiceRepositoryTableModel.loadServiceURIMapping();
		
	}	
	
	/**
	 * Prints a message to VantagePoint GUI console.
	 * @param msg	the message
	 * @param c		color of the message
	 */
	public static void printMessage(final String msg, final Color c){
		
		if(vpGui == null) return;		
		
		java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
            	vpGui.printMessage(msg, c);	
            }
        });
		
	}
	
	/**
	 * Prints a message to VantagePoint GUI console.
	 * @param msg	the message
	 */
	public static void printMessage(final String msg){
		
		if(vpGui == null) return;		
		
		java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
            	vpGui.printMessage(msg);	
            }
        });
		
	}
	
	/**
	 * Closes VantagePoint
	 *
	 */
	public static void exit(){
		
		if(vpGui == null) return;
		
		ServiceRepositoryTableModel.saveServiceURIMapping();
		
		java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {            	
            	vpGui.dispose();
            	vpGui = null;
            }
        });
		
	}
	
	public static VantagePointGUI getGUI(){
		return vpGui;
	}
	
}
