package vtt.vp.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryException;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;

import vtt.vp.context.source.VPContextSource;
import vtt.vp.gui.context.ContextSourceRepositoryPanel;
import vtt.vp.gui.item.ItemPanel;
import vtt.vp.gui.service.ServiceRepositoryPanel;
import vtt.vp.library.IconLibrary;
import vtt.vp.view.VPView;
import vtt.vp.view.VPViewManager;

public class VantagePointGUI extends JFrame implements ChangeListener, WindowListener, ActionListener{

	static final long serialVersionUID = 1;		
	private SimpleAttributeSet printAttributes;	
		
	/**
	 * Class constructor creates GUI components and displays the window.
	 * @param csr
	 * @param sr
	 */
	public VantagePointGUI(){	
		
		try {
	        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());	        
	    } catch (Exception e) {
	    	System.err.println("Could not set system look and feel");
	    	e.printStackTrace();
	    }
				
	    IconLibrary.getIconLibraryObject().loadLibrary(VantagePoint.vpPath);
	    
		initComponents();
		this.setVisible(true);
		mainSplitPane.setDividerLocation(0.8);
		viewConsoleSplitPane.setDividerLocation(0.8);
		
		printAttributes = new SimpleAttributeSet();
		StyleConstants.setForeground(printAttributes, Color.black);
	    StyleConstants.setFontFamily(printAttributes, "Helvetica");
	    StyleConstants.setFontSize(printAttributes, 12);
			    
	}
	
	/**
	 * Prints a message to the console.
	 * @param msg	the message
	 * @param c		color of the message
	 */
	public void printMessage(String msg, Color c){		
		
		StyleConstants.setForeground(printAttributes, c);    
	    int documentLenght = consoleTextPane.getDocument().getLength();
	    
	    try{	    	
	    	consoleTextPane.getDocument().insertString(documentLenght, "\n" + msg, printAttributes);
 	   	}
 	   	catch(BadLocationException e) {
 			System.err.println("BadLocationException in console");
 	   	}
		
	}
	
	/**
	 * Prints a message to the console.
	 * @param msg	the message
	 */
	public void printMessage(String msg){
				 
		StyleConstants.setForeground(printAttributes, Color.black);		
		int documentLenght = consoleTextPane.getDocument().getLength();
	    
	    try{	    	
	    	consoleTextPane.getDocument().insertString(documentLenght, "\n" + msg, printAttributes);
 	   	}
 	   	catch(BadLocationException e) {
 			System.err.println("BadLocationException in console");
 	   	}
		
	}
	
	/**
	 * Shows a context source in the view part of the gui.
	 * @param vpcs
	 */
	public void viewContextSource(VPContextSource vpcs){
		viewManager.viewContextSource(vpcs);
	}
	
	private void exitMenuItemActionPerformed(ActionEvent evt) { 
		System.out.println("X pressed");
        VantagePoint.exit();
    }
	
	private void clearConsoleButtonActionPerformed(ActionEvent evt) {                                             
        consoleTextPane.setText("");
    }
	
	private void executeQuery(){
		
		VPView view = viewManager.getCurrentView();
		
		if(view == null){
			printMessage("Cannot execute query. No context source selected.", Color.red);
			return;
		}
		
		VPContextSource vpcs = view.getContextSource();
		String qStr = (String)queryComboBox.getSelectedItem();
		
		Query query;
		try{
			query = QueryFactory.create(qStr);
		}catch(QueryException qe){
			printMessage("Malformed query statement: " + qe.getMessage(), Color.red);
			return;
		}
		
		QuerySolution solution[];
		boolean answer;
		
		if(query.isSelectType()){
			solution = vpcs.doSelectQuery(qStr);
			printMessage("Query Solution:");
			for(int i=0; i<solution.length; i++)
				printMessage(solution[i].toString());
				
		}else if(query.isAskType()){
			answer = vpcs.doAskQuery(qStr);
			printMessage("Query Solution: " + answer);
		}		
		
	}
	
	public void stateChanged(ChangeEvent ce) {
		
		JTabbedPane pane = (JTabbedPane)ce.getSource();
		System.out.println(pane.getSelectedIndex());
		
		if(pane.getSelectedIndex() == 2)
			itemPanel.refreshLibraryList();
		else if(pane.getSelectedIndex() == 1)
			serviceRepositoryPanel.refreshServiceList();
		
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		
		if(ae.getActionCommand().equals("EXECUTE_QUERY")){			
			executeQuery();			
		}
		
	}
	
	/**
	 * This method is called by the VPCallbackService when anyone registers a service
	 *
	 */
	public void refreshServiceList(){		
			serviceRepositoryPanel.refreshServiceList();
	}	
	
	public ServiceRepositoryPanel getServicePanel(){
		return serviceRepositoryPanel;
	}
	
	/** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */                             
    private void initComponents() {
        mainToolBar = new javax.swing.JToolBar();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        mainSplitPane = new javax.swing.JSplitPane();
        viewConsoleSplitPane = new javax.swing.JSplitPane();
        consolePanel = new javax.swing.JPanel();
        consoleToolBar = new javax.swing.JToolBar();
        clearConsoleButton = new javax.swing.JButton();        
        consoleScrollPane = new javax.swing.JScrollPane();
        consoleTextPane = new javax.swing.JTextPane();
        viewManager = new VPViewManager();
        sideTabbedPane = new javax.swing.JTabbedPane();
        contextSourceRepositoryPanel = new ContextSourceRepositoryPanel(this);
        itemPanel = new ItemPanel(viewManager);
        serviceRepositoryPanel = new ServiceRepositoryPanel();        
        queryPanel = new javax.swing.JPanel();
        queryLabel = new javax.swing.JLabel();
        queryComboBox = new javax.swing.JComboBox();
        executeQueryButton = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();

        addWindowListener(this);
        setTitle("VantagePoint 1.0");        
        mainToolBar.setBorder(null);
        mainToolBar.setFloatable(false);
        jButton1.setText("jButton1");
        mainToolBar.add(jButton1);        

        jButton2.setText("jButton2");
        mainToolBar.add(jButton2);

        jButton3.setText("jButton3");
        mainToolBar.add(jButton3);

        getContentPane().add(mainToolBar, java.awt.BorderLayout.NORTH);
       
        mainSplitPane.setDividerSize(8);
        mainSplitPane.setResizeWeight(0.9);
        mainSplitPane.setOneTouchExpandable(true); 
        
        viewConsoleSplitPane.setDividerSize(8);
        viewConsoleSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        viewConsoleSplitPane.setResizeWeight(0.9);
        viewConsoleSplitPane.setOneTouchExpandable(true);
        
        consoleToolBar.setFloatable(false);
        clearConsoleButton.setText("Clear Console");
        clearConsoleButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt) {
				clearConsoleButtonActionPerformed(evt);				
			}        	
        });
        consoleToolBar.add(clearConsoleButton);       
        
        consoleTextPane.setEditable(false);
        
        consoleScrollPane.setViewportView(consoleTextPane);

        javax.swing.GroupLayout consolePanelLayout = new javax.swing.GroupLayout(consolePanel);
        consolePanel.setLayout(consolePanelLayout);
        consolePanelLayout.setHorizontalGroup(
            consolePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(consoleToolBar, javax.swing.GroupLayout.DEFAULT_SIZE, 592, Short.MAX_VALUE)
            .addComponent(consoleScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 592, Short.MAX_VALUE)
        );
        consolePanelLayout.setVerticalGroup(
            consolePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(consolePanelLayout.createSequentialGroup()
                .addComponent(consoleToolBar, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(consoleScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 140, Short.MAX_VALUE))
        );
        viewConsoleSplitPane.setBottomComponent(consolePanel);
        
        viewConsoleSplitPane.setLeftComponent(viewManager);

        mainSplitPane.setLeftComponent(viewConsoleSplitPane);
        
        sideTabbedPane.addTab("Context sources", contextSourceRepositoryPanel);                 
        sideTabbedPane.addTab("Services", serviceRepositoryPanel);        
        sideTabbedPane.addTab("Items", itemPanel);        
        sideTabbedPane.addChangeListener(this);

        mainSplitPane.setRightComponent(sideTabbedPane);

        getContentPane().add(mainSplitPane, java.awt.BorderLayout.CENTER);

        queryLabel.setText("Query:");

        executeQueryButton.setText("Execute");
        executeQueryButton.setActionCommand("EXECUTE_QUERY");
        executeQueryButton.addActionListener(this);
                
		queryComboBox.setEditable(true);				
		queryComboBox.addActionListener(new ActionListener(){
						
			//This listener adds only new Strings to combo box menu
			//and executes query
			
			public void actionPerformed(ActionEvent ae){
									
				if(ae.getActionCommand().equals("comboBoxEdited")){
					
					JComboBox cb = (JComboBox)ae.getSource();
					String newItem = (String)cb.getSelectedItem();					
					String item;
					boolean itemFound = false;
					int itemCount = cb.getItemCount();
					
					for(int i=0; i<itemCount; i++){
						item = (String)cb.getItemAt(i);
						if(item.equals(newItem))
							itemFound = true;
					}
					
					if(!itemFound)
						cb.addItem(newItem);			
					
					executeQuery();
					
				}
				
			}
			
		});

        javax.swing.GroupLayout queryPanelLayout = new javax.swing.GroupLayout(queryPanel);
        queryPanel.setLayout(queryPanelLayout);
        queryPanelLayout.setHorizontalGroup(
            queryPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(queryPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(queryLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(queryComboBox, javax.swing.GroupLayout.DEFAULT_SIZE, 564, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(executeQueryButton)
                .addContainerGap())
        );
        queryPanelLayout.setVerticalGroup(
            queryPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(queryPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(queryPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(queryLabel)
                    .addComponent(executeQueryButton)
                    .addComponent(queryComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        getContentPane().add(queryPanel, java.awt.BorderLayout.SOUTH);

        fileMenu.setText("File");
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        setJMenuBar(menuBar);

        pack();
    } 
    
    // Variables declaration - do not modify                     
    private javax.swing.JTextPane consoleTextPane;
    private javax.swing.JPanel consolePanel;
    private javax.swing.JScrollPane consoleScrollPane;
    private javax.swing.JToolBar consoleToolBar;
    private ContextSourceRepositoryPanel contextSourceRepositoryPanel;
    private ItemPanel itemPanel;
    private javax.swing.JButton executeQueryButton;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;    
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton clearConsoleButton;   
    private javax.swing.JSplitPane mainSplitPane;
    private javax.swing.JToolBar mainToolBar;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JLabel queryLabel;
    private javax.swing.JPanel queryPanel;
    private javax.swing.JComboBox queryComboBox;
    private ServiceRepositoryPanel serviceRepositoryPanel;
    private javax.swing.JTabbedPane sideTabbedPane;
    private javax.swing.JSplitPane viewConsoleSplitPane;
    private VPViewManager viewManager;
    // End of variables declaration

	public void windowActivated(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void windowClosed(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void windowClosing(WindowEvent arg0) {
		
		System.out.println("X pressed");		
		VantagePoint.exit();
		
	}

	public void windowDeactivated(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void windowDeiconified(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void windowIconified(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void windowOpened(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}	
		
}
