package vtt.vp.gui.item;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.ResourceUtils;
import com.hp.hpl.jena.vocabulary.RDF;

import vtt.vp.library.IconSet;
import vtt.vp.gui.VantagePoint;
import vtt.vp.gui.service.ServiceRepositoryTableModel;
import vtt.vp.library.IconLibrary;
import vtt.vp.utils.VPUtils;
import vtt.vp.view.EditView;
import vtt.vp.view.VPViewManager;

public class ItemPanel extends JPanel implements ListSelectionListener, ActionListener, MouseMotionListener, MouseListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private IconSet iconSet;
	private int iconDirection = 0; 
	private OntModel libraryModel;
	private VPViewManager viewManager;
	private EditView editView;
	
	public ItemPanel(VPViewManager vpvm){
		super();
		viewManager = vpvm;
		initComponents();
	}
	
	public void refreshLibraryList(){
		
		//Update item library list
		File librariesFolder = new File(VantagePoint.vpPath + "items");
		File fileList[] = librariesFolder.listFiles(new FileFilter(){
			public boolean accept(File f) {
				if(!f.isDirectory() || f.getName().endsWith(".owl"))	return true;
				else return false;
			}
		});
		
		DefaultListModel librariesModel = (DefaultListModel)itemLibraryList.getModel();
		librariesModel.removeAllElements();     		
				
		librariesModel.addElement("IconLibrary.owl");
		for(int i=0; i<fileList.length; i++)
			librariesModel.addElement(fileList[i].getName());		
		
	}
	
	public void valueChanged(ListSelectionEvent lse) {    	
    	
    	if(lse.getValueIsAdjusting()) return;    	
	    	  	   	
	    JList list = (JList)lse.getSource();	    	
    	
	    if(list.getName().equals("ITEM_LIBRARY_LIST")){	    	
	    	itemLibrarySelected();	    	
	    }else if(list.getName().equals("ITEMS_LIST")){	    	
	    	itemSelected();	    	
	    }    	
		
	}
	
	public void actionPerformed(ActionEvent ae) {    	
    	
        if(ae.getActionCommand().equals("RIGHT")){        	
        	if(icon.getIcon() != null){
        		iconDirection--;
        		if(iconDirection == -1)
        			iconDirection = 3;        		
        		updateIcon();
        	}        	
        }else if(ae.getActionCommand().equals("LEFT")){        	
        	if(icon.getIcon() != null){
        		iconDirection = (iconDirection + 1) % 4;        		
        		updateIcon();
        	}        	
        }
        
    }
	
	private void itemLibrarySelected(){    	
		
    	DefaultListModel itemsModel = (DefaultListModel)itemsList.getModel();
    	itemsModel.removeAllElements();     		
		
    	DefaultTableModel dtm = (DefaultTableModel)propertiesTable.getModel();
		for(int i=dtm.getRowCount()-1; i>-1; i--)
			dtm.removeRow(i);
    	
    	if(itemLibraryList.getSelectedIndex() == -1) return;
    	
		String itemLibraryName = (String)itemLibraryList.getSelectedValue();
		
		//If default library is selected we use the IconLibrary as item library
		if(itemLibraryName.equals("IconLibrary.owl")){
			
			IconLibrary iLib = IconLibrary.getIconLibraryObject();
			Object keys[] = iLib.getIconNames().toArray();
			for(int i=0; i<keys.length; i++)
				itemsModel.addElement((String)keys[i]);
			
		}else{
		
			//Load item library and list items  	    	
	    	OntModelSpec noReasoning = new OntModelSpec(OntModelSpec.OWL_MEM);
	    	OntDocumentManager dMan = new OntDocumentManager("file:" + VantagePoint.vpPath + "configuration" + File.separator + "ont-policy.rdf");
	    	noReasoning.setDocumentManager(dMan);
	    	libraryModel = ModelFactory.createOntologyModel(noReasoning);
	    	try{
	    		libraryModel.read("file:" +
	    				VantagePoint.vpPath +
	    				File.separator +
	    				"items" +
	    				File.separator +
	    				itemLibraryName);
	    	}catch(Exception e){
	    		System.out.println("Could not load item library: " + itemLibraryName);
	    	}
				    	
	    	StmtIterator itemIter = libraryModel.listStatements(
	    			null,
	    			RDF.type,
	    			ResourceFactory.createResource(VPUtils.VANTAGEPOINT_NS+"Item"));
	    	Statement stmt = null;
	    	while(itemIter.hasNext()){
	    		stmt = itemIter.nextStatement();
	    		itemsModel.addElement(((Resource)stmt.getSubject()).getLocalName());
	    	}	    			    			    	
	    	
		}
		
    }
	
	private void itemSelected(){
    	
    	if(itemsList.getSelectedIndex() == -1){    		
    		icon.setIcon(null);
    		DefaultTableModel propTableModel = (DefaultTableModel)propertiesTable.getModel();
    		for(int i=propTableModel.getRowCount()-1; i>-1; i--)
    			propTableModel.removeRow(i);
    		return;    		
    	}
    	
    	if(itemLibraryList.getSelectedValue().equals("IconLibrary.owl")){
    		
    		IconLibrary iLib = IconLibrary.getIconLibraryObject();
    		IconSet is = iLib.getIconSet((String)itemsList.getSelectedValue()); 
    		iconSet = is;
    		updateIcon();    		
    		
    	}else{
    		
    		//Clear properties table
    		DefaultTableModel propTableModel = (DefaultTableModel)propertiesTable.getModel();
    		for(int i=propTableModel.getRowCount()-1; i>-1; i--)
    			propTableModel.removeRow(i);
    		
    		//Get selected individual from model
    		String localName = (String)itemsList.getSelectedValue();    		
    		Individual selectedItem = libraryModel.getIndividual(libraryModel.getNsPrefixURI("") + localName); 
    		
    		//Get icon set
    		Statement isStmt = selectedItem.getProperty(VPUtils.ICONSET_PROP);
    		String iconSetName = "";
    		if(isStmt != null){    		
    			Individual isInd = (Individual)isStmt.getObject().as(Individual.class);
    			iconSetName = isInd.getLocalName();    			
    		}    		
    		IconLibrary iLib = IconLibrary.getIconLibraryObject();
    		if(iconSetName.startsWith("_")) iconSetName = iconSetName.substring(1, iconSetName.length());
    		IconSet is = iLib.getIconSet(iconSetName);     		
    		iconSet = is;
    		updateIcon(); 
    		
    		//Get property values    		
    		Model subModel = ResourceUtils.reachableClosure(selectedItem);
    		StmtIterator stmtIter = subModel.listStatements();
    		Statement stmt;
    		String subject;
    		String predicate;
    		String object;
    		while(stmtIter.hasNext()){
    			stmt = stmtIter.nextStatement();    			
    			subject = stmt.getSubject().getLocalName().toString();
    			predicate = stmt.getPredicate().getLocalName().toString();
    			if(stmt.getObject().isLiteral())
    				object = stmt.getObject().toString();
    			else
    				object = ((Resource)stmt.getObject()).getLocalName();    			
    			propTableModel.addRow(new String[]{subject, predicate, object});
    		}    		
    		
    	}
    	
    }
	
	private void updateIcon(){
    	
    	if(iconSet == null)
    		return;
    	
    	ImageIcon selectedIcon = null;
    	
    	if(iconDirection == 0)
    		selectedIcon = new ImageIcon(iconSet.getImageSouth());
    	else if(iconDirection == 1)
    		selectedIcon = new ImageIcon(iconSet.getImageWest());
    	else if(iconDirection == 2)
    		selectedIcon = new ImageIcon(iconSet.getImageNorth());
    	else if(iconDirection == 3)
    		selectedIcon = new ImageIcon(iconSet.getImageEast());
    	
    	ImageIcon thumbnail = VPUtils.getIconThumbnail(selectedIcon,150,150);
		icon.setIcon(thumbnail);
		
    }
	
	public void mouseDragged(MouseEvent me) {

		if(viewManager.getCurrentViewType() != VPViewManager.EDITVIEW_TYPE) return;		
		Point p = SwingUtilities.convertPoint(icon, me.getX(), me.getY(), editView);				
		editView.itemFrame.setLocation(p.x/20*20, p.y/20*20);
		
	}

	public void mouseMoved(MouseEvent arg0) {}
	
	public void mouseClicked(MouseEvent arg0) {}
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	
	public void mousePressed(MouseEvent arg0) {
		
		if(viewManager.getCurrentViewType() != VPViewManager.EDITVIEW_TYPE) return;
		
		try{
			
			Image itemCursorImage = ImageIO.read(new File(VantagePoint.vpPath + "graphic\\images\\dragItemCursor.png"));			
			Cursor areaCursor = Toolkit.getDefaultToolkit().createCustomCursor(
					itemCursorImage,
					new Point(0,0),
					"dragItemCursor");
			this.getRootPane().setCursor(areaCursor);
			
		}catch(IOException ioe){
			System.err.println("Drag item cursor image not found.");
		}
		
		editView = (EditView)viewManager.getCurrentView();
		String localName = (String)itemsList.getSelectedValue();		
		String itemLibraryName = (String)itemLibraryList.getSelectedValue();
		String iconSetName;
		
		if(itemLibraryName.equals("IconLibrary.owl")){
			
			iconSetName = (String)itemsList.getSelectedValue();
			
		}else{
			
			Individual itemInd = libraryModel.getIndividual(libraryModel.getNsPrefixURI("") + localName);		
			Statement iconSetStmt = itemInd.getProperty(VPUtils.ICONSET_PROP);
			Resource iconSetRes = (Resource)iconSetStmt.getObject();
			iconSetName = iconSetRes.getLocalName();
			
		}		
		
		if(iconSetName.startsWith("_")) iconSetName = iconSetName.substring(1, iconSetName.length());
		IconSet is = IconLibrary.getIconLibraryObject().getIconSet(iconSetName);
		editView.addItemModeOn(is.getGridWidth()*20, is.getGridHeight()*20);
				
	}

	public void mouseReleased(MouseEvent arg0) {
		
		if(viewManager.getCurrentViewType() != VPViewManager.EDITVIEW_TYPE) return;
		
		this.getRootPane().setCursor(Cursor.getDefaultCursor());
		String localName = (String)itemsList.getSelectedValue();
		String itemLibraryName = (String)itemLibraryList.getSelectedValue();		
		
		if(itemLibraryName.equals("IconLibrary.owl")){
			
			Model m = ModelFactory.createDefaultModel();
			Resource r = m.createResource("http://vantagepoint.fi/owl/Temp.owl#" + localName);
			r.addProperty(RDF.type, VPUtils.ITEM_CLASS);
			r.addProperty(VPUtils.ICONSET_PROP, ResourceFactory.createResource(VPUtils.ICONLIBRARY_NS + localName));
			
			editView.addItem(r, m);
			
			return;
			
		}else{
			
			Individual itemInd = libraryModel.getIndividual(libraryModel.getNsPrefixURI("") + localName);
			Model data = ResourceUtils.reachableClosure(itemInd);
			boolean itemAddedSuccessfully = editView.addItem(itemInd, data);				
		
			if(!itemAddedSuccessfully) return;
			
			if(VantagePoint.vpActivator == null){
				VantagePoint.printMessage("Unable to register services, VantagePoint activator not found", Color.red);
				return;
			}
			
			Resource itemRes = data.getResource(itemInd.getURI());
			StmtIterator serviceIter = itemRes.listProperties(VPUtils.PROVIDESSERVICE_PROP);
			
			Statement stmt;
			Resource serviceRes;
			String serviceDescriptionFileName;
			String serviceGroundingFileName;
			while(serviceIter.hasNext()){
				
				stmt = serviceIter.nextStatement();
				serviceRes = (Resource)stmt.getObject();
				stmt = serviceRes.getProperty(VPUtils.SERVICEDESCRIPTIONURI_PROP);
				serviceDescriptionFileName = stmt.getString();
				stmt = serviceRes.getProperty(VPUtils.SERVICEGROUNDINGURI_PROP);
				serviceGroundingFileName = stmt.getString();
				try{
					String serviceURI = VantagePoint.vpActivator.registerService(
							VantagePoint.vpPath + "services" + File.separator + serviceDescriptionFileName,
							VantagePoint.vpPath + "services" + File.separator + serviceGroundingFileName);
					ServiceRepositoryTableModel.addServiceURIMapping(serviceRes.getURI(), serviceURI);				
				}catch(Exception e){
					VantagePoint.printMessage("Error registering services", Color.red);
					VantagePoint.printMessage(e.getMessage());
				}
			}
		}
			
	}	
	
	@SuppressWarnings("serial")
	private void initComponents() {
        jPanel1 = new javax.swing.JPanel();
        leftButton = new javax.swing.JButton();
        rightButton = new javax.swing.JButton();
        icon = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        itemsList = new javax.swing.JList();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        itemLibraryList = new javax.swing.JList();
        propertiesTable = new javax.swing.JTable();
        jScrollPane3 = new javax.swing.JScrollPane();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Item"));
        leftButton.setText("Left");
        leftButton.setActionCommand("LEFT");
        leftButton.addActionListener(this);

        rightButton.setText("Right"); 
        rightButton.setActionCommand("RIGHT");
        rightButton.addActionListener(this);
        
        propertiesTable.setModel(new DefaultTableModel());
        DefaultTableModel tableModel = (DefaultTableModel)propertiesTable.getModel();
        tableModel.setColumnCount(3);
        tableModel.setColumnIdentifiers(new String[]{"Subject", "Predicate", "Object"});
        
        icon.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);        
        icon.addMouseMotionListener(this);
        icon.addMouseListener(this);
        
        jScrollPane3.setViewportView(propertiesTable);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 377, Short.MAX_VALUE)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(leftButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(rightButton))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(icon, javax.swing.GroupLayout.DEFAULT_SIZE, 357, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addComponent(icon, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(leftButton)
                    .addComponent(rightButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 104, Short.MAX_VALUE))
        );
        add(jPanel1);

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Items"));
        //jPanel2.setAlignmentX(0.0F);
        itemsList.setModel(new DefaultListModel());
        itemsList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        itemsList.addListSelectionListener(this);
        itemsList.setName("ITEMS_LIST");
        jScrollPane2.setViewportView(itemsList);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 366, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
        );
        add(jPanel2);

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Item Libraries"));
        jPanel3.setAlignmentY(0.5F);
        itemLibraryList.setModel(new DefaultListModel());
        itemLibraryList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        itemLibraryList.setName("ITEM_LIBRARY_LIST");
        itemLibraryList.addListSelectionListener(this);
        jScrollPane1.setViewportView(itemLibraryList);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 366, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 50, Short.MAX_VALUE)
        );
        add(jPanel3);

    }// </editor-fold>
    
    
    // Variables declaration - do not modify
	private javax.swing.JButton leftButton;
    private javax.swing.JButton rightButton;
    private javax.swing.JLabel icon;
    private javax.swing.JList itemLibraryList;
    private javax.swing.JList itemsList;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTable propertiesTable;
    // End of variables declaration	
	
}
