package vtt.vp.gui.service;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.RDF;

import vtt.vp.context.source.StaticContextSource;
import vtt.vp.context.source.VPContextSource;
import vtt.vp.gui.VantagePoint;

public class ServiceRepositoryPanel extends JPanel implements CaretListener, ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;	
	
	public ServiceRepositoryPanel(){
		super();
		initComponents();
	}
	
	public void refreshServiceList(){
		ssrTableModel.refreshServiceList();		
	}
	
	public Vector<String> getServiceURIs(){		
		Vector<String> uris = new Vector<String>(5);
		for(int i=0; i<ssrTableModel.getRowCount(); i++)
			uris.add(ssrTableModel.getValueAt(i, 2).toString());				
		return uris;
	}
	
	private void removeButtonActionPerformed(){
		
		int selectedRowIndex = ssrTable.getSelectedRow();
		if(selectedRowIndex == -1) return;
		ssrTableModel.removeService(selectedRowIndex);		
		
	}
	
	private void clearAllButtonActionPerformed(){
		
		if(VantagePoint.vpActivator == null){
			VantagePoint.printMessage("Adapter not found", Color.red);
			return;
		}
		
		VantagePoint.vpActivator.clearAllServices();
		ServiceRepositoryTableModel.clearServiceURIMappings();		
		
	}
	
	private void browseDescButtonActionPerformed(){
		
		int selectOption = descFileChooser.showOpenDialog(this);		
		if(selectOption == JFileChooser.APPROVE_OPTION){			
			File selectedFile = descFileChooser.getSelectedFile();			
			descTextField.setText(selectedFile.getAbsolutePath());				
		}
		
	}

	private void browseGroundButtonActionPerformed(){
	
		int selectOption = descFileChooser.showOpenDialog(this);		
		if(selectOption == JFileChooser.APPROVE_OPTION){			
			File selectedFile = descFileChooser.getSelectedFile();			
			groundTextField.setText(selectedFile.getAbsolutePath());				
		}
	
	}
	
	private void browseTaskButtonActionPerformed(){
		
		int selectOption = descFileChooser.showOpenDialog(this);		
		if(selectOption == JFileChooser.APPROVE_OPTION){			
			File selectedFile = descFileChooser.getSelectedFile();			
			taskDescTextField.setText(selectedFile.getAbsolutePath());				
		}
	
	}
	
	private void addServiceButtonActionPerformed(){
		
		try{
			
			Model m = ModelFactory.createDefaultModel();
			m.read("file:" + descTextField.getText());
			StmtIterator servicesIter = m.listStatements(new SimpleSelector(
					null, 
					RDF.type , 
					ResourceFactory.createResource("http://www.daml.org/services/owl-s/1.1/Service.owl#Service")));
			
			if(!servicesIter.hasNext()){
				VantagePoint.printMessage("No service instances in description file", Color.red);
				return;
			}
			
			Vector<String> serviceURIs = new Vector<String>(5);
			while(servicesIter.hasNext()){
				serviceURIs.add(servicesIter.nextStatement().getSubject().toString());
			}
			
			if(serviceURIs.size() > 1){
				VantagePoint.printMessage("More than 1 service instance found from description file");
				VantagePoint.printMessage("Registering: " + serviceURIs.firstElement());
			}
						
			String serviceURI = VantagePoint.vpActivator.registerService(
					descTextField.getText(),
					groundTextField.getText());
			
			ServiceRepositoryTableModel.addServiceURIMapping(serviceURIs.firstElement(), serviceURI);
			ssrTableModel.refreshServiceList(); //must refresh again to see possible ownership
							
		}catch(Exception e){
			VantagePoint.printMessage("Error registering services", Color.red);
			VantagePoint.printMessage(e.getMessage());
		}
	
	}
	
	private void testTaskRequestButtonActionPerformed(){
		
		if(VantagePoint.vpActivator == null){
			VantagePoint.printMessage("Adapter not found", Color.red);
			return;
		}
		
		String fileURL = taskDescTextField.getText();
		if(!fileURL.startsWith("file:"))
			fileURL = "file:" + fileURL;
		
		String services[] = VantagePoint.vpActivator.getServicesForTask(fileURL);
		
		VantagePoint.printMessage("Services for task:");
		for(int i=0; i<services.length; i++)
			VantagePoint.printMessage(services[i], Color.blue);		
		
	}
	
	private  void testTypeRequestButtonActionPerformed(){
		
		if(VantagePoint.vpActivator == null){
			VantagePoint.printMessage("Adapter not found", Color.red);
			return;
		}
				
		String services[] = VantagePoint.vpActivator.getServicesWithType(serviceTypeTextField.getText());
		
		VantagePoint.printMessage("Services with type:");
		for(int i=0; i<services.length; i++)
			VantagePoint.printMessage(services[i], Color.blue);
		
	}
	
	public void caretUpdate(CaretEvent ce) {

		JTextField source = (JTextField)ce.getSource();
		
		if(source.equals(descTextField) || source.equals(groundTextField)){
		
			String desc = descTextField.getText();
			String ground = groundTextField.getText();
			if(desc.length() > 0 && ground.length() > 0)
				addServiceButton.setEnabled(true);
			else
				addServiceButton.setEnabled(false);
		
		}else if(source.equals(taskDescTextField)){
			
			if(source.getText().length() > 0)
				testTaskRequestButton.setEnabled(true);
			else
				testTaskRequestButton.setEnabled(false);
			
		}else{
			
			if(source.getText().length() > 0)
				testTypeRequestButton.setEnabled(true);
			else
				testTypeRequestButton.setEnabled(false);
			
		}
		
	}
			
	public void actionPerformed(ActionEvent ae) {
		
		if(ae.getActionCommand().equals("REMOVE")){
			removeButtonActionPerformed();
		}else if(ae.getActionCommand().equals("CLEAR_ALL")){
			clearAllButtonActionPerformed();
		}else if(ae.getActionCommand().equals("BROWSE_DESC")){
			browseDescButtonActionPerformed();
		}else if(ae.getActionCommand().equals("BROWSE_GROUND")){
			browseGroundButtonActionPerformed();
		}else if(ae.getActionCommand().equals("ADD_SERVICE")){
			addServiceButtonActionPerformed();
		}else if(ae.getActionCommand().equals("BROWSE_TASK")){
			browseTaskButtonActionPerformed();
		}else if(ae.getActionCommand().equals("TEST_TASK_REQUEST")){
			testTaskRequestButtonActionPerformed();
		}else if(ae.getActionCommand().equals("TEST_TYPE_REQUEST")){
			testTypeRequestButtonActionPerformed();
		}		
		
	}

	private void initComponents() {
        repositoryPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        ssrTable = new javax.swing.JTable();
        removeServiceButton = new javax.swing.JButton();
        clearAllButton = new javax.swing.JButton();
        addServicePanel = new javax.swing.JPanel();
        descLabel = new javax.swing.JLabel();
        descTextField = new javax.swing.JTextField();
        browseDescButton = new javax.swing.JButton();
        groundLabel = new javax.swing.JLabel();
        groundTextField = new javax.swing.JTextField();
        browseGroundButton = new javax.swing.JButton();
        addServiceButton = new javax.swing.JButton();
        ssrTableModel = new ServiceRepositoryTableModel();
        descFileChooser = new JFileChooser();
        testRequestPanel = new javax.swing.JPanel();
        taskDescLabel = new javax.swing.JLabel();
        serviceTypeLabel = new javax.swing.JLabel();
        taskDescTextField = new javax.swing.JTextField();
        serviceTypeTextField = new javax.swing.JTextField();
        browseTaskDescButton = new javax.swing.JButton();
        testTaskRequestButton = new javax.swing.JButton();
        testTypeRequestButton = new javax.swing.JButton();
        
        repositoryPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Service Repository"));
        
        ssrTable.setModel(ssrTableModel);
        ssrTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane1.setViewportView(ssrTable);

        removeServiceButton.setText("Remove");
        removeServiceButton.setActionCommand("REMOVE");
        removeServiceButton.addActionListener(this);

        clearAllButton.setText("Clear All");
        clearAllButton.setActionCommand("CLEAR_ALL");
        clearAllButton.addActionListener(this);

        javax.swing.GroupLayout repositoryPanelLayout = new javax.swing.GroupLayout(repositoryPanel);
        repositoryPanel.setLayout(repositoryPanelLayout);
        repositoryPanelLayout.setHorizontalGroup(
            repositoryPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(repositoryPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(removeServiceButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(clearAllButton)
                .addContainerGap(237, Short.MAX_VALUE))
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 395, Short.MAX_VALUE)
        );
        repositoryPanelLayout.setVerticalGroup(
            repositoryPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, repositoryPanelLayout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 233, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(repositoryPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(removeServiceButton)
                    .addComponent(clearAllButton))
                .addContainerGap())
        );

        addServicePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Add Service"));
        descLabel.setText("Description:");   
        
        descTextField.addCaretListener(this);

        browseDescButton.setText("Browse");
        browseDescButton.setActionCommand("BROWSE_DESC");
        browseDescButton.addActionListener(this);

        groundLabel.setText("Grounding:");
        
        groundTextField.addCaretListener(this);
        
        browseGroundButton.setText("Browse");
        browseGroundButton.setActionCommand("BROWSE_GROUND");
        browseGroundButton.addActionListener(this);

        addServiceButton.setText("Add Service");
        addServiceButton.setActionCommand("ADD_SERVICE");
        addServiceButton.addActionListener(this);
        addServiceButton.setEnabled(false);

        javax.swing.GroupLayout addServicePanelLayout = new javax.swing.GroupLayout(addServicePanel);
        addServicePanel.setLayout(addServicePanelLayout);
        addServicePanelLayout.setHorizontalGroup(
            addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(addServicePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(addServicePanelLayout.createSequentialGroup()
                        .addGroup(addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(descLabel)
                            .addComponent(groundLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(groundTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 213, Short.MAX_VALUE)
                            .addComponent(descTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 213, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(browseDescButton, javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(browseGroundButton, javax.swing.GroupLayout.Alignment.TRAILING)))
                    .addGroup(addServicePanelLayout.createSequentialGroup()
                        .addComponent(addServiceButton)
                        .addContainerGap(296, Short.MAX_VALUE))))
        );
        addServicePanelLayout.setVerticalGroup(
            addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(addServicePanelLayout.createSequentialGroup()
                .addGroup(addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(descLabel)
                    .addComponent(browseDescButton)
                    .addComponent(descTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(addServicePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(groundLabel)
                    .addComponent(browseGroundButton)
                    .addComponent(groundTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(addServiceButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
                
        testRequestPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Test Service Request"));
        taskDescLabel.setText("Task Description:");

        taskDescTextField.addCaretListener(this);
        
        serviceTypeLabel.setText("Service Type:");
        
        serviceTypeTextField.addCaretListener(this);

        browseTaskDescButton.setText("Browse");
        browseTaskDescButton.setActionCommand("BROWSE_TASK");
        browseTaskDescButton.addActionListener(this);

        testTaskRequestButton.setText("getServicesForTask()");
        testTaskRequestButton.setActionCommand("TEST_TASK_REQUEST");
        testTaskRequestButton.addActionListener(this);
        testTaskRequestButton.setEnabled(false);

        testTypeRequestButton.setText("getServicesWithType()");
        testTypeRequestButton.setActionCommand("TEST_TYPE_REQUEST");
        testTypeRequestButton.addActionListener(this);
        testTypeRequestButton.setEnabled(false);

        javax.swing.GroupLayout testRequestPanelLayout = new javax.swing.GroupLayout(testRequestPanel);
        testRequestPanel.setLayout(testRequestPanelLayout);
        testRequestPanelLayout.setHorizontalGroup(
            testRequestPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(testRequestPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(testRequestPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(testRequestPanelLayout.createSequentialGroup()
                        .addComponent(testTaskRequestButton)
                        .addContainerGap(250, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, testRequestPanelLayout.createSequentialGroup()
                        .addComponent(taskDescLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(taskDescTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 226, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(browseTaskDescButton))
                    .addGroup(testRequestPanelLayout.createSequentialGroup()
                        .addComponent(testTypeRequestButton)
                        .addContainerGap(242, Short.MAX_VALUE))
                    .addGroup(testRequestPanelLayout.createSequentialGroup()
                        .addComponent(serviceTypeLabel)
                        .addGap(20, 20, 20)
                        .addComponent(serviceTypeTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE)
                        .addContainerGap())))
        );
        testRequestPanelLayout.setVerticalGroup(
            testRequestPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(testRequestPanelLayout.createSequentialGroup()
                .addGroup(testRequestPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(taskDescLabel)
                    .addComponent(taskDescTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(browseTaskDescButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(testTaskRequestButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(testRequestPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(serviceTypeLabel)
                    .addComponent(serviceTypeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(testTypeRequestButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(testRequestPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(addServicePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(repositoryPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(repositoryPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(addServicePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(testRequestPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        
    }// </editor-fold> 
	
    // Variables declaration - do not modify                     
    private javax.swing.JButton addServiceButton;
    private javax.swing.JPanel addServicePanel;
    private javax.swing.JButton browseDescButton;
    private javax.swing.JButton browseGroundButton;
    private javax.swing.JButton clearAllButton;
    private javax.swing.JLabel descLabel;
    private javax.swing.JTextField descTextField;
    private javax.swing.JLabel groundLabel;
    private javax.swing.JTextField groundTextField;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable ssrTable;
    private javax.swing.JButton removeServiceButton;
    private javax.swing.JPanel repositoryPanel;
    private ServiceRepositoryTableModel ssrTableModel;
    private JFileChooser descFileChooser;
    private javax.swing.JLabel serviceTypeLabel;
    private javax.swing.JTextField serviceTypeTextField;
    private javax.swing.JLabel taskDescLabel;
    private javax.swing.JTextField taskDescTextField;
    private javax.swing.JPanel testRequestPanel;
    private javax.swing.JButton testTaskRequestButton;
    private javax.swing.JButton testTypeRequestButton;
    private javax.swing.JButton browseTaskDescButton;
    // End of variables declaration
	
}
