package vtt.vp.gui.service;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;
import java.util.Map.Entry;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import vtt.vp.gui.VantagePoint;

public class ServiceRepositoryTableModel extends AbstractTableModel{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private String columnNames[] = new String[]{"Name", "Ownership", "URI"};
	private Vector<Object[]> services = new Vector<Object[]>(10);
	private static Hashtable<String, String> serviceURIMapping = new Hashtable<String, String>(10);
	
	public ServiceRepositoryTableModel(){
		super();				
	}
	
	public void refreshServiceList(){
		
		if(VantagePoint.vpActivator == null){
			VantagePoint.printMessage("Adapter not found", Color.red);
			return;
		}
		
		services.removeAllElements();
		String s[] = VantagePoint.vpActivator.getAllServices();
		if(s == null){
			fireTableChanged(new TableModelEvent(this));
			return;
		}
		String altUri;
		String localName;
		String uri;
		for(int i=0; i<s.length; i++){
			uri = s[i];
			localName = uri.substring(uri.indexOf("#"), uri.length());
			altUri = serviceURIMapping.get(uri);
			if(altUri != null)
				services.add(new Object[]{localName, true, uri});
			else
				services.add(new Object[]{localName, false, uri});
		}
		fireTableChanged(new TableModelEvent(this));
	}
	
	public void removeService(int i){
		String uri = (String)services.elementAt(i)[2];
		String altUri = serviceURIMapping.get(uri);
		if(altUri != null){
			VantagePoint.vpActivator.removeService(altUri);
			serviceURIMapping.remove(uri);
		}
		else
			VantagePoint.printMessage("Cannot remove services without ownership");
	}
		
	public static void addServiceURIMapping(String uri, String altUri){
		serviceURIMapping.put(uri, altUri);
	}
	
	public static void clearServiceURIMappings(){
		serviceURIMapping.clear();
	}
	
	public static void saveServiceURIMapping(){
		
		Properties prop = new Properties();
		prop.putAll(serviceURIMapping);
		try {
			FileOutputStream fos = new FileOutputStream(VantagePoint.vpPath+"config"+File.separator+"services.xml");
			prop.storeToXML(fos, "Service URI mappings to the services that VantagePoint has ownership");
			fos.close();
		} catch (Exception e) {
			System.err.println("Could not save service URI mapping to: " + VantagePoint.vpPath+"config"+File.separator+"services.xml");
			e.printStackTrace();		
		}
		printMapping();
	}
	
	public static void loadServiceURIMapping(){
		
		Properties prop = new Properties();		
		try {
			FileInputStream fis = new FileInputStream(VantagePoint.vpPath+"config"+File.separator+"services.xml");
			prop.loadFromXML(fis);
			fis.close();
		} catch (Exception e) {
			System.err.println("Could not load service URI mapping from: " + VantagePoint.vpPath+"config"+File.separator+"services.xml");
			e.printStackTrace();		
		}
		
		Iterator entries = prop.entrySet().iterator();
		while(entries.hasNext()){
			Entry entry = (Entry)entries.next();
			serviceURIMapping.put(entry.getKey().toString(), entry.getValue().toString());
		}
		printMapping();
	}
	
	private static void printMapping(){
		Iterator i = serviceURIMapping.entrySet().iterator();
		while(i.hasNext()){
			Entry e = (Entry)i.next();
			System.out.println("Key: " + e.getKey() + " Value: " + e.getValue());
		}
	}
	
	public int getColumnCount() {		
		return columnNames.length;
	}
	
	@SuppressWarnings("unchecked")
	public Class getColumnClass(int i){
		if(i == 1)
			return Boolean.class;
		else
			return String.class;
	}

	public String getColumnName(int x) {
        return columnNames[x];
    }
	
	public int getRowCount() {
		return services.size();
	}

	public Object getValueAt(int x, int y) {
		return services.elementAt(x)[y];
	}

}
