package vtt.vp.utils;

import java.util.Vector;

import com.hp.hpl.jena.iri.IRI;
import com.hp.hpl.jena.iri.IRIFactory;
import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

import vtt.vp.library.IconLibrary;
import vtt.vp.library.IconSet;
import vtt.vp.utils.VPUtils;
import vtt.vp.context.source.VPContextSource;

public class VPOperations {

	public static void moveVisualComponent(
			String uri, 
			String newParentURI, 
			int gridX, 
			int gridY, 
			OntModel model, 
			VPContextSource vpcs)throws Exception{
				
		long start = System.currentTimeMillis();
		System.out.println("Start move");
		Statement statement;
		Individual parentInd;		
		Individual ind = model.getIndividual(uri);		
		if(ind == null || gridX<0 || gridY<0) throw new Exception("Illegal parameters");					
		
		//Test item move operation
		if(ind.hasRDFType(VPUtils.VANTAGEPOINT_NS+"Item")){
			
			if(newParentURI == null) throw new Exception("Parent does not exist");
			parentInd = model.getIndividual(newParentURI);
			
			//Get item dimensions	
			int itemWidth, itemHeight;
			statement = ind.getProperty(VPUtils.ICONSET_PROP);			
			itemWidth = model.getIndividual(statement.getObject().toString())
						.getProperty(VPUtils.ICONGRIDWIDTH_PROP).getInt();	
			itemHeight = model.getIndividual(statement.getObject().toString())
						.getProperty(VPUtils.ICONGRIDHEIGHT_PROP).getInt();
			
			//Get item direction, because if west or east width and height must be swapped
			statement = ind.getProperty(VPUtils.DIRECTION_PROP);
			String dString = statement.getString();
			
			if(dString.equals("WEST") || dString.equals("EAST")){				
				int temp = itemWidth;
				itemWidth = itemHeight;
				itemHeight = temp;
			}
			
			if(!testItemPlacement(parentInd, itemWidth, itemHeight, gridX, gridY, model))
				throw new Exception("Illegal placement");
				
			//Move operation has passed tests and it is legal
			
			Vector<Statement> removeStatements = new Vector<Statement>(4);			
			removeStatements.add(ind.getProperty(VPUtils.GRIDX_PROP));
			removeStatements.add(ind.getProperty(VPUtils.GRIDY_PROP));
			
			Vector<Statement> addStatements = new Vector<Statement>(4);
			addStatements.add(ResourceFactory.createStatement(
					ind, 
					VPUtils.GRIDX_PROP, 
					model.createTypedLiteral(Integer.toString(gridX),"http://www.w3.org/2001/XMLSchema#int")));
			addStatements.add(ResourceFactory.createStatement(
					ind, 
					VPUtils.GRIDY_PROP, 
					model.createTypedLiteral(Integer.toString(gridY),"http://www.w3.org/2001/XMLSchema#int")));
									
			//get old parent (immediate parent)
			Individual oldParent = model.getIndividual(VPUtils.getImmediateParentURI(ind));
			
			//If old parent area is not the same area
			if(!oldParent.getURI().equals(newParentURI)){
				
				removeStatements.add(ResourceFactory.createStatement(
						oldParent, 
						VPUtils.CONTAINS_PROP, 
						ind));
				removeStatements.add(ResourceFactory.createStatement(
						ind, 
						VPUtils.ISLOCATEDIN_PROP, 
						oldParent));						
				
				addStatements.add(ResourceFactory.createStatement(
						parentInd, 
						VPUtils.CONTAINS_PROP, 
						ind));
				addStatements.add(ResourceFactory.createStatement(
						ind, 
						VPUtils.ISLOCATEDIN_PROP, 
						parentInd));			
				
			}
			
			Statement removes[] = new Statement[removeStatements.size()];
			Statement adds[] = new Statement[addStatements.size()];
			vpcs.modify(removeStatements.toArray(removes), addStatements.toArray(adds));			
			
		}		
		//Test area move operation
		else if(ind.hasRDFType(VPUtils.VANTAGEPOINT_NS+"Area")){			
			
			parentInd = null;
			if(newParentURI != null)
				parentInd = model.getIndividual(newParentURI);
			
			if(parentInd != null && !parentInd.hasRDFType(VPUtils.VANTAGEPOINT_NS + "Area")) 
				throw new Exception("Illegal parent");
			
			String areaString = ind.getProperty(VPUtils.AREAMATRIX_PROP).getString();
			
			if(!(testAreaPlacement(parentInd, ind.getURI(), areaString, gridX, gridY, model)))
				throw new Exception("Illegal placement");
				
			//Move operation has passed tests and is legal
			
			//Change area location
			Vector<Statement> removeStatements = new Vector<Statement>(4);			
			removeStatements.add(ind.getProperty(VPUtils.GRIDX_PROP));
			removeStatements.add(ind.getProperty(VPUtils.GRIDY_PROP));
			
			Vector<Statement> addStatements = new Vector<Statement>(4);			 
			addStatements.add(ResourceFactory.createStatement(
					ind,
					VPUtils.GRIDX_PROP, 
					model.createTypedLiteral(Integer.toString(gridX),"http://www.w3.org/2001/XMLSchema#int")));
			addStatements.add(ResourceFactory.createStatement(
					ind,
					VPUtils.GRIDY_PROP, 
					model.createTypedLiteral(Integer.toString(gridY),"http://www.w3.org/2001/XMLSchema#int")));				
			
			//Update containment relationships
			
			//If old parent is not the same as new parent
			String oldParentURI = VPUtils.getImmediateParentURI(ind);
			
			if(oldParentURI != null && newParentURI != null){
				//Test if the areas are NOT the same
				if(!oldParentURI.equals(newParentURI)){
					
					//remove relationship with old parent area
					Individual oldParent = model.getIndividual(oldParentURI);
					removeStatements.add(ResourceFactory.createStatement(
							ind,
							VPUtils.ISLOCATEDIN_PROP,
							oldParent));
					removeStatements.add(ResourceFactory.createStatement(
							oldParent,
							VPUtils.CONTAINS_PROP,
							ind));					
					
					//add relationship with new parent area
					addStatements.add(ResourceFactory.createStatement(
							ind,
							VPUtils.ISLOCATEDIN_PROP,
							parentInd));
					addStatements.add(ResourceFactory.createStatement(
							parentInd,
							VPUtils.CONTAINS_PROP,
							ind));
					
				}				
			}else{				
				if(oldParentURI == null && newParentURI != null){
					
					//add relationship with new parent area
					addStatements.add(ResourceFactory.createStatement(
							ind,
							VPUtils.ISLOCATEDIN_PROP,
							parentInd));
					addStatements.add(ResourceFactory.createStatement(
							parentInd,
							VPUtils.CONTAINS_PROP,
							ind));
					
				}else if(oldParentURI != null && newParentURI == null){
					
					//remove relationship with old parent area
					Individual oldParent = model.getIndividual(oldParentURI);
					removeStatements.add(ResourceFactory.createStatement(
							ind,
							VPUtils.ISLOCATEDIN_PROP,
							oldParent));
					removeStatements.add(ResourceFactory.createStatement(
							oldParent,
							VPUtils.CONTAINS_PROP,
							ind));
					
				}
			}			
			
			Statement removes[] = new Statement[removeStatements.size()];
			Statement adds[] = new Statement[addStatements.size()];
			vpcs.modify(removeStatements.toArray(removes), addStatements.toArray(adds));
			
		}					
		
		System.out.println("Move time: " + (System.currentTimeMillis()-start));
		
		
	}
	
	public static void deleteVisualComponent(String uri, OntModel model, VPContextSource vpcs)throws Exception{
						
		Individual ind = model.getIndividual(uri);
		
		if(ind == null || 
				(!ind.hasRDFType(VPUtils.VANTAGEPOINT_NS+"Area") && !ind.hasRDFType(VPUtils.VANTAGEPOINT_NS+"Item")))
				throw new Exception("Illegal delete operation target");
		
		Vector<Statement> removeStatements = new Vector<Statement>();
		getDeletedStatements(ind, model, removeStatements);		
		Statement removes[] = new Statement[removeStatements.size()];		
		vpcs.modify(removeStatements.toArray(removes), null);
		
	}
	
	public static void rotateItem(String uri, int direction, OntModel model, VPContextSource vpcs)throws Exception{
		
		//Direction value out of scope
		if(direction < 0 || direction > 4) throw new Exception("Illegal direction value");
		
		//Item not in model
		Individual ind = model.getIndividual(uri);
		if(ind == null) throw new Exception("Item does not exist");
		
		Statement statement;		
		
		//Get item location		
		int gridX, gridY;
		statement = ind.getProperty(VPUtils.GRIDX_PROP);
		gridX = statement.getInt();		
		statement = ind.getProperty(VPUtils.GRIDY_PROP);
		gridY = statement.getInt();
		
		//Get Item dimensions
		int itemWidth, itemHeight;
		statement = ind.getProperty(VPUtils.ICONSET_PROP);			
		itemWidth = model.getIndividual(statement.getObject().toString())
					.getProperty(VPUtils.ICONGRIDWIDTH_PROP).getInt();	
		itemHeight = model.getIndividual(statement.getObject().toString())
					.getProperty(VPUtils.ICONGRIDHEIGHT_PROP).getInt();
		
		statement = ind.getProperty(VPUtils.DIRECTION_PROP);
		String dString = statement.getString();		
		
		//Test for rotating is done only if the direction change has an effect
		//on the actual dimensions of the item
		//w!=h and direction changes dimensions
		if((itemWidth != itemHeight) &&
				(((dString.equals("SOUTH") || dString.equals("NORTH")) && (direction == 1 || direction == 3)) ||
				((dString.equals("WEST") || dString.equals("EAST")) && (direction == 0 || direction == 2)))){
										
			if(direction == 1 || direction == 3){
				int temp = itemWidth;
				itemWidth = itemHeight;
				itemHeight = temp;
			}
			
			statement = ind.getProperty(VPUtils.ISLOCATEDIN_PROP);
			Individual parentArea = model.getIndividual(statement.getObject().toString());
			
			if(!testItemPlacement(parentArea, itemWidth, itemHeight, gridX, gridY, model))
				throw new Exception("Illegal rotation operation");							
				
		}	
		
		//tests passed	
		Statement remStmts[] = new Statement[]{ind.getProperty(VPUtils.DIRECTION_PROP)};
		Statement addStmts[] = new Statement[1];		
		addStmts[0] = ResourceFactory.createStatement(
				ind, 
				VPUtils.DIRECTION_PROP, 
				model.createTypedLiteral(VPUtils.DirectionIntToString(direction), "http://www.w3.org/2001/XMLSchema#string"));
		vpcs.modify(remStmts, addStmts);
		
	}
	
	public static void addItem(
			String uri, 
			String parentURI, 
			int gridX, 
			int gridY, 
			OntModel model,
			Model data,
			VPContextSource vpcs)throws Exception{
		
		//Check URI
		IRI iri = IRIFactory.jenaImplementation().create(uri);
		if(iri.hasViolation(false))			
			throw new Exception("Item URI illegal");
		
		//Check that Item does not exist already
		Individual ind = model.getIndividual(uri);
		if(ind != null)
			throw new Exception("Item already in the model");
		
		//Check parent URI
		if(parentURI == null)
			throw new Exception("Parent URI not defined");
		
		//Check parent existence
		Individual parentInd = model.getIndividual(parentURI);
		if(parentInd == null)
			throw new Exception("Parent does not exist");
		
		//Check parent class
		if(!parentInd.hasRDFType(model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Area")))
			throw new Exception("Parent is not of type VantagePoint:Area");
		
		//Check placement
		Resource itemRes = data.getResource(uri);		
		Statement iconSetStmt = itemRes.getProperty(VPUtils.ICONSET_PROP);
		Resource iconSetRes = (Resource)iconSetStmt.getObject();
		String iconSetName = iconSetRes.getLocalName();
		if(iconSetName.startsWith("_")) iconSetName = iconSetName.substring(1, iconSetName.length()-1);
		IconSet is = IconLibrary.getIconLibraryObject().getIconSet(iconSetName);
		if(!testItemPlacement(parentInd, is.getGridWidth(), is.getGridHeight(), gridX, gridY, model))
			throw new Exception("Illegal item placement");
		
		Vector<Statement> addStatements = new Vector<Statement>();
		
		StmtIterator iter = data.listStatements();
		while(iter.hasNext())
			addStatements.add(iter.nextStatement());
					
		//Add parent		
		addStatements.add(ResourceFactory.createStatement(itemRes, VPUtils.ISLOCATEDIN_PROP, parentInd));		
		
		//Set coordinates
		addStatements.add(ResourceFactory.createStatement(itemRes, VPUtils.GRIDX_PROP, model.createTypedLiteral(gridX)));
		addStatements.add(ResourceFactory.createStatement(itemRes, VPUtils.GRIDY_PROP, model.createTypedLiteral(gridY)));
				
		//Set direction
		addStatements.add(ResourceFactory.createStatement(itemRes, VPUtils.DIRECTION_PROP, model.createTypedLiteral("SOUTH")));	
		
		Statement addStmts[] = new Statement[addStatements.size()];
		vpcs.modify(null, addStatements.toArray(addStmts));
		
	}
	
	private static void getDeletedStatements(
			Individual ind, 
			OntModel model, 
			Vector<Statement> removeStatements)throws Exception{
				
		//Get all the statements where the VisualComponent is the subject
		StmtIterator iter = model.listStatements(new SimpleSelector(ind, null, (RDFNode)null));		
		while(iter.hasNext())
			removeStatements.add(iter.nextStatement());
		
		//Get all the statements where the VisualComponent is the object
		iter = model.listStatements(new SimpleSelector(null, null, ind));		
		while(iter.hasNext())
			removeStatements.add(iter.nextStatement());
		
		//recurse if individual is an area
		if(ind.hasRDFType(VPUtils.VANTAGEPOINT_NS+"Area")){
			Vector<Individual> children = VPUtils.listImmediateChildren(ind, model);
			for(Individual child : children){
				getDeletedStatements(child, model, removeStatements);
			}
		}
		
	}
	
	/**
	 * Checks that item placement is legal. Items can overlap each other but they
	 * cannot intersect boundaries of their parent space or peer areas. 
	 * 
	 * @param parent		the parent space Individual
	 * @param itemWidth		width of the item in grid squares
	 * @param itemHeight	height of the item in grid squares
	 * @param gridX			item x coordinate in grid
	 * @param gridY			item y coordiante in grid
	 * @return	true if placement is legal and false if not
	 */
	private static boolean testItemPlacement(Individual parent, int itemWidth, int itemHeight, int gridX, int gridY, OntModel model){
		
		Statement statement;
		boolean areaMatrix[][];
		
		//1.
		//Test that item is completely inside Area		
		
		if(parent == null)
			return false;
		
		//Get parent Area area matrix			
		statement = parent.getProperty(VPUtils.AREAMATRIX_PROP);			
		areaMatrix = VPUtils.parseAreaString(statement.getString());
					
		//Test that item is completely inside the parent Area						
		for(int j=gridY; j<gridY+itemHeight; j++){				
			for(int i=gridX; i<gridX+itemWidth; i++){
				//System.out.println("j: "+j);
				//System.out.println("i: "+i);
				try{
					if(!areaMatrix[j][i]){
						//System.out.println("Outside area bounds2");
						return false;					
					}
				}catch(ArrayIndexOutOfBoundsException aiobe){
					//System.out.println("Outside area bounds1");
					return false;	
				}					
			}				
		}		
		
		//Gather all parent's immediate child Areas			
		Vector<Individual> children = new Vector<Individual>();
		Vector<Individual> childAreaIndividuals = new Vector<Individual>();
		children = VPUtils.listImmediateChildren(parent, model);		
		for(int i=0; i<children.size(); i++){
			if(children.elementAt(i).hasRDFType(VPUtils.VANTAGEPOINT_NS + "Area"))
				childAreaIndividuals.add(children.elementAt(i));
		}
		
		//Test that item does not intersect with immediate chilAreas
		int childAreaX;
		int childAreaY;
		int shiftX;
		int shiftY;			
		for(int i=0; i<childAreaIndividuals.size(); i++){			
						
			statement = childAreaIndividuals.elementAt(i).getProperty(VPUtils.GRIDX_PROP);
			childAreaX = statement.getInt();				
			//System.out.println("x: "+childAreaX);				
			statement = childAreaIndividuals.elementAt(i).getProperty(VPUtils.GRIDY_PROP);
			childAreaY = statement.getInt();				
			//System.out.println("y: "+childAreaY);				
			statement = childAreaIndividuals.elementAt(i).getProperty(VPUtils.AREAMATRIX_PROP);			
			areaMatrix = VPUtils.parseAreaString(statement.getString());
			
			shiftX = gridX - childAreaX;
			shiftY = gridY - childAreaY;				
			//System.out.println("shift x: "+shiftX);
			//System.out.println("shift y: "+shiftY);				
			
			for(int j=0; j<itemHeight; j++){
				for(int k=0; k<itemWidth; k++){
					//System.out.println("test x: "+(k+shiftX));
					//System.out.println("test y: "+(j+shiftY));
					try{
						if(areaMatrix[j+shiftY][k+shiftX]){
							//System.out.println("Intersects with childArea");
							return false;
						}
					}catch(ArrayIndexOutOfBoundsException aiobe){}
				}
			}				
		}
	
		return true;
		
	}			
	
	private static boolean testAreaPlacement(Individual parentInd, String areaURI, String areaString, int gridX, int gridY, OntModel model){
		
		Statement statement;
		//Get Area's area matrix
		boolean areaMatrix[][];				
		areaMatrix = VPUtils.parseAreaString(areaString);
		
		if(parentInd != null){
			
			//Test that area is completely inside its parent											
			boolean parentAreaMatrix[][];
			statement = parentInd.getProperty(VPUtils.AREAMATRIX_PROP);
			parentAreaMatrix = VPUtils.parseAreaString(statement.getString());
			
			for(int j=gridY; j<gridY+areaMatrix.length; j++){					
				for(int i=gridX; i<gridX+areaMatrix[0].length; i++){
					try{
						if(areaMatrix[j-gridY][i-gridX] && !parentAreaMatrix[j][i]){
							//System.out.println("area not completely inside parent area1");
							return false;		
						}
					}catch(ArrayIndexOutOfBoundsException aiobe){
						//System.out.println("area not completely inside parent area2");
						return false;
					}
				}					
			}
			
		}							
		
		//Gather parent areas immediate child components
		//Do not include the component moved itself
		Vector<Individual> childComponents = new Vector<Individual>();
		childComponents = VPUtils.listImmediateChildren(parentInd, model);
		for(int i=0; i<childComponents.size(); i++)
			if(childComponents.elementAt(i).getURI().equals(areaURI))
				childComponents.remove(i);
					
		//Test that area does not intersect with parent areas immediate child components
		Individual childInd;
		int childX, childY, childWidth, childHeight, shiftX, shiftY;
		boolean childAreaMatrix[][];
		for(int i=0; i<childComponents.size(); i++){				
				
			childInd = childComponents.elementAt(i);
			if(childInd.hasRDFType(VPUtils.VANTAGEPOINT_NS+"Item")){
				
				//get item dimensions
				statement = childInd.getProperty(VPUtils.ICONSET_PROP);			
				childWidth = model.getIndividual(statement.getObject().toString())
							.getProperty(VPUtils.ICONGRIDWIDTH_PROP).getInt();	
				childHeight = model.getIndividual(statement.getObject().toString())
							.getProperty(VPUtils.ICONGRIDHEIGHT_PROP).getInt();
				statement = childInd.getProperty(VPUtils.GRIDX_PROP);
				childX = statement.getInt();
				statement = childInd.getProperty(VPUtils.GRIDY_PROP);
				childY = statement.getInt();
				
				shiftX = childX - gridX;
				shiftY = childY - gridY;
				
				//Test intersection with item
				for(int j=0; j<childHeight; j++){
					for(int k=0; k<childWidth; k++){
						//System.out.println("test x: "+(k+shiftX));
						//System.out.println("test y: "+(j+shiftY));
						try{
							if(areaMatrix[j+shiftY][k+shiftX]){
								//System.out.println("Intersects with Item");
								return false;
							}
						}catch(ArrayIndexOutOfBoundsException aiobe){}
					}
				}
				
			}else{
				
				//get area dimensions									
				statement = childInd.getProperty(VPUtils.GRIDX_PROP);
				childX = statement.getInt();							
				statement = childInd.getProperty(VPUtils.GRIDY_PROP);
				childY = statement.getInt();									
				statement = childInd.getProperty(VPUtils.AREAMATRIX_PROP);			
				childAreaMatrix = VPUtils.parseAreaString(statement.getString());
				
				shiftX = childX - gridX;
				shiftY = childY - gridY;
				
				//Test intersection with area
				for(int j=0; j<childAreaMatrix.length; j++){
					for(int k=0; k<childAreaMatrix[0].length; k++){
						//System.out.println("test x: "+(k+shiftX));
						//System.out.println("test y: "+(j+shiftY));
						try{
							if(childAreaMatrix[j][k] && areaMatrix[j+shiftY][k+shiftX]){
								//System.out.println("Intersects with Area");
								return false;
							}
						}catch(ArrayIndexOutOfBoundsException aiobe){}
					}
				}
				
			}
			
		}
		
		
		return true;
		
	}
	
}
