package vtt.vp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import vtt.vp.context.source.VPContextSource;
import vtt.vp.gui.VantagePoint;
import vtt.vp.library.IconLibrary;
import vtt.vp.library.IconSet;
import vtt.vp.utils.AreaRectangle;
import vtt.vp.utils.VPOperations;
import vtt.vp.utils.VPUtils;
import vtt.vp.utils.Grid;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.ResourceUtils;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.vocabulary.RDF;

public class EditView extends VPView 
implements ActionListener, MouseListener, MouseMotionListener {
		
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * The context source to be visualized.
	 */
	private VPContextSource contextSource;	
	private OntModel model;
	private JLayeredPane layeredPane = new JLayeredPane();		
			
	/**
	 * The currently selected VisualComponent in VPEditView
	 */
	private Component selected;
	
	/**
	 * The currently highlighted component. 
	 */
	private Component highlighted;
		
	private boolean moveEnabled = false;
	private Point grabPoint;
	private Point viewportStartLocation;	
	private Vector<String[]> defectiveInstances;		
	private boolean addRectangleAreaEnabled = false;
	private Point rectangleStartCorner = new Point();
	private Point rectangleEndCorner = new Point();
	private AreaRectangle rectangle;		
	public JLabel itemFrame = new JLabel();		
	
	private Model iconLibModel;
	private Model vantagePointModel;
	
	private Container oldContainer;
	private int oldX;
	private int oldY;
	
	/**
	 * Class constructor. 
	 * @param m	the model to be visualized
	 */
	public EditView(VPContextSource vpcs){
		
		super();
		contextSource = vpcs;
		
		//Initialize OntModel
		OntDocumentManager mgr = new OntDocumentManager("file:"+VantagePoint.vpPath+"config"+File.separator+"ont-policy.rdf");		
        OntModelSpec s = new OntModelSpec(OntModelSpec.OWL_MEM_MICRO_RULE_INF);        			
		s.setDocumentManager(mgr);
		model = ModelFactory.createOntologyModel(s);		
		
		//Load IconLibrary.owl
		iconLibModel = ModelFactory.createDefaultModel();		
		iconLibModel.read("file:" + VantagePoint.vpPath + "ontologies" + File.separator + "IconLibrary.owl");
				
		//Load VantagePoint.owl
		vantagePointModel = ModelFactory.createDefaultModel();		
		vantagePointModel.read("file:" + VantagePoint.vpPath + "ontologies" + File.separator + "VantagePoint.owl");
				
		this.defectiveInstances = new Vector<String[]>();
		layeredPane.addMouseListener(this);
		layeredPane.addMouseMotionListener(this);
		
		initComponents();
		refresh();		
			
	}

	/**
	 * Scans the model for visualizable components and refreshes the
	 * view. First create all areas and items to vectors. Then locate
	 * components in recursive fashion starting from the root areas.
	 *
	 */
	public void refresh(){			
				
		Statement statement;
		Individual individual;
		int gridX;
		int gridY;
		int gridW;
		int gridH;		
		Vector<VPArea> areas = new Vector<VPArea>();		
		Vector<VPItem> items = new Vector<VPItem>();
		
		//Empty model from statements
		model.removeAll();
		
		//Read the context source
		InputStream rdfStream = contextSource.getRDFDataStream();
		model.read(rdfStream, "");
		
		//Add imports
		model.add(iconLibModel);
		model.add(vantagePointModel);
		
		//First clear all components
		layeredPane.removeAll();
		this.defectiveInstances = new Vector<String[]>();
				
		//Set visualization size
		gridW = 50;
		gridH = 50;
		layeredPane.setSize(gridW*20, gridH*20);
		layeredPane.setPreferredSize(new Dimension(gridW*20, gridH*20));		
		
		//Get all areas in model
		OntClass areaOntClass = model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Area");		
		ExtendedIterator areasIterator = areaOntClass.listInstances();		
		Object areaArray[] = areasIterator.toSet().toArray();		
		
		//Create EditViewAreas
		//
		//-create and set properties
		//-add to vectors		
		//
		for(int i=0; i<areaArray.length; i++){
			
			//Create EditViewArea instance, areamatrix (obligatory)
			individual = (Individual)areaArray[i];
			statement = individual.getProperty(VPUtils.AREAMATRIX_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Area matrix property not found."});
				continue;
			}
			EditViewArea evArea = new EditViewArea(individual.getURI(),statement.getString());
			
			//Set location (obligatory)
			statement = individual.getProperty(VPUtils.GRIDX_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid x property not found."});
				continue;
			}
			gridX = statement.getInt();
			statement = individual.getProperty(VPUtils.GRIDY_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid y property not found."});
				continue;
			}
			gridY = statement.getInt();
			evArea.setLocation(gridX*20, gridY*20);					
			
			//Set child components (optional)
			StmtIterator childIter = individual.listProperties(VPUtils.CONTAINS_PROP);
			String childURI;
			Individual childIndividual;			
			while(childIter.hasNext()){							
				
				childURI = childIter.nextStatement().getObject().toString();				
				childIndividual = model.getIndividual(childURI);
				
				if(childIndividual.hasRDFType(areaOntClass))
					evArea.addChildAreaURI(childURI);
				else
					evArea.addChildItemURI(childURI);
			}			
			
			//Set parent area
			String parentURI = VPUtils.getImmediateParentURI(individual);
			
			//If the area is a root area it can be attached straight away
			if(parentURI == null){				
				layeredPane.add(evArea);
				layeredPane.setLayer(evArea, 1);
			}			
			evArea.setParentAreaURI(parentURI);
						
			//Add to vector
			areas.add(evArea);
						
		}
		
		//Get all items in model
		OntClass itemOntClass = model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Item");		
		ExtendedIterator itemsIterator = itemOntClass.listInstances();		
		Object [] itemArray = itemsIterator.toSet().toArray();		
		
		//Create EditViewItems
		//
		//-create and set properties
		//-add to vector		
		//
		for(int i=0; i<itemArray.length; i++){
			
			//Create EditViewItem instance, iconset (obligatory)
			individual = (Individual)itemArray[i];
			statement = individual.getProperty(VPUtils.ICONSET_PROP);			
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Icon set property not found."});
				continue;
			}
			Individual iconSetInd = model.getIndividual(statement.getObject().toString());			
			if(iconSetInd == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Icon set does not exist."});
				continue;
			}
			statement = iconSetInd.getProperty(VPUtils.ICONGRIDWIDTH_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Icon grid width property not found."});
				continue;
			}
			gridW = statement.getInt();
			statement = iconSetInd.getProperty(VPUtils.ICONGRIDHEIGHT_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Icon grid height property not found."});
				continue;
			}
			gridH = statement.getInt();
			EditViewItem evItem = new EditViewItem(individual.getURI(),gridW,gridH);
						
			//Set location (obligatory)
			statement = individual.getProperty(VPUtils.GRIDX_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid x property not found."});
				continue;
			}
			gridX = statement.getInt();
			statement = individual.getProperty(VPUtils.GRIDY_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid y property not found."});
				continue;
			}
			gridY = statement.getInt();
			evItem.setLocation(gridX*20, gridY*20);	
			
			//Set direction (optional)
			statement = individual.getProperty(VPUtils.DIRECTION_PROP);
			if(statement == null)
				evItem.setDirection(VPItem.ITEM_SOUTH);
			else
				evItem.setDirection(VPUtils.DirectionStringToInt(statement.getString()));			
						
			//Set parent area (obligatory)
			String parentURI = VPUtils.getImmediateParentURI(individual);
			if(parentURI == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Parent area property not found."});
				continue;
			}			
			evItem.setParentAreaURI(parentURI);			
			
			//Add to vector			
			items.add(evItem);
						
		}
		
		//Build swing containment hierarchy recursively		
		buildComponentTree(layeredPane, areas, items);
				
		//Add grid
		Grid grid = new Grid(layeredPane.getWidth()/20,layeredPane.getHeight()/20);
		layeredPane.add(grid, new Integer(0));
		
		this.repaint();
		
	}	
	
	public VPContextSource getContextSource(){
		return contextSource;
	}
	
	/**
	 * Empties the view from components and clears the defective instances vector.
	 *
	 */
	public void clear(){
		layeredPane.removeAll();
		defectiveInstances = new Vector<String[]>();
	}
	
	/**
	 * Get the list of components that were disqualified from the visualization.
	 * The vector contains String arrays in which the first element is the URI
	 * of the defective component and second element is the reason in few words.
	 * 
	 * @return	defective components vector
	 */
	public Vector<String[]> getDefectiveInstances(){
		return defectiveInstances;
	}
	
	/**
	 * Sets the highlighted selection off from layeredPane view.
	 */
	public void selectionOff(){
		if(selected != null){
			((VisualComponent)selected).setSelected(false);
		}
	}
	
	/**
	 * Sets the add rectangle area state on and turns the mouse
	 * cursor to add rectangle area cursor.
	 *
	 */
	public void addRectangleArea(){
					
		addRectangleAreaEnabled = true;
		Image areaCursorImage = null;
		
		try{
			
			areaCursorImage = ImageIO.read(new File(VantagePoint.vpPath + "graphic\\images\\rectangularAreaCursor.png"));
			
			Cursor areaCursor = Toolkit.getDefaultToolkit().createCustomCursor(
					areaCursorImage,
					new Point(8,8),
					"rectangulerAreaCursor");
			layeredPane.setCursor(areaCursor);
			
		}catch(IOException ioe){
			System.err.println("Rectangular area cursor image not found.");
		}
			
	}
	
	/**
	 * Sets the add item state on and sets the item icon to
	 * follow mouse cursor.
	 * @param arguments
	 */
	public void addItemModeOn(int itemFrameWidth, int itemFrameHeight){		
				
		BufferedImage itemFrameImage = new BufferedImage(
				itemFrameWidth,
				itemFrameHeight,
				BufferedImage.TYPE_INT_ARGB);
		Graphics g = itemFrameImage.getGraphics();
		g.setColor(Color.blue);
		g.drawRect(0, 0, itemFrameWidth-1, itemFrameHeight-1);
		g.drawRect(1, 1, itemFrameWidth-3, itemFrameHeight-3);
		itemFrame.setSize(itemFrameWidth, itemFrameHeight);
		itemFrame.setIcon(new ImageIcon(itemFrameImage));
		itemFrame.setLocation(-1000, -1000);
		layeredPane.add(itemFrame, new Integer(3));
		
	}
	
	public boolean addItem(Resource itemRes, Model data){
			
		//Get parent URI
		layeredPane.remove(itemFrame);
		Component topmost = SwingUtilities.getDeepestComponentAt(layeredPane, itemFrame.getX(), itemFrame.getY());				
		String parentURI = null;
		if(topmost instanceof VisualComponent){				
			VisualComponent vc = (VisualComponent)topmost;				
			if(topmost instanceof VPItem){
				parentURI = vc.getParentAreaURI();
				topmost = topmost.getParent();
			}else
				parentURI = vc.getURI();				
		}		
		
		//Get new relative location
		Point newLocation = SwingUtilities.convertPoint(layeredPane, itemFrame.getX(), itemFrame.getY(), topmost);
				
		//Ask for item URI	    
		String input = (String)JOptionPane.showInputDialog("Enter item URI", model.getNsPrefixURI(""));		
		if(input == null){
			return false;
		}		
		Resource newItem = ResourceFactory.createResource(input);
		
		//Change data
		Model newData = ModelFactory.createDefaultModel();
		StmtIterator iter = data.listStatements();
		Statement stmt;
		while(iter.hasNext()){
			
			stmt = iter.nextStatement();
			if(stmt.getSubject().getURI().equals(itemRes.getURI()))
				stmt = ResourceFactory.createStatement(newItem, stmt.getPredicate(), stmt.getObject());
			if(stmt.getObject().isResource() &&  ((Resource)stmt.getObject()).getURI().equals(itemRes.getURI()))
				stmt = ResourceFactory.createStatement(stmt.getSubject(), stmt.getPredicate(), newItem);
			newData.add(stmt);
			
		}
		
		//Create item
		try{
			VPOperations.addItem(newItem.getURI(), parentURI, newLocation.x/20, newLocation.y/20, model, newData, contextSource);
			this.refresh();
		}catch(Exception e){				
			VantagePoint.printMessage(e.toString(), Color.red);
			return false;
		}				
		
		return true;
		
	}
	
	/**
	 * Constructs a swing component tree recursively. layeredPane method should be called
	 * with root parameter EditView and the immediate child areas and items already attached
	 * to the EditView.
	 * 
	 * (TRANSITIVE PROPERTY CAUSES REDUNDANCY) Not anymore, see getImmediateChildComponents() in VPUtils
	 *  
	 * @param root 		the root container for recursion
	 * @param areas		vector holding all found areas
	 * @param items		vector holding all found items
	 */
	private void buildComponentTree(Container root, Vector<VPArea> areas, Vector<VPItem> items){
				
		if(root instanceof JLayeredPane){
			
			Component comps[] = layeredPane.getComponents();
			for(int i=0; i<comps.length; i++)
				buildComponentTree((Container)comps[i], areas, items);
			
		}
		
		if(root instanceof VPArea){
			
			VPArea area = (VPArea)root;
			
			//Attach area to its parent		
			if(area.getParentAreaURI() != null){				
				VPArea parentArea = (VPArea)VPUtils.getVisualComponentFromVector(areas, area.getParentAreaURI());
				parentArea.add(area);
			}
			
			//Attach child items
			VPItem childItem;
			Vector<String> childItemURIs = area.getChildItemURIs();
			for(int i=0; i<childItemURIs.size(); i++){			
				childItem = (VPItem)VPUtils.getVisualComponentFromVector(items, childItemURIs.elementAt(i));
				area.add(childItem);			
			}
			
			//Build child area component trees
			VPArea childArea;
			Vector<String> childAreaURIs = area.getChildAreaURIs();
			for(int i=0; i<childAreaURIs.size(); i++){			
				childArea = (VPArea)VPUtils.getVisualComponentFromVector(areas, childAreaURIs.elementAt(i));
				buildComponentTree(childArea,areas,items);
			}
			
		}		
		
	}			
	
	/**
	 * Gathers properties relating to the instance given as argument and
	 * sends them to VPGui to be displayed.
	 * @param vc	the VisualComponent whose properties are displayed
	 */
	private void displayInstanceProperties(VisualComponent vc){
		
		Statement stm;
		Individual ind = model.getIndividual(vc.getURI());		
		StmtIterator iter = ind.listProperties();	
				
		VantagePoint.printMessage("Instance attributes:");		
		
		while(iter.hasNext()){			
			
			stm = iter.nextStatement();	
			VantagePoint.printMessage(
					"("+stm.getSubject().toString()+
					", "+stm.getPredicate().toString()+
					", "+stm.getObject().toString()+")",
					new Color(0,0,200));		
			
		}		
		
	}

	/**
	 * Creates a new JDialog whose owner is the JFrame of the VPGui and
	 * attaches a panel to it that contains all the necessaty widgets
	 * for naming an area.
	 *
	 */
	private void showNameAreaDialog(){		
		
		classComboBox.removeAllItems();
		classComboBox.addItem("<default>");
		ExtendedIterator classIter = model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Area").listSubClasses();
		while(classIter.hasNext()){			
			OntClass oc = (OntClass)classIter.next();
			if(!oc.getURI().startsWith("http://www.w3.org/2002/07/owl"))
				classComboBox.addItem(oc.getURI());			
		}	
			
		nameAreaDialog.setVisible(true);
		
	}
	
	/**
	 * Processes the name area dialog OK button press. Adds a new
	 * rectangular area to the model.
	 *
	 */
	private void processAddRectangularArea(){
		
		//Get area string
		int w = rectangle.getWidth()/20;
		int h = rectangle.getHeight()/20;			
		String areaString = VPUtils.getRectangularAreaString(w, h);	
				
		//Get area instance URI
		String areaURI = namespaceEntry.getText() + localNameEntry.getText();
		
		//Get parent URI
		layeredPane.remove(rectangle);
		Component topmost = SwingUtilities.getDeepestComponentAt(layeredPane, rectangle.getX(), rectangle.getY());				
		String parentURI = null;
		if(topmost instanceof VisualComponent){				
			VisualComponent vc = (VisualComponent)topmost;				
			if(topmost instanceof VPItem){
				parentURI = vc.getParentAreaURI();
				topmost = topmost.getParent();
			}else
				parentURI = vc.getURI();				
		}		
		
		//Get new relative location
		Point newLocation = SwingUtilities.convertPoint(layeredPane, rectangle.getX(), rectangle.getY(), topmost);
				
		//Get class
		String classURI = (String)classComboBox.getSelectedItem();
		if(classURI.equals("<default>"))
			classURI = null;
		
		//Create area
		//if(!vpController.addArea(areaURI, parentURI, newLocation.x/20, newLocation.y/20, areaString, classURI))
		//	vpGui.printMessage("Could not create Area instance. Area location or URI is illegal.", null);
		
		nameAreaDialog.setVisible(false);
		nameAreaDialog = null;		
		layeredPane.repaint();
		
	}
	
	private void processAttachService(){
		
		if(attachServiceDialog == null){
			JFrame frame = (JFrame)this.getRootPane().getParent();
			attachServiceDialog = new JDialog(frame, "Attach service", true);
			attachServiceDialog.setSize(500, 100);
			attachServiceDialog.setResizable(false);
			attachServiceDialog.setLocationRelativeTo(null);
			attachServiceDialog.add(attachServicePanel);
		}
		
		DefaultComboBoxModel m = (DefaultComboBoxModel)serviceComboBox.getModel();
		m.removeAllElements();
		Vector<String> uris = VantagePoint.getGUI().getServicePanel().getServiceURIs();
		for(int i=0; i<uris.size(); i++)
			m.addElement(uris.elementAt(i));
		
		attachServiceDialog.setVisible(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		
		//vpGui.printMessage(ae.getActionCommand(), null);
		
		if(ae.getActionCommand().equals("INSTANCE_INFORMATION")){
			displayInstanceProperties((VisualComponent)selected);
		}else if(ae.getActionCommand().equals("DELETE")){
			String URI = ((VisualComponent)selected).getURI();
			try {
				VPOperations.deleteVisualComponent(URI, model, contextSource);
				this.refresh();
			} catch (Exception e) {
				VantagePoint.printMessage("Could not delete individual: "+URI);				
			}
						
		}else if(ae.getActionCommand().equals("MOVE")){
			moveEnabled = true;
			layeredPane.setCursor(new Cursor(Cursor.MOVE_CURSOR));
			oldContainer = selected.getParent();
			oldX = selected.getX();
			oldY = selected.getY();
			selected.setLocation(SwingUtilities.convertPoint(selected.getParent(), selected.getLocation(), layeredPane));
			layeredPane.add(selected, new Integer(2));
			layeredPane.setLayer(selected, 2);						
		}else if(ae.getActionCommand().equals("SEND_TO_BACK")){
			Container focusParent = selected.getParent();
			focusParent.setComponentZOrder(selected, focusParent.getComponentCount()-1);
			selected.repaint();
		}else if(ae.getActionCommand().equals("ROTATE_SOUTH")){
			VPItem focusItem = (VPItem)selected;
			try {
				VPOperations.rotateItem(focusItem.getURI(), 0, model, contextSource);
				this.refresh();
			} catch (Exception e) {
				VantagePoint.printMessage("Could not rotate item: " +focusItem.getURI());
				VantagePoint.printMessage(e.getMessage(), Color.red);				
			}											
		}else if(ae.getActionCommand().equals("ROTATE_WEST")){
			VPItem focusItem = (VPItem)selected;
			try {
				VPOperations.rotateItem(focusItem.getURI(), 1, model, contextSource);
				this.refresh();
			} catch (Exception e) {
				VantagePoint.printMessage("Could not rotate item: " +focusItem.getURI());
				VantagePoint.printMessage(e.getMessage(), Color.red);				
			}			
		}else if(ae.getActionCommand().equals("ROTATE_NORTH")){
			VPItem focusItem = (VPItem)selected;
			try {
				VPOperations.rotateItem(focusItem.getURI(), 2, model, contextSource);
				this.refresh();
			} catch (Exception e) {
				VantagePoint.printMessage("Could not rotate item: " +focusItem.getURI());
				VantagePoint.printMessage(e.getMessage(), Color.red);				
			}			
		}else if(ae.getActionCommand().equals("ROTATE_EAST")){
			VPItem focusItem = (VPItem)selected;
			try {
				VPOperations.rotateItem(focusItem.getURI(), 3, model, contextSource);
				this.refresh();
			} catch (Exception e) {
				VantagePoint.printMessage("Could not rotate item: " +focusItem.getURI());
				VantagePoint.printMessage(e.getMessage(), Color.red);				
			}			
		}else if(ae.getActionCommand().equals("ATTACH_SERVICE")){			
			processAttachService();
		}else if(ae.getActionCommand().equals("NAME_AREA_OK")){			
			processAddRectangularArea();
		}else if(ae.getActionCommand().equals("NAME_AREA_CANCEL")){			
			nameAreaDialog.setVisible(false);			
			layeredPane.remove(rectangle);
			layeredPane.repaint();
		}else if(ae.getActionCommand().equals("ATTACH")){			
			
			//add statement
			String serviceURI = (String)serviceComboBox.getSelectedItem();
			if(serviceURI == null || serviceURI.length() < 1){
				attachServiceDialog.setVisible(false);
			}
				
			VPItem item = (VPItem)selected;
			Statement s = ResourceFactory.createStatement(
					ResourceFactory.createResource(item.getURI()),
					VPUtils.PROVIDESSERVICE_PROP, 
					ResourceFactory.createResource(serviceURI));
			contextSource.modify(null, new Statement[]{s});	
			
			refresh();
			attachServiceDialog.setVisible(false);
			
		}else if(ae.getActionCommand().equals("CANCEL_ATTACH")){			
			attachServiceDialog.setVisible(false);
		}
		
		
	}
	
	/**
	 * Selects new focus component.
	 */
	public void mouseClicked(MouseEvent me) {		
		
		if(moveEnabled)return;
		
		VisualComponent vc;
		
		//previous selection off
		if(selected != null){
			vc = (VisualComponent)selected;			
			vc.setSelected(false);			
		}
		
		//acquire new focus			
		selected = SwingUtilities.getDeepestComponentAt(layeredPane, me.getX(), me.getY());
		
		//select new focus
		if(selected instanceof VisualComponent){
			vc = (VisualComponent)selected;
			vc.setSelected(true);
			//if(vpGui.getGraphicalQueryFlag())
			//	vpGui.performGraphicalQuery(vc.getURI());
		}else
			selected = null;		
		
		//Signal VPGui about selection
		//vpGui.setSelectedInstance((VisualComponent)selected, layeredPane);
		
		//Show popup menu
		if(selected != null && me.getButton() == MouseEvent.BUTTON3){			
			if(selected instanceof VPArea){
				rotateMenu.setEnabled(false);
				sendToBackMenuItem.setEnabled(false);
				attachServiceMenuItem.setEnabled(false);
			}
			else{
				rotateMenu.setEnabled(true);	
				sendToBackMenuItem.setEnabled(true);
				attachServiceMenuItem.setEnabled(true);
			}
			popupMenu.show(me.getComponent(),me.getX(), me.getY());				
		}
		
	}

	public void mouseEntered(MouseEvent me) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent me) {
		//vpGui.updateCoordinateArea(0,0);		
	}

	public void mousePressed(MouseEvent me) {

		Container parent = layeredPane.getParent();
		
		//Store rectangle area starting corner point
		if(addRectangleAreaEnabled && me.getButton() == MouseEvent.BUTTON1){
			rectangleStartCorner.setLocation(me.getX()/20*20, me.getY()/20*20);
			rectangle = new AreaRectangle();
			rectangle.setSize(0, 0);
			rectangle.setLocation(rectangleStartCorner);
			layeredPane.add(rectangle);
			layeredPane.setLayer(rectangle, 3);
		}
		
		//Drag viewport
		if(parent instanceof JViewport){
	 		grabPoint = MouseInfo.getPointerInfo().getLocation();
	 		viewportStartLocation = ((JViewport)parent).getViewPosition();
		}		
		
	}

	public void mouseReleased(MouseEvent me) {
		
		if(moveEnabled){
			
			String URI = ((VisualComponent)selected).getURI();		
			String parentURI = null;	
			Point newLocation;
			
			moveEnabled = false;
			layeredPane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));	
			
			//Focus is removed from the VPEditView to use the
			//SwingUtilities.getDeepestComponentAt() correctly			
			layeredPane.remove(selected);		
			Component topmost = SwingUtilities.getDeepestComponentAt(layeredPane, me.getX(), me.getY());
						
			//Get the URI of the topmost Area
			if(topmost instanceof VisualComponent){				
				VisualComponent vc = (VisualComponent)topmost;				
				if(topmost instanceof VPItem){
					parentURI = vc.getParentAreaURI();
					topmost = topmost.getParent();
				}else
					parentURI = vc.getURI();				
			}				
			
			newLocation = SwingUtilities.convertPoint(layeredPane, me.getX(), me.getY(), topmost);
						
			try {
				VPOperations.moveVisualComponent(URI, parentURI, newLocation.x/20, newLocation.y/20, model, contextSource);
				this.refresh();				
			} catch (Exception e) {				
				VantagePoint.printMessage("Illegal move operation:", Color.red);
				VantagePoint.printMessage(e.getMessage(), Color.black);							
				selected.setLocation(oldX, oldY);
				oldContainer.add(selected);
				this.repaint();
			}			
						
		}
		
		if(addRectangleAreaEnabled && me.getButton() == MouseEvent.BUTTON1){
						
			addRectangleAreaEnabled = false;
			layeredPane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));			
			showNameAreaDialog();
			
		}
		
	}

	public void mouseDragged(MouseEvent me) {
		
		//VantagePoint.printMessage(me.getX()/20 + ", " + me.getY()/20);
		
		//Move block
		if(moveEnabled){
			if(me.getX() >= 0 && me.getY() >= 0)
				selected.setLocation(((me.getX())/20)*20,((me.getY())/20)*20);
		}
				
		//Add rectangle area block
		if(addRectangleAreaEnabled){
						
			rectangleEndCorner.setLocation(me.getX(), me.getY());
			if(me.getX() < 0) rectangleEndCorner.x = 0;
			if(me.getY() < 0) rectangleEndCorner.y = 0;
			
			int width = 0;
			int height = 0;
			int x = 0;
			int y = 0;
			
			if(rectangleEndCorner.x >= rectangleStartCorner.x && 
					rectangleEndCorner.y >= rectangleStartCorner.y){	
				
				width = rectangleEndCorner.x - rectangleStartCorner.x;
				height = rectangleEndCorner.y - rectangleStartCorner.y;
				x = rectangleStartCorner.x;	
				y = rectangleStartCorner.y;
				
			}else if(rectangleEndCorner.x > rectangleStartCorner.x && 
					rectangleEndCorner.y < rectangleStartCorner.y){	
				
				width = rectangleEndCorner.x - rectangleStartCorner.x;
				height = rectangleStartCorner.y - rectangleEndCorner.y;
				x = rectangleStartCorner.x;	
				y = rectangleEndCorner.y;				
				
			}else if(rectangleEndCorner.x <= rectangleStartCorner.x && 
					rectangleEndCorner.y <= rectangleStartCorner.y){	
				
				width = rectangleStartCorner.x - rectangleEndCorner.x;
				height = rectangleStartCorner.y - rectangleEndCorner.y;
				x = rectangleEndCorner.x;	
				y = rectangleEndCorner.y;	
				
			}else if(rectangleEndCorner.x < rectangleStartCorner.x && 
					rectangleEndCorner.y > rectangleStartCorner.y){	
				
				width = rectangleStartCorner.x - rectangleEndCorner.x;
				height = rectangleEndCorner.y - rectangleStartCorner.y;
				x = rectangleEndCorner.x;	
				y = rectangleStartCorner.y;	
				
			}											
			
			if(width < 20 && width > -20) width = 20;
			if(height < 20 && height > -20) height = 20;			
								
			if(width % 20 != 0) width = width + 20;
			if(height % 20 != 0) height = height + 20;
			
			rectangle.setSize(width/20*20, height/20*20);
			rectangle.setLocation(x/20*20,y/20*20);
			
		}
		
		//View port move block
		if(!moveEnabled && !addRectangleAreaEnabled && layeredPane.getParent() instanceof JViewport){
						
			JViewport parentViewport = (JViewport)layeredPane.getParent();			
			
			Point cursorLocation = MouseInfo.getPointerInfo().getLocation();
			Point cursorShift = new Point(cursorLocation.x - grabPoint.x,
					cursorLocation.y - grabPoint.y);
						 
			Point newViewportLocation = new Point(
					viewportStartLocation.x - cursorShift.x,
					viewportStartLocation.y - cursorShift.y);
			
			if(newViewportLocation.x < 0) newViewportLocation.x = 0;
			if(newViewportLocation.x > layeredPane.getWidth() - parentViewport.getWidth()) 
				newViewportLocation.x = layeredPane.getWidth() - parentViewport.getWidth();
			if(newViewportLocation.y < 0) newViewportLocation.y = 0;
			if(newViewportLocation.y > layeredPane.getHeight() - parentViewport.getHeight()) 
				newViewportLocation.y = layeredPane.getHeight() - parentViewport.getHeight();			
			
			parentViewport.setViewPosition(newViewportLocation);			 
			
		}
		
	}

	/**
	 * Updates highligh.
	 */
	public void mouseMoved(MouseEvent me) {
		
		//VantagePoint.printMessage(me.getX()/20 + ", " + me.getY()/20);
		
		if(moveEnabled)return;	
				
		//acquire new highlight
		Component newHighlight = SwingUtilities.getDeepestComponentAt(layeredPane, me.getX(), me.getY());
				
		//previous highlight off
		if(!newHighlight.equals(highlighted) && highlighted instanceof VisualComponent)
			((VisualComponent)highlighted).setHighlight(false);
				
		//set current highlight on
		if(newHighlight instanceof VisualComponent)
			((VisualComponent)newHighlight).setHighlight(true);
		
		highlighted = newHighlight;
		
	}	
	
	private void initComponents(){
		
		JScrollPane scrollPane = new JScrollPane(layeredPane);		
		this.setLayout(new BorderLayout());
		this.add(scrollPane, BorderLayout.CENTER);
		
		popupMenu = new JPopupMenu();		 
	 	
		//Instance Information
		JMenuItem informationMenuItem = new JMenuItem("Instance Information...");
		informationMenuItem.setActionCommand("INSTANCE_INFORMATION");
		informationMenuItem.addActionListener(this);
	    popupMenu.add(informationMenuItem);
	    
	    //Move
	    JMenuItem moveMenuItem = new JMenuItem("Move");
	    moveMenuItem.setActionCommand("MOVE");
	    moveMenuItem.addActionListener(this);
	    popupMenu.add(moveMenuItem);
			    
	    //Delete
	    JMenuItem deleteMenuItem = new JMenuItem("Delete");
	    deleteMenuItem.setActionCommand("DELETE");
	    deleteMenuItem.addActionListener(this);
	    popupMenu.add(deleteMenuItem);
	    
	    //Rotate
	    rotateMenu = new JMenu("Rotate");
	    popupMenu.add(rotateMenu);    
	    JMenuItem rotateNorth = new JMenuItem("North", new ImageIcon("graphic/arrow_north.png"));
	    rotateNorth.setActionCommand("ROTATE_NORTH");
	    rotateNorth.addActionListener(this);
	    rotateMenu.add(rotateNorth);   
	    JMenuItem rotateEast = new JMenuItem("East", new ImageIcon("graphic/arrow_east.png"));
	    rotateEast.setActionCommand("ROTATE_EAST");
	    rotateEast.addActionListener(this);
	    rotateMenu.add( rotateEast);	    
	    JMenuItem rotateSouth = new JMenuItem("South", new ImageIcon("graphic/arrow_south.png"));
		rotateSouth.setActionCommand("ROTATE_SOUTH");
		rotateSouth.addActionListener(this);
		rotateMenu.add(rotateSouth);	
		JMenuItem rotateWest = new JMenuItem("West", new ImageIcon("graphic/arrow_west.png"));
		rotateWest.setActionCommand("ROTATE_WEST");
		rotateWest.addActionListener(this);
		rotateMenu.add(rotateWest);
		
		//Send to back
		sendToBackMenuItem = new JMenuItem("Send to back");
		sendToBackMenuItem.setActionCommand("SEND_TO_BACK");
		sendToBackMenuItem.addActionListener(this);
	    popupMenu.add(sendToBackMenuItem);
	    
	    //Attach service
		attachServiceMenuItem = new JMenuItem("Attach service...");
		attachServiceMenuItem.setActionCommand("ATTACH_SERVICE");
		attachServiceMenuItem.addActionListener(this);
	    popupMenu.add(attachServiceMenuItem);
		
	    //Name area dialog
	    //JFrame mainWindow = (JFrame)layeredPane.getRootPane().getParent();
		//nameAreaDialog = new JDialog(mainWindow, "Name Area", true);				
		//nameAreaDialog.setSize(400, 190);
		//nameAreaDialog.setLocationRelativeTo(null);
		//nameAreaDialog.setResizable(false);			
		
		/*nameAreaDialog.addWindowListener(new WindowAdapter(){			
			public void windowClosing(WindowEvent we){				
				nameAreaDialog = null;
				rectangle.setSize(0, 0);
				rectangle.repaint();
				rectangle = null;				
			}			
		});*/
		
		//namespaceEntry.setText(model.getNsURIPrefix(""));		
	    
	    //Name Area Panel
	    nameAreaPanel = new JPanel(null);
	    nameAreaPanel.setSize(400,190);
	    //nameAreaDialog.add(nameAreaPanel);
	    
	    JLabel namespaceLabel = new JLabel("Namespace:");
	    namespaceLabel.setBounds(10,10,70,20);
	    nameAreaPanel.add(namespaceLabel);
	    
	    namespaceEntry = new JTextField();
	    namespaceEntry.setBounds(75,10,310,20);
	    namespaceEntry.addCaretListener(new CaretListener(){
			public void caretUpdate(CaretEvent ce){				
				if(namespaceEntry.getText().length() < 1 ||
						localNameEntry.getText().length() < 1){
					nameAreaOKButton.setEnabled(false);
				}else{
					nameAreaOKButton.setEnabled(true);
				}
			}
		});
	    nameAreaPanel.add(namespaceEntry);
	    
	    JLabel localNameLabel = new JLabel("Local Name:");
	    localNameLabel.setBounds(10,40,70,20);
	    nameAreaPanel.add(localNameLabel);
	    
	    localNameEntry = new JTextField();
	    localNameEntry.setBounds(75,40,310,20);
	    localNameEntry.addCaretListener(new CaretListener(){
			public void caretUpdate(CaretEvent ce){
				if(namespaceEntry.getText().length() < 1 ||
						localNameEntry.getText().length() < 1){
					nameAreaOKButton.setEnabled(false);
				}else{
					nameAreaOKButton.setEnabled(true);
				}
			}
		});
	    nameAreaPanel.add(localNameEntry);
	    
	    JLabel classLabel = new JLabel("Class:");
	    classLabel.setBounds(10,70,70,20);
	    nameAreaPanel.add(classLabel);
	    
	    classComboBox = new JComboBox();
	    classComboBox.setBounds(75,70,310,20);
	    classComboBox.addItem("<default>");
	    nameAreaPanel.add(classComboBox);
	    
	    nameAreaOKButton = new JButton("OK");
	    nameAreaOKButton.setActionCommand("NAME_AREA_OK");
	    nameAreaOKButton.addActionListener(this);
	    nameAreaOKButton.setBounds(235, 130, 70, 25);
	    nameAreaOKButton.setEnabled(false);
	    nameAreaPanel.add(nameAreaOKButton);
	    
	    nameAreaCancelButton = new JButton("Cancel");
	    nameAreaCancelButton.setActionCommand("NAME_AREA_CANCEL");
	    nameAreaCancelButton.addActionListener(this);
	    nameAreaCancelButton.setBounds(315, 130, 70, 25);
	    nameAreaPanel.add(nameAreaCancelButton);
	    
	    attachServicePanel = new JPanel();
	    serviceComboBox = new javax.swing.JComboBox();
        cancelAttachServiceButton = new javax.swing.JButton();
        attachServiceButton = new javax.swing.JButton();

        serviceComboBox.setModel(new javax.swing.DefaultComboBoxModel());

        cancelAttachServiceButton.setText("Cancel");
        cancelAttachServiceButton.addActionListener(this);
        cancelAttachServiceButton.setActionCommand("CANCEL_ATTACH");

        attachServiceButton.setText("Attach");
        attachServiceButton.addActionListener(this);
        attachServiceButton.setActionCommand("ATTACH");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(attachServicePanel);
        attachServicePanel.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(serviceComboBox, 0, 258, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(attachServiceButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelAttachServiceButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(serviceComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cancelAttachServiceButton)
                    .addComponent(attachServiceButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
	    
	    
	}
	
	private JDialog attachServiceDialog;
	private JPanel attachServicePanel;
	private JButton attachServiceButton;
	private JButton cancelAttachServiceButton;
	private JComboBox serviceComboBox;
	private JPopupMenu popupMenu;
	private JMenu rotateMenu;
	private JMenuItem sendToBackMenuItem;
	private JMenuItem attachServiceMenuItem;
	private JDialog nameAreaDialog;
	private JPanel nameAreaPanel;
	private JTextField namespaceEntry;
	private JTextField localNameEntry;
	private JButton nameAreaOKButton;
	private JButton nameAreaCancelButton;
	private JComboBox classComboBox;

}
