package vtt.vp.view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

import vtt.vp.utils.VPUtils;

public class EditViewArea extends VPArea{
	
	//Removes the serializable warning
	private static final long serialVersionUID = 111;
		
	/**
	 * Boolean matrix that defines the shape and size of this area
	 */
	private boolean areaMap[][];
	
	/**
	 * Integer matrix that defines the tile style in each square
	 */
	private int tileMap[][];
	
	//Area square colors			
	private final Color squareFill = new Color(180,200,255);
	private final Color squareLine = new Color(170,190,255);
	private final Color squareEdge = new Color(0,0,0);
	private final Color squareEdgeH = new Color(100,100,100);
	private final Color squareEdgeS = new Color(80,80,255);
	private final Color squareEdgeHS = Color.blue;
	
	private boolean highlighted = false;
	private boolean selected = false;
		
	/**
	 * Class constructor that initializes the area visualization. 
	 * @param URI	the URI of this EditViewArea
	 * @param areaMatrixString	the area matrix in a String representation
	 */
	public EditViewArea(String URI, String areaMatrixString) {
		
		super(URI);			
		areaMap = VPUtils.parseAreaString(areaMatrixString);		
		tileMap = VPUtils.convertToTileCodeMap(areaMap);
				
		//Initialize size		
		this.setSize(areaMap[0].length*20,areaMap.length*20);
		
	}
	
	/**
	 * Implements VisualComponent.setHighlight
	 */
	public void setHighlight(boolean b){
		highlighted = b;
		this.repaint();
	}
	
	/**
	 * Implements VisualComponent.isHighlighted
	 */
	public boolean isHighlighted(){
		return highlighted;
	}
	
	/**
	 * Implements VisualComponent.setSelected
	 */
	public void setSelected(boolean b){
		selected = b;
		this.repaint();
	}
	
	/**
	 * Implements VisualComponent.isSelected
	 */
	public boolean isSelected(){
		return selected;
	}
	
	/**
	 * Tests whether the given point is inside this area.
	 * The point is given in the coordinate system of this
	 * area and is in pixel coordinates.
	 */
	public boolean contains(int x, int y){
		
		boolean hit = false;
		
		if(x<0 || y<0 || x>this.getWidth()-1 || y>this.getHeight()-1)
			return false;
				
		try{
			hit = areaMap[y/20][x/20];
		}catch(ArrayIndexOutOfBoundsException e){
			return false;
		}
		
		return hit;
	}
	
	public boolean contains(Point p){
		return contains(p.x,p.y);
	}
	
	/**
	 * Paint area
	 */
	public void paintComponent(Graphics g){
						
		Color fill = squareFill;
		Color line = squareLine;
		Color edge;
		
		if(highlighted && selected){			
			edge = squareEdgeHS;
		}else if(selected){			
			edge = squareEdgeS;	
		}else if(highlighted){			
			edge = squareEdgeH;
		}else{			
			edge = squareEdge;
		}
		
		for(int y=0 ; y<tileMap.length ; y++){
			for(int x=0 ; x<tileMap[0].length ; x++){								
				
				switch(tileMap[y][x]){
				
				case 0:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawRect(x*20, y*20, 19, 19);
					break;
					
				case 1:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.setColor(edge);
					g.drawLine(x*20, y*20+19, x*20+19, y*20+19);
					if(selected || highlighted)
						g.drawLine(x*20, y*20+18, x*20+19, y*20+18);
					break;
					
				case 2:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.setColor(edge);
					g.drawLine(x*20, y*20, x*20, y*20+19);
					if(selected || highlighted)
						g.drawLine(x*20+1, y*20, x*20+1, y*20+19);
					break;
					
				case 3:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.setColor(edge);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					if(selected || highlighted)
						g.drawLine(x*20, y*20+1, x*20+19, y*20+1);
					break;
					
				case 4:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.setColor(edge);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					if(selected || highlighted)
						g.drawLine(x*20+18, y*20, x*20+18, y*20+19);
					break;
					
				case 5:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);					
					g.setColor(edge);
					g.drawLine(x*20, y*20, x*20, y*20+19);
					g.drawLine(x*20, y*20+19, x*20+19, y*20+19);
					if(selected || highlighted){
						g.drawLine(x*20+1, y*20, x*20+1, y*20+18);
						g.drawLine(x*20+1, y*20+18, x*20+19, y*20+18);
					}
					break;
					
				case 6:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);					
					g.setColor(edge);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					if(selected || highlighted){
						g.drawLine(x*20+1, y*20+19, x*20+1, y*20+1);
						g.drawLine(x*20+1, y*20+1, x*20+19, y*20+1);
					}
					break;
					
				case 7:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.drawLine(x*20, y*20+19, x*20, y*20);					
					g.setColor(edge);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					if(selected || highlighted){
						g.drawLine(x*20, y*20+1, x*20+18, y*20+1);
						g.drawLine(x*20+18, y*20+1, x*20+18, y*20+19);
					}
					break;
					
				case 8:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20, y*20, x*20+19, y*20);					
					g.setColor(edge);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					if(selected || highlighted){
						g.drawLine(x*20+18, y*20, x*20+18, y*20+18);
						g.drawLine(x*20+18, y*20+18, x*20, y*20+18);
					}
					break;
					
				case 9:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20, x*20+19, y*20);										
					g.setColor(edge);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					if(selected || highlighted){
						g.drawLine(x*20+18, y*20, x*20+18, y*20+18);
						g.drawLine(x*20+18, y*20+18, x*20+1, y*20+18);
						g.drawLine(x*20+1, y*20+18, x*20+1, y*20);
					}
					break;
					
				case 10:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);										
					g.setColor(edge);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					if(selected || highlighted){
						g.drawLine(x*20+19, y*20+18, x*20+1, y*20+18);
						g.drawLine(x*20+1, y*20+18, x*20+1, y*20+1);
						g.drawLine(x*20+1, y*20+1, x*20+19, y*20+1);
					}
					break;
					
				case 11:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);										
					g.setColor(edge);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					if(selected || highlighted){
						g.drawLine(x*20+1, y*20+19, x*20+1, y*20+1);
						g.drawLine(x*20+1, y*20+1, x*20+18, y*20+1);
						g.drawLine(x*20+18, y*20+1, x*20+18, y*20+19);
					}
					break;
					
				case 12:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20+19, x*20, y*20);										
					g.setColor(edge);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					if(selected || highlighted){
						g.drawLine(x*20, y*20+1, x*20+18, y*20+1);
						g.drawLine(x*20+18, y*20+1, x*20+18, y*20+18);
						g.drawLine(x*20+18, y*20+18, x*20, y*20+18);
					}
					break;
					
				case 13:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);					
					g.setColor(edge);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					if(selected || highlighted){
						g.drawLine(x*20+19, y*20+18, x*20, y*20+18);
						g.drawLine(x*20, y*20+1, x*20+19, y*20+1);
					}
					break;
					
				case 14:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);
					g.setColor(line);
					g.drawLine(x*20, y*20, x*20+19, y*20);
					g.drawLine(x*20+19, y*20+19, x*20, y*20+19);					
					g.setColor(edge);
					g.drawLine(x*20, y*20+19, x*20, y*20);
					g.drawLine(x*20+19, y*20, x*20+19, y*20+19);
					if(selected || highlighted){
						g.drawLine(x*20+1, y*20+19, x*20+1, y*20);
						g.drawLine(x*20+18, y*20, x*20+18, y*20+19);
					}
					break;
					
				case 15:
					g.setColor(fill);
					g.fillRect(x*20+1, y*20+1, 18, 18);										
					g.setColor(edge);
					g.drawRect(x*20, y*20, 19, 19);	
					if(selected || highlighted)
						g.drawRect(x*20+1, y*20+1, 18, 18);	
					break;
				
				}
				
			}
		}			
		
	}
	
}
