package vtt.vp.view;

import java.awt.Graphics;
import java.awt.Color;

public class EditViewItem extends VPItem {

	//Removes the serializable warning
	private static final long serialVersionUID = 333;
		
	private boolean highlighted = false;
	private boolean selected = false;	
	private int direction;
	
	private final Color fillColor = new Color(255,255,255,180);
	private final Color fillColorH = new Color(255,255,255,255);
	private final Color fillColorS = new Color(100,100,255,180);
	private final Color fillColorHS = new Color(100,100,255,255);
	
	/**
	 * Class constructor initializes the size of the item.
	 * @param URI			the URI of this item
	 * @param gridWidth		width in grid squares
	 * @param gridHeight	height in grid squares
	 */
	public EditViewItem(String URI, int gridWidth, int gridHeight){
		
		super(URI);		
		
		direction = VPItem.ITEM_SOUTH;
		
		//Initialize size		
		this.setSize(gridWidth*20, gridHeight*20);
		
	}
	
	public boolean isHighlighted(){
		return highlighted;
	}
	
	public void setHighlight(boolean b){
		highlighted = b;
		this.repaint();
	}
	
	public boolean isSelected(){
		return selected;
	}
	
	public void setSelected(boolean b){
		selected = b;
		this.repaint();
	}
	
	/**
	 * Change item direction. Direction change affects to the size
	 * only if the direction changes from north or south to west or east
	 * and vice versa.
	 * @param d
	 */
	public void setDirection(int d){		
		if(d < 0 || d > 3)return;		
		if(((direction == VPItem.ITEM_SOUTH 
				|| direction == VPItem.ITEM_NORTH) 
				&& (d == VPItem.ITEM_WEST || d == VPItem.ITEM_EAST)) 
				|| ((direction == VPItem.ITEM_WEST 
						|| direction == VPItem.ITEM_EAST) 
						&& (d == VPItem.ITEM_SOUTH || d == VPItem.ITEM_NORTH)))
			this.setSize(this.getHeight(), this.getWidth());
		direction = d;
	}
	
	public int getDirection(){		
		return direction;
	}
	
	public void paintComponent(Graphics g){
				
		//Set fill color
		if(selected && highlighted) g.setColor(fillColorHS);
		else if(selected)g.setColor(fillColorS);
		else if(highlighted) g.setColor(fillColorH);
		else g.setColor(fillColor);
		
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
		g.setColor(Color.black);
		g.drawRect(0, 0, this.getWidth()-1, this.getHeight()-1);		
						
		if(this.getDirection() == 0){
			g.drawChars("\u2193".toCharArray(), 0, 1, 6, 12);
		}else if(this.getDirection() == 1){			
			g.drawChars("\u2190".toCharArray(), 0, 1, 6, 12);
		}else if(this.getDirection() == 2){			
			g.drawChars("\u2191".toCharArray(), 0, 1, 6, 12);
		}else if(this.getDirection() == 3){			
			g.drawChars("\u2192".toCharArray(), 0, 1, 6, 12);
		}	
		
		g.drawChars(this.getLocalName().toCharArray(), 0, this.getLocalName().length(), 22, 14);
				
	}	
	
}
