package vtt.vp.view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.BufferedImage;

import vtt.vp.library.IconLibrary;
import vtt.vp.utils.VPUtils;

public class IsometricArea extends VPArea{

	static final long serialVersionUID = 123;
	
	/**
	 * Boolean matrix that defines the shape and size of this area
	 */
	private boolean areaMap[][];
	
	/**
	 * Integer matrix that defines the tile style in each square
	 */
	private int tileMap[][];
	
	/**
	 * Area texture images fetched from icon library
	 */
	private BufferedImage isometricTiles[];
	
	/**
	 * The image created for the area from isometric tiles
	 */
	private BufferedImage areaImage;		
	
	private String texture;					//The texture of the Area
	
	private boolean highlighted = false;
	private boolean selected = false;
	
	private int gridWidth;
	private int gridHeight;
	
	private int xOffset;
	
	private int gridX;
	private int gridY;
	
	private final Color squareEdgeH = new Color(120,120,120);
	private final Color squareEdgeS = Color.blue;	
	
	/**
	 * Class constructor.
	 * 
	 * @param URI
	 * @param areaMatrixString
	 */
	public IsometricArea(String URI, String areaMatrixString, String txt){
		
		super(URI);					
		areaMap = VPUtils.parseAreaString(areaMatrixString);		
		tileMap = VPUtils.convertToTileCodeMap(areaMap);
		
		if(txt == null)
			texture = IconLibrary.AT_DEFAULT;
		else
			texture = txt;
		
		isometricTiles = IconLibrary.getIconLibraryObject().getAreaTexture(texture);		
		
		if(isometricTiles == null)
			isometricTiles = IconLibrary.getIconLibraryObject().getAreaTexture(IconLibrary.AT_DEFAULT);
		
		//Initialize size		
		gridWidth = areaMap[0].length;
		gridHeight = areaMap.length;
		
		this.setSize((gridWidth+gridHeight)*16,(gridWidth+gridHeight)*8);
		xOffset = gridHeight*16-16;
								
		createAreaImage();
		
	}
	
	public void setGridX(int x){
		gridX = x;
	}
	
	public int getGridX(){
		return gridX;
	}

	public void setGridY(int y){
		gridY = y;		
	}
	
	public int getGridY(){
		return gridY;
	}
	
	public void setGridWidth(int w){
		gridWidth = w;
	}
	
	public int getGridWidth(){
		return gridWidth;
	}
	
	public void setGridHeight(int h){
		gridHeight = h;
	}
	
	public int getGridHeight(){
		return gridHeight;
	}
	
	public String getTexture(){
		return this.texture;
	}	
	
	/**
	 * Implements VisualComponent.setHighlight
	 */
	public void setHighlight(boolean b){
		highlighted = b;
		this.repaint();
	}
	
	/**
	 * Implements VisualComponent.isHighlighted
	 */
	public boolean isHighlighted(){
		return highlighted;
	}
	
	/**
	 * Implements VisualComponent.setSelected
	 */
	public void setSelected(boolean b){
		selected = b;
		this.repaint();
	}
	
	/**
	 * Implements VisualComponent.isSelected
	 */
	public boolean isSelected(){
		return selected;
	}
	
	public boolean contains(int x, int y){
		
		if(x < 0 || y < 0 || x > this.getWidth()-1 || y > this.getHeight()-1)
			return false;		
					
		Point gridPoint = VPUtils.isometricPointToGridPoint(x, y, new Point(xOffset+16,0));
				
		try{
			return(areaMap[gridPoint.y][gridPoint.x]);
		}catch(ArrayIndexOutOfBoundsException aiobe){
			return false;
		}	
		
	}
	
	public boolean contains(Point p){
		return contains(p.x, p.y);
	}
	
	public void paintComponent(Graphics g){		
				
		g.drawImage(areaImage, 0, 0, null);
		
		if(selected || highlighted){
		
			Color edgeColor = null;
			
			if(selected && highlighted)
				edgeColor = squareEdgeS;
			else if(selected)
				edgeColor = squareEdgeS;
			else if(highlighted)
				edgeColor = squareEdgeH;		
				
			g.setColor(edgeColor);
			
			int tileX, tileY;		
			for(int y=0 ; y<gridHeight ; y++){
				for(int x=0 ; x<gridWidth ; x++){
					
					tileX = 16*(x-y)+xOffset+1;
					tileY = 8*(x+y);			
					
					switch(tileMap[y][x]){
													
						case 1:					
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							break;
							
						case 2:					
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							break;
							
						case 3:					
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							break;
							
						case 4:	
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;
							
						case 5:					
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);					
							break;
							
						case 6:					
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							break;
							
						case 7:					
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;
							
						case 8:					
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							break;
							
						case 9:					
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;
							
						case 10:					
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							break;
							
						case 11:					
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;
							
						case 12:
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;
							
						case 13:
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);		
							break;
							
						case 14:
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;
							
						case 15:					
							g.drawLine(tileX-2, tileY+7, tileX+15, tileY+15);
							g.drawLine(tileX+15, tileY-1, tileX-2, tileY+7);
							g.drawLine(tileX+31, tileY+7, tileX+14, tileY-1);
							g.drawLine(tileX+14, tileY+15, tileX+31, tileY+7);
							break;				
					}					
				}					
			}			
		}
		
	}
	
	/**
	 * Builds the area images from isometric tile images.
	 * All the highlight images are also build at the same time
	 * by rescaling the original image.
	 *
	 */
	private void createAreaImage(){		
		
		areaImage = new BufferedImage(this.getWidth(),this.getHeight(),BufferedImage.TYPE_INT_ARGB);
		Graphics areaG = areaImage.getGraphics();			
		int tileIndex;		
		for(int j=0 ; j<gridHeight ; j++){
			for(int i=0 ; i<gridWidth ; i++){
				
				tileIndex = tileMap[j][i];				
				if(tileIndex!=-1)					
					areaG.drawImage(isometricTiles[tileIndex],
							16*(i-j)+xOffset+1,//adjustment
							8*(i+j),								
							null);
				
			}					
		}		
		areaG.dispose();		
		
	}
	
}
