package vtt.vp.view;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.image.DataBuffer;
import java.awt.image.Raster;
import java.awt.image.RescaleOp;
import java.awt.image.SampleModel;

import vtt.vp.library.IconSet;
import vtt.vp.view.VPItem;

public class IsometricItem extends VPItem {

	//Removes the serializable warning
	private static final long serialVersionUID = 333;
		
	private boolean highlighted = false;
	private boolean selected = false;
	
	private int gridX;
	private int gridY;
	private int gridWidth;
	private int gridHeight;
	private int direction;
	private int xOffset;
	private int yOffset;
	
	private IconSet iconSet;
	
	private boolean hitMapSouth[][];
	private boolean hitMapWest[][];
	private boolean hitMapNorth[][];
	private boolean hitMapEast[][];	
	
	/**
	 * Class constructor initializes the size of the item.
	 * @param URI			the URI of this item
	 * @param gridWidth		width in grid squares
	 * @param gridHeight	height in grid squares
	 */
	public IsometricItem(String URI, IconSet is){
		
		super(URI);
		
		iconSet = is;
		
		gridWidth = is.getGridWidth();
		gridHeight = is.getGridHeight();
		
		direction = VPItem.ITEM_SOUTH;
		
		yOffset = iconSet.getImageHeight()-((gridWidth+gridHeight)*8); //always the same
		xOffset = gridHeight*16; //changes when rotated
		
		//Initialize size stays always the same		
		this.setSize(is.getImageWidth(), iconSet.getImageHeight());
		
		//Initialize hit detection maps for each image
		initializeHitMaps(iconSet);
		
	}
	
	public void setGridX(int x){
		gridX = x;
	}
	
	public int getGridX(){
		return gridX;
	}

	public void setGridY(int y){
		gridY = y;		
	}
	
	public int getGridY(){
		return gridY;
	}
	
	public void setGridWidth(int w){
		gridWidth = w;
	}
	
	public int getGridWidth(){
		return gridWidth;
	}
	
	public void setGridHeight(int h){
		gridHeight = h;
	}
	
	public int getGridHeight(){
		return gridHeight;
	}
	
	public boolean isHighlighted(){
		return highlighted;
	}
	
	public void setHighlight(boolean b){
		highlighted = b;
		this.repaint();
	}
	
	public boolean isSelected(){
		return selected;
	}
	
	public void setSelected(boolean b){
		selected = b;
		this.repaint();
	}
	
	public void setDirection(int d){
		if(d < 0 || d > 3)return;
		direction = d;
		if(direction == VPItem.ITEM_SOUTH || direction == VPItem.ITEM_NORTH) 
			xOffset = gridHeight*16;
		if(direction == VPItem.ITEM_WEST || direction == VPItem.ITEM_EAST) 
			xOffset = gridWidth*16;
	}
	
	public int getDirection(){
		return direction;
	}
	
	public int getXOffset(){
		return xOffset;
	}
	
	public int getYOffset(){
		return yOffset;
	}
	
	public boolean contains(int x, int y){
		
		if(x < 0 || y < 0 || x > this.getWidth()-1 || y > this.getHeight()-1)
			return false;
		
		if(direction == VPItem.ITEM_SOUTH){			
			try{
				return hitMapSouth[y][x];
			}catch(ArrayIndexOutOfBoundsException aiobe){
				System.out.println(x + " " + y);
				return false;
			}			
		}
		
		if(direction == VPItem.ITEM_WEST){
			try{
				return hitMapWest[y][x];
			}catch(ArrayIndexOutOfBoundsException aiobe){				
				return false;
			}		
		}
		
		if(direction == VPItem.ITEM_NORTH){
			try{
				return hitMapNorth[y][x];
			}catch(ArrayIndexOutOfBoundsException aiobe){				
				return false;
			}		
		}
		
		if(direction == VPItem.ITEM_EAST){
			try{
				return hitMapEast[y][x];
			}catch(ArrayIndexOutOfBoundsException aiobe){				
				return false;
			}		
		}		
		
		return false;
		
	}
	
	public boolean contains(Point p){
		return contains(p.x, p.y);
	}
		
	public void paintComponent(Graphics g){	
		
		Graphics2D g2 = null;		
		RescaleOp blueOp = null;
		RescaleOp lightOp = null;
		RescaleOp blueLightOp = null;
		
		if(selected || highlighted){
			g2 = (Graphics2D)g;		
			blueOp = new RescaleOp(new float[]{0.5f,0.5f,2f,1f}, new float[]{0,0,0,0},null);
			lightOp = new RescaleOp(new float[]{1.2f,1.2f,1.2f,1f}, new float[]{0,0,0,0},null);
			blueLightOp = new RescaleOp(new float[]{0.7f,0.7f,2f,1f}, new float[]{0,0,0,0},null);
		}		
		
		if(direction == VPItem.ITEM_SOUTH){	
			if(selected && highlighted)
				g2.drawImage(iconSet.getImageSouth(), blueLightOp, 0, 0);
			else if(selected)				
				g2.drawImage(iconSet.getImageSouth(), blueOp, 0, 0);
			else if(highlighted)
				g2.drawImage(iconSet.getImageSouth(), lightOp, 0, 0);
			else
				g.drawImage(iconSet.getImageSouth(), 0, 0, null);			
		}
		
		if(direction == VPItem.ITEM_WEST){
			if(selected && highlighted)
				g2.drawImage(iconSet.getImageWest(), blueLightOp, 0, 0);
			else if(selected)				
				g2.drawImage(iconSet.getImageWest(), blueOp, 0, 0);
			else if(highlighted)
				g2.drawImage(iconSet.getImageWest(), lightOp, 0, 0);
			else
				g.drawImage(iconSet.getImageWest(), 0, 0, null);
		}
		
		if(direction == VPItem.ITEM_NORTH){
			if(selected && highlighted)
				g2.drawImage(iconSet.getImageNorth(), blueLightOp, 0, 0);
			else if(selected)				
				g2.drawImage(iconSet.getImageNorth(), blueOp, 0, 0);
			else if(highlighted)
				g2.drawImage(iconSet.getImageNorth(), lightOp, 0, 0);
			else
				g.drawImage(iconSet.getImageNorth(), 0, 0, null);
		}
		
		if(direction == VPItem.ITEM_EAST){
			if(selected && highlighted)
				g2.drawImage(iconSet.getImageEast(), blueLightOp, 0, 0);
			else if(selected)				
				g2.drawImage(iconSet.getImageEast(), blueOp, 0, 0);
			else if(highlighted)
				g2.drawImage(iconSet.getImageEast(), lightOp, 0, 0);
			else
				g.drawImage(iconSet.getImageEast(), 0, 0, null);
		}
		
	}
	
	/**
	 * Initializes hit maps (2D boolean arrays) according to IsometricItem images
	 * from different compass directions. These hit maps are then used to 
	 * override the JComponent.contains method.
	 * 
	 * @param is	the IconSet according to which the hit maps are initialized
	 */
	private void initializeHitMaps(IconSet is){
		
		Raster rasterSouth = is.getImageSouth().getData();
		Raster rasterWest = is.getImageWest().getData();
		Raster rasterNorth = is.getImageNorth().getData();
		Raster rasterEast = is.getImageEast().getData();
		
		SampleModel smSouth = rasterSouth.getSampleModel();
		SampleModel smWest = rasterWest.getSampleModel();
		SampleModel smNorth = rasterNorth.getSampleModel();
		SampleModel smEast = rasterEast.getSampleModel();
		
		DataBuffer dbSouth = rasterSouth.getDataBuffer();
		DataBuffer dbWest = rasterWest.getDataBuffer();
		DataBuffer dbNorth = rasterNorth.getDataBuffer();
		DataBuffer dbEast = rasterEast.getDataBuffer();
		
		int pixel[] = new int[4];
		
		hitMapSouth = new boolean[iconSet.getImageHeight()][iconSet.getImageWidth()];
		hitMapWest = new boolean[iconSet.getImageHeight()][iconSet.getImageWidth()];
		hitMapNorth = new boolean[iconSet.getImageHeight()][iconSet.getImageWidth()];
		hitMapEast = new boolean[iconSet.getImageHeight()][iconSet.getImageWidth()];
				
		for(int y=0; y<iconSet.getImageHeight(); y++){
			for(int x=0; x<iconSet.getImageWidth(); x++){
				
				smSouth.getPixel(x, y, pixel, dbSouth);
				if(pixel[3] > 0)hitMapSouth[y][x] = true;
				else hitMapSouth[y][x] = false;
				
				smWest.getPixel(x, y, pixel, dbWest);
				if(pixel[3] > 0)hitMapWest[y][x] = true;
				else hitMapWest[y][x] = false;
				
				smNorth.getPixel(x, y, pixel, dbNorth);
				if(pixel[3] > 0)hitMapNorth[y][x] = true;
				else hitMapNorth[y][x] = false;
				
				smEast.getPixel(x, y, pixel, dbEast);
				if(pixel[3] > 0)hitMapEast[y][x] = true;
				else hitMapEast[y][x] = false;
				
			}			
		}	
		
	}
	
}
