package vtt.vp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;

import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;

import vtt.vp.context.source.VPContextSource;

public class TextView extends VPView{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
		
	private VPContextSource contextSource;
	private OntModel model;	
	
	public TextView(VPContextSource vpcs){
		
		super();	
		initComponents();		
		contextSource = vpcs;
		refresh();	
	    
	}

	public void refresh() {
		
		model = null;
		model = ModelFactory.createOntologyModel();	
		InputStream rdfStream = contextSource.getRDFDataStream();
		model.read(rdfStream, "");
		
		try {
			rdfStream.close();
		} catch (IOException e) {
			System.err.println("Error closing RDF stream");
			e.printStackTrace();
		}
		
		StringWriter writer = new StringWriter();
		model.write(writer);			
		writeRDF(writer.toString());		
		
		try {
			writer.close();
		} catch (IOException e) {
			System.err.println("Error closing RDF writer");
			e.printStackTrace();
		}		
		
	}
	
	public VPContextSource getContextSource(){
		return contextSource;
	}
	
	private void writeRDF(String rdfData){
		
		SimpleAttributeSet normal = new SimpleAttributeSet();
		StyleConstants.setForeground(normal, Color.black);
	    StyleConstants.setFontFamily(normal, "Helvetica");
	    StyleConstants.setFontSize(normal, 12);
	    
	    SimpleAttributeSet element = new SimpleAttributeSet();
		StyleConstants.setForeground(element, new Color(100,10,100));
	    StyleConstants.setFontFamily(element, "Helvetica");
	    StyleConstants.setFontSize(element, 14);	
	    StyleConstants.setBold(element, true);
	    
	    SimpleAttributeSet quote = new SimpleAttributeSet();
		StyleConstants.setForeground(quote, Color.blue);
	    StyleConstants.setFontFamily(quote, "Helvetica");
	    StyleConstants.setFontSize(quote, 12);	
	    
	    SimpleAttributeSet value = new SimpleAttributeSet();
		StyleConstants.setForeground(value, Color.black);
	    StyleConstants.setFontFamily(value, "Helvetica");
	    StyleConstants.setFontSize(value, 12);
	    StyleConstants.setBold(value, true);	    
		
		SimpleAttributeSet attrs;
		int documentLength;
		Document document = textPane.getDocument();
		char c;
		boolean quoteOn = false;
		boolean elementOn = false;
		boolean valueOn = false;
		
		for(int i=0; i<rdfData.length(); i++){						
			
			c = rdfData.charAt(i);			
						
			if(c == ' ' && elementOn)
				elementOn = false;
			
			if(c == '>')
				elementOn = false;				
			
			if(c == '<')
				valueOn = false;
			
			if(c == '"')
				quoteOn = !quoteOn;
			
			if(quoteOn)
				attrs = quote;
			else if(elementOn)
				attrs = element;
			else if(valueOn)
				attrs = value;
			else
				attrs = normal;
			
			if(c == '"')
				attrs = quote;
			
			documentLength = document.getLength();
			
			try {
				textPane.getDocument().insertString(documentLength, Character.toString(c), attrs);
			} catch (BadLocationException e) {
				System.err.println("Error writing RDF to text pane");
				e.printStackTrace();
			}
			
			if(c == '>')
				valueOn = true;
			
			if(c == '<')
				elementOn = true;	
			
		}
		
	}
	
	private void initComponents() {
		this.setLayout(new BorderLayout());		
		scrollPane = new JScrollPane();
		this.add(scrollPane, BorderLayout.CENTER);
		textPane = new JTextPane();
        scrollPane.setViewportView(textPane);            
    }  	
    
    
    // Variables declaration - do not modify     
    private JScrollPane scrollPane;
	private JTextPane textPane;
    // End of variables declaration

}
