package vtt.vp.view;

import java.util.Vector;

import javax.swing.JComponent;

/**
 * Abstract base class for VantagePoint Areas. This class implements basic methods
 * that all Areas must have. The implementation of a more defined Area is left
 * to the subclasses of this abstract class.
 * @author tptoni
 *
 */
public abstract class VPArea extends JComponent implements VisualComponent {

	private String URI;						//URI of this Area instance			
	private String parentAreaURI;			//URI of the parent area	
	private Vector<String> childItemURIs;	//URIs of the items inside this area
	private Vector<String> childAreaURIs;	//URIs of the areas inside this area
	
	/**
	 * Class constructor. URI is the only mandatory property of an Area. 
	 * @param URI
	 */
	public VPArea(String URI){		
		super();
		this.URI = URI;			
		this.childAreaURIs = new Vector<String>();
		this.childItemURIs = new Vector<String>();		
	}
	
	/**
	 * Converts area to String representation
	 */
	public String toString(){	
		return URI;		
	}	
	
	//
	//VisualComponent interface methods
	//
	
	public String getURI(){
		return this.URI;
	}
	
	public String getLocalName(){
		
		int index = this.URI.indexOf("#");		
		if(index == 0)
			return this.URI;
		else
			return this.URI.substring(index+1, this.URI.length());
		
	}	
	
	public String getParentAreaURI(){
		return this.parentAreaURI;
	}
	
	public void setParentAreaURI(String URI){
		this.parentAreaURI = URI;
	}	
	
	//
	//VPArea methods
	//
	
	public Vector<String> getChildItemURIs(){
		return this.childItemURIs;
	}
	
	public void addChildItemURI(String URI){
		this.childItemURIs.add(URI);
	}
	
	public void removeChildItemURI(String URI){		
		for(int i=0; i<childItemURIs.size(); i++){
			if(childItemURIs.elementAt(i).equals(URI))
				childItemURIs.remove(i);
		}		
	}
	
	public Vector<String> getChildAreaURIs(){
		return this.childAreaURIs;
	}
	
	public void addChildAreaURI(String URI){
		this.childAreaURIs.add(URI);
	}
	
	public void removeChildAreaURI(String URI){		
		for(int i=0; i<childAreaURIs.size(); i++){
			if(childAreaURIs.elementAt(i).equals(URI))
				childAreaURIs.remove(i);
		}		
	}
	
	public void print(){
		
		String areaString = "(URI: "+URI+"\ngridX: "+this.getLocation().x/20+
		"\ngridY: "+this.getLocation().y/20+"\ngridWidth: "+this.getWidth()/20+
		"\ngridHeight: "+this.getHeight()/20+"\nparentArea: "+
		parentAreaURI+"\nchildItems:\n";		
		for(int i=0; i<childItemURIs.size(); i++)
			areaString = areaString + childItemURIs.elementAt(i) + "\n";		
		areaString = areaString + "childAreas\n";		
		for(int i=0; i<childAreaURIs.size(); i++)
			areaString = areaString + childAreaURIs.elementAt(i);		
		areaString = areaString + ")";
		
		System.out.println(areaString);
		
	}
	
}
