package vtt.vp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import vtt.vp.context.source.VPContextSource;

public class VPViewManager extends JPanel implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public static final int TEXTVIEW_TYPE = 0;
	public static final int EDITVIEW_TYPE = 1;
	public static final int ISOMETRICVIEW_TYPE = 2;
	
	private VPView activeView;
	private VPContextSource activeCS;
	private int currentViewType;
	
	public VPViewManager(){
		super();
		initComponents();		
	}    
	
	public void viewContextSource(VPContextSource vpcs){
		
		activeCS = vpcs;
		
		if(vpcs.getType() == VPContextSource.SENSOR_CS_TYPE)
			viewSensorContextSource(vpcs);
		else if(vpcs.getType() == VPContextSource.STATIC_CS_TYPE)
			viewStaticContextSource(vpcs);
		else if(vpcs.getType() == VPContextSource.DYNAMIC_CS_TYPE)
			viewDynamicContextSource(vpcs);
		
	}
	
	public void clearView(){
		
		if(activeView != null){
			viewPanel.remove(activeView);
			viewPanel.repaint();
			closeViewButton.setEnabled(false);
			availableViewsComboBox.setEnabled(false);
			availableViewsComboBox.removeAllItems();
			titleLabel.setText("");
		}
		
		activeCS = null;
		currentViewType = -1;
		
	}
	
	public int getCurrentViewType(){
		return currentViewType;
	}
	
	public VPView getCurrentView(){
		return activeView;
	}
	
	private void viewDynamicContextSource(VPContextSource vpcs) {
		// TODO Auto-generated method stub
		
	}

	private void viewStaticContextSource(VPContextSource vpcs) {
		
		activeCS = vpcs;	
		
		closeViewButton.setEnabled(true);		
		titleLabel.setText(activeCS.getName());		
		availableViewsComboBox.setEnabled(true);
		availableViewsComboBox.removeAllItems();
		availableViewsComboBox.addItem("text view");
		availableViewsComboBox.addItem("edit view");				
		availableViewsComboBox.addItem("isometric view");
				
	}

	private void viewSensorContextSource(VPContextSource vpcs) {
		// TODO Auto-generated method stub
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		
		JComboBox cb = (JComboBox)ae.getSource();
        String viewName = (String)cb.getSelectedItem();
        
        if(viewName == null) return;        
		
        if(viewName.equals("text view") && !(activeView instanceof TextView)){
        	System.out.println(viewName);      
        	if(activeView != null)
    			viewPanel.remove(activeView);
        	activeView = new TextView(activeCS);
        	currentViewType = TEXTVIEW_TYPE;
    		viewPanel.add(activeView, BorderLayout.CENTER);    		
    		this.revalidate();
        }else if(viewName.equals("edit view") && !(activeView instanceof EditView)){
        	System.out.println(viewName);   
        	if(activeView != null)
    			viewPanel.remove(activeView);
        	activeView = new EditView(activeCS);
        	currentViewType = EDITVIEW_TYPE;
    		viewPanel.add(activeView, BorderLayout.CENTER);     		
    		this.revalidate();
        }else if(viewName.equals("isometric view") && !(activeView instanceof IsometricView)){
        	System.out.println(viewName);
        	if(activeView != null)
    			viewPanel.remove(activeView);
        	activeView = new IsometricView(activeCS);
        	currentViewType = ISOMETRICVIEW_TYPE;
    		viewPanel.add(activeView, BorderLayout.CENTER);
    		this.revalidate();
        }        
        
	}	
	
	private void initComponents(){
		
		this.setLayout(new BorderLayout());
		
		titlePanel = new JPanel();
		titlePanel.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
		titlePanel.setPreferredSize(new Dimension(100,32));
		this.add(titlePanel, BorderLayout.NORTH);
		
		titleLabel = new JLabel();		
		titlePanel.add(titleLabel);
		
		closeViewButton = new JButton("X");
		closeViewButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				clearView();				
			}        	
        });
		closeViewButton.setEnabled(false);
		titlePanel.add(closeViewButton);
		
		availableViewsComboBox = new JComboBox();
		availableViewsComboBox.setModel(new DefaultComboBoxModel());
		availableViewsComboBox.setEnabled(false);
		availableViewsComboBox.addActionListener(this);
		titlePanel.add(availableViewsComboBox);
		
		SpringLayout titlePanelSpringLayout = new SpringLayout();
		titlePanelSpringLayout.putConstraint(SpringLayout.NORTH, titleLabel, 4, SpringLayout.NORTH, titlePanel);
		titlePanelSpringLayout.putConstraint(SpringLayout.WEST, titleLabel, 5, SpringLayout.WEST, titlePanel);
		titlePanelSpringLayout.putConstraint(SpringLayout.NORTH, closeViewButton, 1, SpringLayout.NORTH, titlePanel);
		titlePanelSpringLayout.putConstraint(SpringLayout.EAST, closeViewButton, 0, SpringLayout.EAST, titlePanel);
		titlePanelSpringLayout.putConstraint(SpringLayout.NORTH, availableViewsComboBox, 2, SpringLayout.NORTH, titlePanel);
		titlePanelSpringLayout.putConstraint(SpringLayout.EAST, availableViewsComboBox, -30, SpringLayout.WEST, closeViewButton);
		titlePanel.setLayout(titlePanelSpringLayout);
		
		viewPanel = new JPanel(new BorderLayout());
		viewPanel.setBackground(Color.DARK_GRAY);
		this.add(viewPanel, BorderLayout.CENTER);
		
	}	
		
	private JPanel titlePanel;
	private JPanel viewPanel;
	private JButton closeViewButton;
	private JLabel titleLabel;
	private JComboBox availableViewsComboBox;
	
}
